# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.builder import Builder
from winkirip.context import Context
from winkirip.worker_chdir import WorkerChDir
from winkirip.worker_dir_struct import WorkerDirStruct
from winkirip.worker_size import WorkerCalcSize
from winkirip.worker_audio_video_dvd import WorkerAudioVideoDVD
from winkirip.worker_author_dvd import WorkerAuthorDVD
from winkirip.worker_create_iso import WorkerCreateISO
from winkirip.worker_link import WorkerLinkSource
from winkirip.worker_cleanup import WorkerCleanup
from winkirip.container import CONTAINER_DVD
from winkirip.factory_audio import FactoryAudio
from winkirip.factory_audio_compress import FactoryAudioCompress
from winkirip.factory_palette import FactoryPalette
from winkirip.factory_subtitle import FactorySubtitle
from winkirip.factory_video import FactoryVideo
from winkirip.factory_multiplex import FactoryMultiplex

class BuilderDVD(Builder):
    def __init__(self, project):
        Builder.__init__(self, self.__build)

        self.__project = project
        return

    # -- private interface --

    def __build(self):
        simple = True

        # if we have to rip just one audio stream and no subtitles, we can
        # combine the audio and video steps
        containers = self.__project.get_dest()
        for container in containers:
            for title in container:
                audio_list = title.get_audio()
                subtitle_list = title.get_subtitles()
                if len(audio_list) > 1 or len(subtitle_list) > 0:
                    simple = False
                    break

        # we cannot use the simplified procedure with tcrequant
        cfg = self.__project.get_config()
        shrink_method = cfg.get_video_shrink_method_dvd()
        if shrink_method == 'requantize':
            simple = False

        if not simple:
            self.__extended()
        else:
            self.__extended()
            # todo: enable? it's fine with one source container but won't work with more
            # WorkerAudioVideo is not functional
            #self.__simple()

        return True

    def __extended(self):
        context = Context()
        context.set_project(self.__project)

        # preparation steps
        chdir = WorkerChDir(context)
        self.add_worker(chdir)

        dirstruct = WorkerDirStruct(context)
        self.add_worker(dirstruct)

        # aquisition steps
        for source_container in self.__project.get_source():
            source_id = source_container.get_id()

            link = WorkerLinkSource(context, source_id)
            self.add_worker(link)

            audio_factory = FactoryAudio(self.__project)
            audio = audio_factory.create_worker(context, source_id)
            if audio:
                self.add_worker(audio)

            acompress_factory = FactoryAudioCompress(self.__project)
            acompress = acompress_factory.create_worker(context, source_id)
            if acompress:
                self.add_worker(acompress)

            palette_factory = FactoryPalette(self.__project)
            palette = palette_factory.create_worker(context, source_id)
            if palette:
                self.add_worker(palette)

            subtitle_factory = FactorySubtitle(self.__project)
            subtitle = subtitle_factory.create_worker(context, source_id)
            if subtitle:
                self.add_worker(subtitle)

            video_factory = FactoryVideo(self.__project)
            video = video_factory.create_worker(context, source_id)
            # if we have to do 1 or 2 pass encoding, we need to calculate the file sizes first.
            # for that reason we skip the conversion here and
            # video_factory.create_shrinker() will call video_factory.create_worker() later
            if video and video_factory.get_shrink_method() < 1:
                self.add_worker(video)

            # todo: no. ejecting here if we havent acquired the video will be a mistake
            #eject = WorkerEjectMedia(context, source_id)
            #self.add_worker(eject)

        # conversion steps
        sizes = WorkerCalcSize(context)
        self.add_worker(sizes)

        for source_container in self.__project.get_source():
            source_id = source_container.get_id()

            # this creates either the video shrinking or the video acquisition worker
            shrink = video_factory.create_shrinker(context, source_id)
            if shrink:
                self.add_worker(shrink)

        multiplex_factory = FactoryMultiplex(self.__project)
        multiplexers = multiplex_factory.create_workers(context)
        for multiplex in multiplexers:
            self.add_worker(multiplex)

        author = WorkerAuthorDVD(context)
        self.add_worker(author)

        image = WorkerCreateISO(context)
        self.add_worker(image)

        # remove /tmp/source_X, divx2pass.log, audio/, video/ etc.
        cleanup = WorkerCleanup(context)
        self.add_worker(cleanup)
        return True

    def __simple(self):
        # todo: multiple sources
        context = Context()
        context.set_project(self.__project)

        chdir = WorkerChDir(context)
        self.add_worker(chdir)

        dirs = WorkerDirStruct(context)
        self.add_worker(dirs)

        link = WorkerLinkSource(context)
        self.add_worker(link)

        video = WorkerAudioVideoDVD(context)
        self.add_worker(video)

        author = WorkerAuthorDVD(context)
        self.add_worker(author)

        return True


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
