# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import events_pending, main_iteration
from os import kill
from signal import SIGHUP
from commands import getoutput
from popen2 import Popen3
from winkirip.lib import debug, error, info, to_int
from time import sleep

class Command:
    def __init__(self, command, callback = None, finish = None, ui_loop = True):
        self.__command = command
        self.__callback = None
        self.__finish = None
        if callback and callable(callback):
            self.__callback = callback
        if finish and callable(finish):
            self.__finish = finish
        self.__bufsize = 1024
        self.__cmd_obj = None
        self.__pid = -1
        self.__partial_line = ''
        self.__result = -1
        self.__pure_command = ''
        self.__from_stderr = False
        self.__collect_output = True
        #self.__any_output = False
        self.__ui_loop = ui_loop
        return

    # -- public interface --

    def get_status(self):
        status = -1
        if self.__cmd_obj:
            status = self.__cmd_obj.poll()
        else:
            status = self.__result
        return status

    def set_bufsize(self, bufsize):
        self.__bufsize = bufsize
        return

    def read_from_stderr(self):
        self.__from_stderr = True
        return

    def collect_output(self, enable):
        self.__collect_output = enable
        return

    #def use_any_output(self, use = True):
    #   self.__any_output = use
    #   return

    def stop(self):
        self.__kill()

        self.__cmd_obj = None
        return

    def run(self):
        self.__build_pure_name()

        debug('Command.run()', 'Running: ' + self.__command, info)

        self.__cmd_obj = Popen3(self.__command, self.__from_stderr, self.__bufsize)
        self.__pid = self.__cmd_obj.pid

        text = None
        while self.__cmd_obj and self.__cmd_obj.poll() == -1:
            text = self.__read()
            if text:
                self.__process_block(text)

            if self.__ui_loop:
                while events_pending():
                    main_iteration(False)

                if not text:
                    sleep(0.01)       # give up some cpu time if we did nothing
            else:
                # give up more CPU time since we are not responsible for the UI loop
                if not text:
                    sleep(0.1)

        # process the remaining output
        text = ''
        if self.__cmd_obj:
            text = self.__read(True)
            if text is None:
                text = ''
        self.__process_block(text + '\n') # force last line

        debug('Command.run()', 'stopping command')
        self.stop()
        if self.__finish:
            debug('Command.run()', 'finishing command')
            self.__finish()
        return

    def to_stdin(self, cmd):
        if not self.__cmd_obj:
            debug('Command.to_stdin()', 'IN pipe not open', error)
            return

        self.__cmd_obj.tochild.write(cmd)
        self.__cmd_obj.tochild.flush()
        return

    # -- helpers --

    def __read(self, all = False):
        text = None
        if self.__from_stderr:
            if all:
                text = self.__cmd_obj.childerr.read()
            else:
                text = self.__cmd_obj.childerr.read(self.__bufsize)
        else:
            if all:
                text = self.__cmd_obj.fromchild.read()
            else:
                text = self.__cmd_obj.fromchild.read(self.__bufsize)
        return text

    def __build_pure_name(self):
        if not self.__command or not len(self.__command):
            debug('Command.__build_pure_name()', 'no self.command', error)
            return

        self.__pure_command = self.__command.replace('"', '')
        return  

    def __process_block(self, text):
        "Splits the block into lines and calls the callback if needed"
        if not self.__collect_output or not self.__callback:
            return

        start = 0
        end = len(text)
        while start < end:
            # Find earliest of \n or \r. Generally \r will occur early in mplayer.
            eol_pos = text.find('\r', start)
            if eol_pos < 0:
                eol_pos = text.find('\n', start)
            else:
                lf_pos = text.find('\n', start, eol_pos) # earlier?
                if lf_pos > -1:
                    eol_pos = lf_pos

            if eol_pos < 0:
                self.__partial_line += text[start : ]
                return
            
            if self.__partial_line:
                self.__callback(self.__partial_line + text[start : eol_pos])
                self.__partial_line = ''
            elif start < eol_pos:  # avoid empty line callbacks on \r\n
                self.__callback(text[start : eol_pos])
                                
            start = eol_pos + 1
        return

    def __kill(self):
        if self.__cmd_obj:
            self.__result = self.__cmd_obj.poll()

        if self.__pid > -1:
            try:
                kill(self.__pid, SIGHUP)
                #print 'killing: %d' % self.__pid
            except:
                pass

        # crude but effective...
        pid = self.__find_pid()
        while pid != -1:
            try:
                #print 'more killing: %d' % pid
                kill(pid, SIGHUP)
            except:
                debug('Command.__kill()', 'error killing proccess', error)
            pid = self.__find_pid()

        if self.__cmd_obj:
            self.__cmd_obj.tochild.close()
            self.__cmd_obj.fromchild.close()
        return

    def __find_pid(self):
        output = getoutput('ps -Af')
        lines = output.split('\n')

        pure_commands = self.__pure_command.split('|')
        for i in range(len(pure_commands)):
            parts = pure_commands[i].replace('LC_ALL=C', '').strip().split()
            pure_commands[i] = ''.join(parts)

        for line in lines:
            parts = line.split()
            fixed_line = ''.join(parts)

            cmd_found = False
            for pcommand in pure_commands:
                pos = fixed_line.find(pcommand)
                if pos > -1:
                    cmd_found = True
                    break

            if cmd_found:
                pid = to_int(parts[1])
                if pid != -1:
                    return pid

        return -1

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
