# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from threading import Thread
from gtk import events_pending, main_iteration
from winkirip.command import Command
from winkirip.lib import debug, error
from time import sleep


STATE_UNKNOWN = -1
STATE_RUNNING = 0
STATE_FINISHED = 1
STATE_CANCELED = 2

class UICommand:
    def __init__(self):
        self.__command = None
        self.__command_data = None
        self.__shell_cmd = None
        self.__thread = None
        self.__state = STATE_UNKNOWN
        self.__progress_callback = None
        self.__estimation_callback = None
        self.__parse_callback = None
        self.__exec_log_callback = None
        self.__logfile = None
        self.__can_progress = False
        self.__can_estimate = False
        self.__update_ui = True
        self.__error_text = ''
        return

    # -- public interface --

    def set_command(self, command):
        if command and callable(command):
            self.__command = command
        return

    def execute(self):
        if not self.__command:
            return False

        result = self.__command(self.__command_data, self)
        self.__do_log()
        return result

    def run_command(self, cmd_line, success_codes = [0], from_stderr = False):
        self.__do_log()
        self.__shell_cmd = Command(cmd_line, self.__parser, self.__finish,
                                   ui_loop = False)
        if from_stderr:
            self.__shell_cmd.read_from_stderr()

        self.__thread = Thread(group = None, target = self.__shell_cmd.run)
        self.__thread.start()

        while self.__thread.isAlive():
            if self.__state == STATE_CANCELED:
                break
            while events_pending():
                main_iteration(False)
            sleep(0.01)          # give up some CPU time

        self.__thread = None

        status = self.__shell_cmd.get_status()
        success = False
        for code in success_codes:
            if code == status:
                success = True
                break

        # restore state else the command will just fail if called again
        cmd_state = self.__state
        self.__state = STATE_UNKNOWN

        self.__do_log()
        return (success and cmd_state == STATE_FINISHED)

    def cancel(self):
        if self.__thread:
            if self.__shell_cmd:
                self.__state = STATE_CANCELED
                self.__shell_cmd.stop()
            else:
                print 'command not running'
        else:
            print 'command not running'
        return

    def set_log_file(self, filename):
        self.__logfile = filename
        return

    def set_data(self, data):
        self.__command_data = data
        return

    def set_error(self, error_text):
        self.__error_text = error_text
        return

    def get_error(self):
        return self.__error_text

    def set_can_progress(self, ability = True):
        self.__can_progress = ability
        return

    def set_can_estimate(self, ability = True):
        self.__can_estimate = ability
        return

    def set_update_ui(self, action = True):
        self.__update_ui = action

        # todo: hmm - decide if it pays out to enable this
        #if self.__shell_cmd:
        #    self.__shell_cmd.collect_output(action)
        return

    def get_can_progress(self):
        return self.__can_progress

    def get_can_estimate(self):
        return self.__can_estimate

    def set_parse_callback(self, parse_callback):
        if parse_callback and callable(parse_callback):
            self.__parse_callback = parse_callback
        else:
            self.__parse_callback = None
        return

    def set_progress_callback(self, progress_callback):
        if progress_callback and callable(progress_callback):
            self.__progress_callback = progress_callback
        else:
            self.__progress_callback = None
        return

    def set_estimation_callback(self, estimation_callback):
        if estimation_callback and callable(estimation_callback):
            self.__estimation_callback = estimation_callback
        else:
            self.__estimation_callback = None
        return

    def set_exec_log_callback(self, log_callback):
        if log_callback and callable(log_callback):
            self.__exec_log_callback = log_callback
        else:
            self.__exec_log_callback = None
        return

    # -- helpers --

    def __parser(self, text):
        progress = 0.0
        estimation = None

        if self.__state == STATE_CANCELED:
            self.__shell_cmd.stop()
            return

        if self.__update_ui:

            # let the worker parse the command output (it knows how)
            if self.__parse_callback:
                progress, estimation = self.__parse_callback(text)

            # and update the ui
            if self.__can_progress and  self.__progress_callback:
                self.__progress_callback(progress)

            if self.__can_estimate and self.__estimation_callback:
                self.__estimation_callback(estimation)
        return

    def __finish(self):
        # update the ui
        if self.get_can_estimate() and self.__progress_callback:
            self.__progress_callback(1.0)

        if self.__state == STATE_CANCELED:
            # notify that we are ready?
            return

        self.__state = STATE_FINISHED

        # notify that we are ready?
        return

    def __do_log(self):
        if not self.__logfile:
            debug('CommandUI.__do_log()', 'no log file name', error)
            return

        if not self.__exec_log_callback or not callable(self.__exec_log_callback):
            debug('CommandUI.__do_log()', 'no log callback', error)
            return

        fobj = open(self.__logfile, 'a')
        self.__exec_log_callback(fobj)
        fobj.close()
        return

    def __repr__(self):
        return '%s' % self.__command


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
