# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from container import CONTAINER_UNKNOWN, CONTAINER_DVD, CONTAINER_VCD
from container import CONTAINER_SVCD, CONTAINER_FILE, CONTAINER_STREAMS
from winkirip.constructor import Constructor
from winkirip.lib import debug, error, does_path_exist

class ConstructorContainer(Constructor):
    def __init__(self, tools):
        Constructor.__init__(self)
        self.__tools = tools
        self.__video = []
        self.__audio = []
        self.__subtitles = []
        self.__outname = None
        return

    # -- public interface --

    def add_video(self, vfile):
        self.__video.append(vfile)
        return

    def add_audio(self, afile):
        self.__audio.append(afile)
        return

    def add_subtitle(self, sfile):
        self.__subtitles.append(sfile)
        return

    def set_outname(self, outname):
        self.__outname = outname
        return

    def get_tools(self):
        return self.__tools

    def get_video(self):
        return self.__video

    def get_audio(self):
        return self.__audio

    def get_subtitles(self):
        return self.__subtitles

    def get_subtitle(self, index):
        return self.__subtitles[index]

    def get_outname(self):
        return self.__outname


class ConstructorAvi(ConstructorContainer):
    def __init__(self, tools):
        ConstructorContainer.__init__(self, tools)
        return

    # -- methods inherited from Constructor --

    def construct(self):
        program = None
        if self.get_tools().has_tool('avimerge'):
            program = self.get_tools().get_tool_location('avimerge')

        if not program or len(self.get_video()) < 1 or len(self.get_audio()) < 1 or not self.get_outname():
            return False, ''

        video = ' -i %s' % (self.get_video()[0])

        aid = 0
        audio = ''
        for afile in self.get_audio():
            # avimerge supports up to 2 audio streams

            # this restriction is not implemented in the UI because the user
            # may switch the output file format any time after selecting the
            # audio languages
            if aid > 1:
                break

            if aid == 0:
                audio += (' -p %s -A %d' % (afile, aid))
            else:
                audio += (' %s -A %d' % (afile, aid))
            aid += 1

        result = '%s -o %s %s %s' % (program, self.get_outname(), video, audio)
        return True, result


class ConstructorMp4(ConstructorContainer):
    def __init__(self, tools, lang_conv, disc_nr):
        ConstructorContainer.__init__(self, tools)
        self.__lang_conv = lang_conv
        self.__disc_nr = disc_nr
        self.__chapter_file = None
        return

    # -- public interface --

    def set_chapter_file(self, chapterfile):
        self.__chapter_file = chapterfile
        return


    # -- methods inherited from Constructor --

    def construct(self):
        program = None
        if self.get_tools().has_tool('MP4Box'):
            program = self.get_tools().get_tool_location('MP4Box')

        if not program or len(self.get_video()) < 1 or not self.get_outname():
            return False, ''

        if self.get_video()[0].get_fps() < 0.:
            return False, ''

        fps = ' -fps %f' % self.get_video()[0].get_fps()

        video = (' -add %s#video' % (self.get_video()[0].get_filename()))

        audio = ''
        # mp4box assigns 1-based file IDs (the first one goes to the video in this case)
        stream_id = 2
        for astream in self.get_audio():
            lang = self.__lang_conv.find_iso_639_1(astream.get_language())
            alang = lang.get_iso_639_2()
            audio += (' -add %s#audio -lang %d=%s' % (astream.get_filename(), stream_id, alang))
            stream_id += 1

        subtitle = ''
        for sstream in self.get_subtitles():
            lang = self.__lang_conv.find_iso_639_1(sstream.get_language())
            slang = lang.get_iso_639_2()
            subtitle += (' -add %s -lang %d=%s' % (sstream.get_filename(), stream_id, slang))
            stream_id += 1

        chapters = ''
        if self.__chapter_file:
            chapters = ' -chap %s' % self.__chapter_file

        # todo: experiment with -3gp -nosys -mpeg4 -nodrop ...

        result = '%s %s %s %s %s %s %s' % (program, fps, video, audio, subtitle, chapters, self.get_outname())
        return True, result


class Constructor3GPMobile(ConstructorContainer):
    def __init__(self, tools, lang_conv, disc_nr):
        ConstructorContainer.__init__(self, tools)
        self.__lang_conv = lang_conv
        self.__disc_nr = disc_nr
        return

    # -- methods inherited from Constructor --

    def construct(self):
        program = None
        if self.get_tools().has_tool('ffmpeg'):
            program = self.get_tools().get_tool_location('ffmpeg')

        if not program or len(self.get_video()) < 1 or not self.get_outname():
            return False, ''

        istreams = ''
        str_id = 0
        if len(self.get_video()) > 0:
            video = self.get_video()[0]
            istreams += ' -i %s -map %d.0' % (video.get_filename(), str_id)
            str_id += 1

        if len(self.get_audio()) > 0:
            audio = self.get_audio()[0]
            istreams += ' -i %s -map %d.0' % (audio.get_filename(), str_id)
            str_id += 1

        # ffmpeg -i movie.avi -i audiodump.wav -b 48 -ac 1 -ab 12 -map 0.0 -map 1.0 undo.3gp
        opts = '-vcodec h263 -s qcif -aspect 4:3 -acodec amr_nb -b 48 -ac 1 -ab 12'

        result = '%s %s %s %s' % (program, istreams, opts, self.get_outname())
        return True, result


class ConstructorMkv(ConstructorContainer):
    def __init__(self, tools, lang_conv, disc_nr, title_nr):
        ConstructorContainer.__init__(self, tools)
        self.__lang_conv = lang_conv
        self.__disc_nr = disc_nr
        self.__title_nr = title_nr
        self.__fps = -1.
        self.__tag_file = None
        self.__chapter_file = None
        self.__title = None
        return

    # -- public interface --

    def set_fps(self, fps):
        self.__fps = fps
        return

    def set_tag_file(self, tagfile):
        self.__tag_file = tagfile
        return

    def set_chapter_file(self, chapterfile):
        self.__chapter_file = chapterfile
        return

    def set_title(self, title):
        self.__title = title
        return

    # -- methods inherited from Constructor --

    def construct(self):
        if len(self.get_video()) < 1 or not self.get_outname():
            debug('ConstructorMkv.construct()', 'no video stream or output name', error)
            return False, ''

        if not self.__lang_conv or self.__title_nr < 0:
            debug('ConstructorMkv.construct()', 'no lang convertor or title number', error)
            return False, ''

        program = None
        if self.get_tools().has_tool('mkvmerge'):
            program = self.get_tools().get_tool_location('mkvmerge')

        if not program or self.__fps < 0.:
            debug('ConstructorMkv.construct()', 'no program or fps info', error)
            return False, ''

        vstream = self.get_video()[0]

        aspect = ''
        if vstream.get_float_aspect() < 0.:
            debug('ConstructorMkv.construct()', 'aspect not set', error)
        else:
            aspect = ' --aspect-ratio %d:%f' % (self.__title_nr, vstream.get_float_aspect())

        # video stream
        file_id = 0
        video = ' -S -A %s' % (vstream.get_filename())
        vstream.set_id(file_id)
        file_id += 1

        # audio streams
        apos = -1
        audio = ''
        for astream in self.get_audio():
            lang = self.__lang_conv.find_iso_639_1(astream.get_language())
            alang = lang.get_iso_639_2()
            afile = astream.get_filename()
            astream.set_id(file_id)
            file_id += 1
            apos += 1

            audio += ' --language %d:%s -D -S %s' % (self.__title_nr, alang, afile)

        # subtitles
        subtitle = self.__create_subtitle_section(file_id)

        tags = ''
        if self.__tag_file:
            tags = ' --global-tags %s' % self.__tag_file

        chapters = ''
        if self.__chapter_file:
            chapters = ' --chapters %s' % self.__chapter_file

        title = ''
        if self.__title:
            title = ' --title "%s"' % self.__title

        result = '%s -o %s %s %s %s %s %s %s %s' % (program, self.get_outname(), aspect, video, audio, subtitle, tags, chapters, title)
        return True, result

    # -- helpers --

    def __create_subtitle_section(self, file_id):
        spos = -1
        subtitle = ''
        common_name = ''
        for i in range(len(self.get_subtitles())):
            sstream = self.get_subtitle(i)
            if i == 0:
                common_name = sstream.get_filename()

            lang = self.__lang_conv.find_iso_639_1(sstream.get_language())
            slang = lang.get_iso_639_2()
            sfile = sstream.get_filename()
            sstream.set_id(file_id)
            file_id += 1
            spos += 1

            if common_name != sfile or i == (len(self.get_subtitles()) - 1):
                subtitle += ' --language %d:%s -D -A %s' % (spos, slang, common_name)
                spos = -1
                common_name = sstream.get_filename()
            else:
                subtitle += ' --language %d:%s' % (spos, slang)

        return subtitle


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
