# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.lib import sec2str, debug
from winkirip.container_video import Video
from winkirip.container_audio import Audio
from winkirip.container_subtitle import Subtitle
from winkirip.container_chapter import Chapter
from copy import deepcopy


class Title:
    def __init__(self, src = None):
        self.__name = None
        self.__id = -1
        self.__container_id = -1
        self.__video = []
        self.__audio = []
        self.__subtitles = []
        self.__chapters = []
        self.__crop = []
        self.__palette = None
        self.__palette_file = None
        self.__chapter_file = None

        if src:
            self.__name = deepcopy(src.__name)
            self.__id = src.__id
            self.__container_id = src.__container_id
            self.__link_video(src.__video)
            self.__link_audio(src.__audio)
            self.__link_subtitles(src.__subtitles)
            self.__link_chapters(src.__chapters)
            self.__crop.extend(deepcopy(src.__crop))
            self.__palette = deepcopy(src.__palette)
            self.__palette_file = deepcopy(src.__palette_file)
        return

    # -- get/set --

    def get_name(self):
        if self.__name:
            return self.__name

        title_len = self.get_length()
        fmt_time = sec2str(title_len)
        generated = _('Title %(title_index)d (%(time_string)s)') % {'title_index':self.__id, 'time_string':fmt_time}
        return generated

    def get_id(self):
        return self.__id

    def get_container_id(self):
        return self.__container_id

    def get_video(self):
        return self.__video

    def get_audio(self):
        return self.__audio

    def get_subtitles(self):
        return self.__subtitles

    def get_chapters(self):
        return self.__chapters

    def get_crop(self):
        return self.__crop 

    def get_palette(self):
        return self.__palette

    def get_palette_file(self):
        return self.__palette_file

    def get_chapter_file(self):
        return self.__chapter_file

    def get_video_at(self, index):
        return self.__video[index]

    def get_audio_at(self, index):
        return self.__audio[index]

    def get_subtitle_at(self, index):
        return self.__subtitles[index]

    def get_audio_pos(self, aid):
        for i in range(len(self.__audio)):
            if self.__audio[i].get_id() == aid:
                return i
        return -1

    def get_subtitle_pos(self, sid):
        for i in range(len(self.__subtitles)):
            if self.__subtitles[i].get_id() == sid:
                return i
        return -1

    def get_chapter_at(self, index):
        return self.__chapters[index]

    def get_length(self):
        chapter_len = 0
        for chapter in self.__chapters:
            chapter_len += chapter.get_length()
        return chapter_len

    def set_name(self, name):
        self.__name = name
        return

    def set_id(self, id):
        if not isinstance(id, int):
            assert(False)

        self.__id = id

        for vid in self.__video:
            vid.set_title_id(self.__id)

        for aud in self.__audio:
            aud.set_title_id(self.__id)

        for subt in self.__subtitles:
            subt.set_title_id(self.__id)

        for chap in self.__chapters:
            chap.set_title_id(self.__id)
        return

    def set_container_id(self, id):
        if not isinstance(id, int):
            assert(False)

        self.__container_id = id

        for vid in self.__video:
            vid.set_container_id(self.__container_id)

        for aud in self.__audio:
            aud.set_container_id(self.__container_id)

        for subt in self.__subtitles:
            subt.set_container_id(self.__container_id)

        for chap in self.__chapters:
            chap.set_container_id(self.__container_id)
        return

    def get_first_chapter_id(self):
        if not len(self.__chapters):
            return -1

        return self.__chapters[0].get_id()

    def get_last_chapter_id(self):
        if not len(self.__chapters):
            return -1

        last = len(self.__chapters) - 1
        return self.__chapters[last].get_id()

    # -- public interface --

    def add_video(self, video):
        if not isinstance(video, Video):
            debug('container_title.add_video()', 'video of wrong type passed')
            return False

        video.set_title_id(self.__id)
        video.set_container_id(self.__container_id)
        self.__video.append(video)
        return True

    def add_audio(self, audio):
        if not isinstance(audio, Audio):
            debug('container_title.add_audio()', 'audio of wrong type passed')
            return False

        audio.set_title_id(self.__id)
        audio.set_container_id(self.__container_id)
        self.__audio.append(audio)
        return True

    def add_subtitle(self, subtitle):
        if not isinstance(subtitle, Subtitle):
            debug('container_title.add_subtitle()', 'subtitle of wrong type passed')
            return False

        subtitle.set_title_id(self.__id)
        subtitle.set_container_id(self.__container_id)
        self.__subtitles.append(subtitle)
        return True

    def add_chapter(self, chapter):
        if not isinstance(chapter, Chapter):
            debug('container_title.add_chapter()', 'chapter of wrong type passed')
            return False

        chapter.set_title_id(self.__id)
        chapter.set_container_id(self.__container_id)
        self.__chapters.append(chapter)
        return True

    def set_crop(self, crop_values):
        if not crop_values or len(crop_values) != 4:
            debug('container_title.set_crop()', 'invalid crop values list passed')
            return False

        self.__crop = []
        self.__crop.extend(crop_values)
        return True

    def set_palette(self, palette):
        assert(len(palette) == 16)
        self.__palette = palette
        return

    def set_palette_file(self, pfile):
        self.__palette_file = pfile
        return

    def set_chapter_file(self, chapter_file):
        self.__chapter_file = chapter_file
        return

    def reset_video(self):
        self.__video = []
        return

    def reset_audio(self):
        self.__audio = []
        return

    def reset_subtitles(self):
        self.__subtitles = []
        return

    def reset_chapters(self):
        self.__chapters = []
        return

    def reset_crop(self):
        self.__crop = []
        return

    # -- helpers --

    def __link_video(self, video_list):
        result = deepcopy(video_list)
        for index in range(len(result)):
            result[index].set_source(video_list[index])
            #result[index].set_id(index + 1)

        self.__video = result
        return

    def __link_audio(self, audio_list):
        result = deepcopy(audio_list)
        for index in range(len(result)):
            result[index].set_source(audio_list[index])
            #result[index].set_id(index + 1)

        self.__audio = result
        return

    def __link_subtitles(self, subtitle_list):
        result = deepcopy(subtitle_list)
        for index in range(len(result)):
            result[index].set_source(subtitle_list[index])
            #result[index].set_id(index + 1)

        self.__subtitles = result
        return

    def __link_chapters(self, chapter_list):
        result = deepcopy(chapter_list)
        for index in range(len(result)):
            result[index].set_source(chapter_list[index])
            result[index].set_id(index + 1)

        self.__chapters = result
        return

    def __repr__(self):
        result = '  Title(%s, id = %d, cid = %d, crop = %s, pal = %s)\n' % (self.__name, self.__id, self.__container_id, self.__crop, self.__palette_file)
        for video in self.__video:
            result += '%s' % video

        for audio in self.__audio:
            result += '%s' % audio

        for subt in self.__subtitles:
            result += '%s' % subt

        for chapter in self.__chapters:
            result += '%s' % chapter

        return result

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
