# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from container_stream import Stream, TYPE_VIDEO
from winkirip.lib import sec2str, to_float
from copy import deepcopy

VIDEO_FMT_UNKNOWN = -1
VIDEO_FMT_PAL = 0
VIDEO_FMT_NTSC = 1

VIDEO_UNKNOWN = -1
VIDEO_MPEG1 = 0
VIDEO_MPEG2 = 1
VIDEO_MPEG4 = 2
VIDEO_XVID = 3
VIDEO_THEORA = 4
VIDEO_H264 = 5
VIDEO_REAL = 6
VIDEO_DIVX = 7
VIDEO_WMV = 8
VIDEO_FLASH = 9
VIDEO_QTIME = 10
VIDEO_MOBILE = 11  # mpeg-4 actually

# -- implementation --

class Video(Stream):
    def __init__(self, src = None):
        self.__format = VIDEO_FMT_UNKNOWN
        self.__type = VIDEO_UNKNOWN
        self.__bitrate = -1
        self.__width = 0
        self.__height = 0
        self.__fps = 0.0
        self.__aspect = None
        self.__length = 0
        self.__filename = None
        self.__id = -1
        self.__title_id = -1
        self.__container_id = -1
        self.__descr = None
        self.__quant_factor = 0.0
        self.__shrink_needed = True
        self.__source = None

        if src:
            self.__format = src.__format
            self.__type = src.__type
            self.__bitrate = src.__bitrate
            self.__width = src.__width
            self.__height = src.__height
            self.__fps = src.__fps
            self.__aspect = deepcopy(src.__aspect)
            self.__length = src.__length
            self.__filename = deepcopy(src.__filename)
            self.__id = src.__id
            self.__title_id = src.__title_id
            self.__container_id = src.__container_id
            self.__descr = deepcopy(src.__descr)
            self.__quant_factor = src.__quant_factor 
            self.__shrink_needed = src.__shrink_needed
        return

    # -- methods inherited from Stream --

    def stream_type(self):
        return TYPE_VIDEO

    # -- get/set --

    def get_format(self):
        return self.__format

    def get_format_str(self):
        if self.__format == VIDEO_FMT_NTSC:
            return 'ntsc'

        return 'pal'

    def get_type(self):
        return self.__type

    def get_bitrate(self):
        return self.__bitrate

    def get_width(self):
        return self.__width

    def get_height(self):
        return self.__height

    def get_fps(self):
        return self.__fps

    def get_aspect(self, hint = None):
        if not self.__aspect:
            return ''

        if hint and hint == 'dvdauthor':
            if isinstance(self.__aspect, str):
                if self.__aspect.find('/') > -1:
                    return self.__aspect.replace('/', ':')

                elif to_float(self.__aspect) > -1:
                    return self.__find_standard_aspect_string(to_float(self.__aspect)).replace('/', ':')

                else:
                    # brrr... let's hope the best for dvdauthor
                    return self.__aspect

            elif isinstance(self.__aspect, float):
                return self.__find_standard_aspect_string(self.__aspect).replace('/', ':')

            else:
                # unexpected aspect type
                assert(False)

        if isinstance(self.__aspect, float):
            return self.__find_standard_aspect_string(self.__aspect)

        elif isinstance(self.__aspect, int):
            return self.__find_standard_aspect_string(float(self.__aspect))

        return self.__aspect

    def get_float_aspect(self):
        if not self.__aspect:
            return -1.

        if isinstance(self.__aspect, str):
            items = self.__aspect.split('/')
            if len(items) == 2 and to_float(items[0]) > -1 and to_float(items[1]) > -1:
                return to_float(items[0]) / to_float(items[1])
            else:
                return to_float(self.__aspect)

        elif isinstance(self.__aspect, float):
            return self.__aspect

        elif isinstance(self.__aspect, int):
            return float(self.__aspect)

        # oops - aspect has and unexpected type
        return -1.

    def get_length(self):
        return self.__length

    def get_filename(self):
        return self.__filename

    def get_id(self):
        return self.__id

    def get_title_id(self):
        return self.__title_id

    def get_container_id(self):
        return self.__container_id

    def get_descr(self):
        return self.__descr

    def get_quant_factor(self):
        return self.__quant_factor

    def get_shrink_needed(self):
        return self.__shrink_needed

    def get_source(self):
        return self.__source

    def set_format(self, format):
        self.__format = format
        return

    def set_type(self, type):
        self.__type = type
        return

    def set_bitrate(self, bitrate):
        self.__bitrate = bitrate
        return

    def set_width(self, width):
        self.__width = width
        return

    def set_height(self, height):
        self.__height = height
        return

    def set_fps(self, fps):
        self.__fps = float(fps)
        return

    def set_aspect(self, aspect):
        self.__aspect = aspect
        return

    def set_length(self, length):
        self.__length = length
        return

    def set_filename(self, filename):
        self.__filename = filename
        return

    def set_id(self, id):
        self.__id = id
        return

    def set_title_id(self, title_id):
        self.__title_id = title_id
        return

    def set_container_id(self, container_id):
        self.__container_id = container_id
        return

    def set_descr(self, descr):
        self.__descr = descr
        return

    def set_quant_factor(self, factor):
        self.__quant_factor = factor
        return

    def set_shrink_needed(self, do_shrink):
        self.__shrink_needed = do_shrink
        return

    def set_source(self, source):
        if isinstance(source, Video):
            self.__source = source
        else:
            assert(False)
        return

    # -- public interface --

    def get_formatted_length(self):
        return sec2str(self.__length)

    # -- helpers --

    def __find_standard_aspect_string(self, faspect):
        assert(isinstance(faspect, float))

        if abs(faspect - (16.0 / 9.0)) < 1e-3:
            return '16/9'

        elif abs(faspect - (4.0 / 3.0)) < 1e-3:
            return '4/3'

        return '%.2f/1' % faspect


    def __repr__(self):
        result = '    Video(%s, %d)\n' % (self.__filename, self.__id)
        result += '      format: %d, type: %d, bitrate: %d, width: %d, height: %d, fps: %f, aspect: %s, len: %d, descr: %s, requant: %f, shrink: %s [CID=%d/TID=%d]\n' % (self.__format, self.__type, self.__bitrate, self.__width, self.__height, self.__fps, self.__aspect, self.__length, self.__descr, self.__quant_factor, self.__shrink_needed, self.__container_id, self.__title_id)
        result += '     SOURCE: %s\n' % self.__source
        return result

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
