# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from string import letters, digits, whitespace, maketrans
from os.path import join, split, splitext, expanduser, normpath, exists, isfile, isdir
import os
from gtk import ListStore, CellRendererText
from gobject import TYPE_STRING, TYPE_PYOBJECT
from pango import FontDescription


# program installation path (winki_path is set here _and_ in ../setup.py)
winki_path = join('/', 'usr', 'share',  'winkirip')

# enumerations for the priority of the debug messages
info = 1
error = 2
critical = 4

# variable which controls the debug output
#show = info | error | critical
show = error

def debug(module, message, priority = info):
    if show & info and priority & info:
        print 'info: %s - %s' % (module, message)

    if show & error and priority & error:
        print 'error: %s - %s' % (module, message)

    if show & critical and priority & critical:
        print 'critical: %s - %s' % (module, message)
    return

# -- conversion --

def to_int(str):
    try:
        return int(str)
    except:
        return -1

def to_float(str):
    try:
        return float(str)
    except:
        return -1

def hex2dec(hex_str):
    return int(hex_str, 16)

# -- string manipulation --

def lower_strip(text):
    return text.strip().lower()
    
def str2sec(line):
    hh, mm, ss = line.split(':')

    result = float(ss)
    result += (int(mm) * 60)
    result += (int(hh) * 3600)

    return result

def sec2str(sec, show_ms = True):
    hour = int(sec) / (3600)
    sec -= (hour * 3600)

    mmin = int(sec) / 60
    sec -= (mmin * 60)

    hh = '%.2d' % hour
    mm = '%.2d' % mmin
    ss = '%.2d' % sec

    if show_ms:
        msec = sec - int(sec)
        ms = '%.3d' % (msec * 1000)
        result = '%s:%s:%s.%s' % (hh, mm, ss, ms)
    else:
        result = '%s:%s:%s' % (hh, mm, ss)

    return result

# -- file names / file operations --

def is_allowed_in_filename(ch):
    # i don't know if this is worth since string.letters etc. are modifiyable...
    if letters.find(ch) > -1:
        return True
    elif digits.find(ch) > -1:
        return True
    elif whitespace.find(ch) > -1:
        return True
    elif '/'.find(ch) > -1:
        return True

    return False

def fix_name(name):
    # see ord(): file:///usr/share/doc/python2.1/html/lib/built-in-funcs.html
    if not name or not len(name):
        return ''

    name = filter(is_allowed_in_filename, name)
    name = name.replace(' ', '_')
    name = name.replace('\t', '_')

    return name

def fix_fname(name):
    # a file name should not contain directory separators
    name = fix_name(name)
    name = name.replace('/', '_')
    return name

def escape_name(name):
    result = name.replace('"', '\\"')
    result = result.replace('|', '\\|')
    result = result.replace('$', '\\$')
    result = result.replace('!', '\\!')
    result = result.replace(':', '\\:')
    #result = result.replace('(', '\\(')
    #result = result.replace(')', '\\)')
    #result = result.replace(' ', '\\ ')
    return result

def quote_path(path):
    """Quotes a path suitably for passing as a command line argument. Puts
    single quotes around the path, replaces any inner single quotes with
    escape sequences, and deals with filenames starting with "-"."""

    if not path:
        return ''

    if path[0] == '-':
        path = './' + path

    return "'%s'" % path.replace("'", "'\\''")

def is_quoted(name):
    if not name or not len(name):
        return False

    return name[0] == '"' and name[len(name) - 1] == '"'

def remove_quotes(name):
    if not name or not len(name):
        return name

    return name[1 : len(name) - 1]

def set_quotes(name, use_flag = True):
    if not name or not len(name) or not use_flag:
        return name

    if is_quoted(name):
        return name

    return '"%s"' % name

def split_path(full_name):
    #print 'split_path(%s)' % full_name
    path = None
    fname = None
    ext = None
    if not full_name or not len(full_name):
        return [path, fname, ext]

    quoted = is_quoted(full_name)
    if quoted:
        full_name = remove_quotes(full_name)

    if full_name == '.' or full_name == '..' or full_name == '~':
        path = set_quotes(full_name, quoted)
        return [path, fname, ext]

    path, name = split(full_name)
    if not name or not len(name):
        return [set_quotes(path, quoted), fname, ext]

    fname, ext = splitext(name)
    return [set_quotes(path, quoted), set_quotes(fname, quoted), set_quotes(ext, quoted)]

def normalize_path(path):
    if not path or not len(path):
        return path

    quoted = is_quoted(path)
    if quoted:
        path = remove_quotes(path)

    if path.find('~') > -1:
        path = expanduser(path)
    
    #for ch in path:
    #   if is_allowed_in_filename(ch):
    #       pass
    #   else:
    #       debug('lib.normalize_path()', 'please don\'t use %c in file names' % ch, error)
    #       return None

    if path.find('~') > -1:
        msg_info = 'mailformed path %s' % path
        debug('lib.normalize_path()', msg_info, error)
        return None

    path =  normpath(path)
    return set_quotes(path, quoted)
        
def build_path(items):
    path = ''
    if not items or not len(items):
        return None

    quoted = False
    for item in items:
        quoted |= is_quoted(item)

    for item in items:
        if not item or not len(item):
            continue

        if is_quoted(item):
            item = remove_quotes(item)

        path = join(path, item)

    return normalize_path(set_quotes(path, quoted))

def set_extension(fullpath, extension):
    if not extension or not len(extension):
        return fullpath

    quoted = is_quoted(fullpath)
    quoted |= is_quoted(extension)

    if is_quoted(quoted):
        fullpath = remove_quotes(fullpath)

    if is_quoted(extension):
        extension = remove_quotes(extension)
        
    if extension[0] == '.':
        extension = extension[1: ]

    fname, ext = splitext(fullpath)
    result = '%s.%s' % (fname, extension)

    return set_quotes(result, quoted)

def append_to_filename(fullpath, appendix):
    if not appendix or not len(appendix):
        return fullpath

    quoted = is_quoted(fullpath)
    quoted |= is_quoted(appendix)

    if is_quoted(fullpath):
        fullpath = remove_quotes(fullpath)

    if is_quoted(appendix):
        appendix = remove_quotes(appendix)
        
    path, name, ext = split_path(fullpath)
    if name:
        name = '%s%s' % (name, appendix)

    fname = set_extension(name, ext)
    return build_path([set_quotes(path, quoted), set_quotes(fname, quoted)])

def does_path_exist(path):
    if not path or not len(path):
        return False

    if is_quoted(path):
        path = remove_quotes(path)

    return exists(path)

def number_lines(path):
    if not does_path_exist(path):
        return 0

    try:
        f = open(path, 'r')
        raw = f.read()
        f.close()
    except:
        return 0

    return len(raw.split('\n'))

def is_file(path):
    if not does_path_exist(path):
        return False

    if is_quoted(path):
        path = remove_quotes(path)

    if isfile(path) == 0:
        return False;
        
    return True

def is_dir(path):
    if not does_path_exist(path):
        return False

    if is_quoted(path):
        path = remove_quotes(path)

    if isdir(path) == 0:
        return False;
        
    return True

def is_readable(path):
    if not does_path_exist(path):
        return False

    if is_quoted(path):
        path = remove_quotes(path)

    if os.access(path, os.R_OK)  == 0:
        return False

    return True

def is_writable(path):
    if not does_path_exist(path):
        return False

    if is_quoted(path):
        path = remove_quotes(path)

    if os.access(path, os.W_OK)  == 0:
        return False

    return True

def is_executable(path):
    if not does_path_exist(path):
        return False

    if is_quoted(path):
        path = remove_quotes(path)

    if os.access(path, os.X_OK)  == 0:
        return False

    return True

def is_dir_empty(path):
    if not does_path_exist(path):
        return False

    if is_quoted(path):
        path = remove_quotes(path)

    if not is_dir(path):
        return False

    if not is_readable(path):
        return False

    file_list = os.listdir(path)
    return (len(file_list) == 0)


def file_size(fname):
    if not fname or not len(fname):
        return -1

    if is_quoted(fname):
        fname = remove_quotes(fname)

    fsize = -1
    try:
        f = os.open(fname, os.O_RDONLY)
        fsize = os.lseek(f, 0, 2)
        os.close(f)
    except:
        pass

    return fsize

def mkdir(path):
    if is_quoted(path):
        path = remove_quotes(path)
    
    path = normalize_path(path)
    if does_path_exist(path):
        if is_dir(path):
            return True
        else:
            return False

    try:
        os.makedirs(path)
    except:
        debug('lib.mkdir', 'cannot create %s' % path, error)
        return False

    return True

def move(src, dest):
    if is_quoted(src):
        src = remove_quotes(src)
    if is_quoted(dest):
        dest = remove_quotes(dest)
    
    try:
        os.rename(src, dest)
    except:
        debug('lib.move', 'cannot move %s to %s' % (src, dest), error)
        return False

    return True

def unlink_dir(dirname):
    if not does_path_exist(dirname):
        return

    flist = os.listdir(dirname)

    for fname in flist:
        full_name = build_path([dirname, fname])

        if is_dir(full_name):
            unlink_dir(full_name)
        else:
            os.unlink(full_name)
                                                                                
    os.rmdir(dirname)
    return

# -- widgets --

def check_widgets(xml, widget_names):
    for widget_name in widget_names:
        widget = xml.get_widget(widget_name)
        if not widget:
            msg = 'The widget "%s" cannot be loaded' % widget_name
            print 'lib.check_widgets(): %s' % msg
            print 'Please check your glade file'
            assert(False)

    return

def set_button_data(button, text, img_pixbuf = None):
    alignment = button.get_children()[0]
    hbox = alignment.get_children()[0]
    img, label = hbox.get_children()
    label.set_text_with_mnemonic(text)

    if img_pixbuf:
        img.set_from_pixbuf(img_pixbuf)
    return

# simple combo box (text only) manipulation routines

COLUMN_TEXT = 0
COLUMN_VALUE = 1

def textcombo_setup(combo, value_list, default_value, bold = False):
    active_index = -1

    # create/clear the model
    model = combo.get_model()
    if not model:
        model = ListStore(TYPE_STRING, TYPE_PYOBJECT)
        combo.set_model(model)

        text_renderer = CellRendererText()
        text_renderer.set_property('xalign', 0)

        if bold:
            font = FontDescription('bold')
            text_renderer.set_property('font-desc', font)

        combo.pack_start(text_renderer, True)
        combo.add_attribute(text_renderer, 'text', COLUMN_TEXT)
    else:
        model.clear()

    # fix default value: if it's not in the list
    found_default = False
    for item in value_list:
        item_text, item_value = item
        if item_value == default_value:
            found_default = True
            break

    if not found_default and len(value_list) > 0:
        item_text, item_value = value_list[0]
        default_value = item_value

    # fill the combo and remember the active index
    for index in range(len(value_list)):
        item_text, item_value = value_list[index]
        if item_value == default_value:
            active_index = index
        model.append([item_text, item_value])

    # setup the combo appearence
    if active_index > -1:
        combo.set_active(active_index)

    if not len(value_list):
        combo.set_sensitive(False)
    return

def textcombo_get_value(combo):
    model = combo.get_model()
    active = combo.get_active()
    if active < 0:
        return ''

    return model[active][COLUMN_VALUE]

def textcombo_set_value(combo, keys):
    model = combo.get_model()
    if not model:
        return False

    found = False
    for key in keys:
        index = 0
        for row in model:
            if row[COLUMN_VALUE] == key:
                combo.set_active(index)
                found = True
                break
            index += 1
        if found:
            break

    if not found:
        return False

    return True

def treepath_tuple_to_string(path):
    result = ''

    for i in range(len(path)):
        result += '%d' % path[i]
        if i < (len(path) - 1):
            result += ':'
    return result

def treepath_string_to_tuple(path):
    result = ()

    if not path or not len(path):
        return result

    items = path.split(':')
    for item in items:
        result += int(item),
    return result


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
