# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk.glade import XML
from os import system
from gettext import textdomain
from winkirip.lib import check_widgets, debug, error, build_path, winki_path
from winkirip.alert import AlertWarning, AlertError
from winkirip.select_properties import SelectProperties

class BasePage:
    def __init__(self, glade_name, app, druid):
        self.__xml = None
        self.__app = app
        self.__druid = druid
        self.__needs_init = True

        self.__load_xml(glade_name)

        self.__props = None
        return

    # -- public interface --

    def check_widgets(self):
        check_widgets(self.__xml, self.get_widget_list())
        return

    def get_widget(self, widget_name):
        return self.__xml.get_widget(widget_name)

    def get_content(self):
        return self.__xml.get_widget('main')

    def get_icon(self):
        return 'winki.png'

    def get_title(self):
        return 'Default'

    def get_widget_list(self):
        debug('BasePage.get_widget_list()', 'not overwritten', error)
        return []

    def get_signal_list(self):
        debug('BasePage.get_signal_list()', 'not overwritten', error)
        return {}

    def get_project(self):
        return self.__app.get_project()

    def get_app(self):
        return self.__app

    def get_druid(self):
        return self.__druid

    def get_needs_init(self):
        return self.__needs_init

    def set_needs_init(self, needs_init):
        self.__needs_init = needs_init

    def on_prepare(self):
        debug('BasePage.on_prepare()', 'not overwritten', error)
        return False

    def on_next(self):
        debug('BasePage.on_next()', 'not overwritten', error)
        return False

    def on_prev(self):
        debug('BasePage.on_prev()', 'not overwritten', error)
        return False

    def can_exit(self):
        debug('BasePage.can_exit()', 'not overwritten', error)
        return False

    def invalidate(self):
        self.set_needs_init(True)
        return

    def update_project(self):
        debug('BasePage.update_project()', 'not overwritten', error)
        return False

    def get_settings_callback(self):
        """This function should return a callback which deals with settings changes"""
        return None

    def show_help(self):
        # temp: this is ugly workaround for gnome.help_display() not working for me
        # todo: use gnome.help_display()
        #gnome.help_display('winki', None)
        try:
            system('yelp ghelp:/usr/share/gnome/help/winki/C/winki.xml')
        except:
            pass
        # end temp
        return True

    def show_alert(self, primary, secondary, affirmative):
        cfg = self.get_project().get_config()
        if not cfg.get_show_popups():
            return True

        dlg_info = AlertWarning()
        dlg_info.show(primary, secondary, affirmative)

        if dlg_info.state == 'ok':
            return True

        return False

    def show_error(self, primary, secondary):
        cfg = self.get_project().get_config()
        if not cfg.get_show_popups():
            return

        dlg_error = AlertError()
        dlg_error.show(primary, secondary)
        return

    def set_stoplight(self, light):
        self.__druid.set_stoplight(light)
        return

    def set_window_title(self, title):
        self.__druid.set_window_title(title)
        return

    def disable_buttons(self, button_list = None):
        if not button_list:
            button_list = ['button_settings', 'button_quit', 'button_back', 'button_next']
        self.__druid.enable_buttons(False, button_list)
        return

    def enable_buttons(self, button_list = None):
        if not button_list:
            button_list = ['button_settings', 'button_quit', 'button_back', 'button_next']
        self.__druid.enable_buttons(True, button_list)
        return

    def alert_missing_tool(self, toolname):
        debug('PageBase.alert_missing_tool()', 'no %s installed' % toolname, error)
        primary = _('No %s installation found') % toolname
        secondary = _('Winki the Ripper cannot work without %(tool_name)s. Install %(tool_name)s and try again.') % {'tool_name':toolname, 'tool_name':toolname}
        self.show_error(primary, secondary)
        return

    def show_stream_properties(self, vlist, alist, slist, update_callback = None):
        if not self.__props:
            iso_codes = self.get_project().get_iso_codes()
            self.__props = SelectProperties(iso_codes)
            self.__props.set_store_position(self.__druid.store_window_position)
            self.__props.set_restore_position(self.__druid.restore_window_position)

        self.__props.show(vlist, alist, slist, update_callback)
        return

    # -- helpers --

    def __load_xml(self, glade_name):
        xml_path = build_path([winki_path, 'glade', glade_name])
        try:
            self.__xml = XML(xml_path, 'main', textdomain())
        except:
            print 'BasePage.__load_xml(): Cannot load "%s" UI file' % glade_name
            return

        signals = self.get_signal_list()
        if len(signals) > 0:
            self.__xml.signal_autoconnect(signals)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
