# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk.glade import XML
from winkirip.lib import build_path, winki_path, debug, error, set_button_data, to_float
from winkirip.command import Command
from time import sleep


class BasePlayerPage:
    def __init__(self):
        self.__xml = None
        self.__mplayer = None
        self.__action_callback = None
        self.__ready_callback = None
        self.__disable_update = False
        self.__current_position = -1
        self.__action_text = [_('Play'), _('Stop')]
        self.__action_pixbuf = None
        return

    # -- public interface --

    def build_player_command(self):
        # oops: you're supposed to overwrite this func in your deirved class
        assert(False)
        return None

    def get_player_callback(self):
        # oops: you're supposed to overwrite this func in your deirved class
        assert(False)
        return None

    def get_player_target_widget(self):
        # oops: you're supposed to overwrite this func in your deirved class
        assert(False)
        return None

    def get_ready_callback(self):
        # this function will be called after stopping the player
        return None

    def set_action_text(self, start_text, stop_text):
        if start_text:
            self.__action_text[0] = start_text

        if stop_text:
            self.__action_text[1] = stop_text

        self.__set_action_text()
        return

    def set_action_pixbuf(self, pixbuf):
        self.__action_pixbuf = pixbuf
        return

    def set_player_stoplight(self, light):
        return

    def set_range(self, slider_range):
        slider = self.__xml.get_widget('hscale_slider')
        adjustment = slider.get_adjustment()
        adjustment.upper = slider_range
        slider.set_adjustment(adjustment)
        return

    def dock(self, dock_target):
        xml_path = build_path([winki_path, 'glade', 'player.glade'])
        try:
            self.__xml = XML(xml_path, 'main')
        except:
            print 'BasePlayerPage: Cannot load "main" from %s' % (xml_path)
            raise SystemExit

        dock_target.add(self.__xml.get_widget('main'))

        dic = {'on_button_action_clicked' : self.on_action,
               'on_button_30_backward_clicked' : self.on_backward_30,
               'on_button_10_backward_clicked' : self.on_backward_10,
               'on_button_1_backward_clicked' : self.on_backward_1,
               'on_button_1_forward_clicked' : self.on_forward_1,
               'on_button_10_forward_clicked' : self.on_forward_10,
               'on_button_30_forward_clicked' : self.on_forward_30,
               'on_hscale_slider_value_changed' : self.on_hscale_value_changed,
               'on_hscale_slider_change_value' : self.on_hscale_change_value,
               'on_hscale_slider_scroll' : self.on_hscale_scroll}
        self.__xml.signal_autoconnect(dic)

        self.__enable_ui(False)
        return
            
    def has_player(self):
        return (self.__mplayer != None)

    # -- signal handlers --

    def on_action(self, widget, event = None):
        if self.has_player():
            self.__stop_player()
        else:
            self.__start_player()
        return True

    def on_hscale_change_value(self, widget, event, pos):
        adjustment = widget.get_adjustment()
        if pos >= adjustment.upper:
            return True
        return False

    def on_hscale_scroll(self, widget, event = None):
        # todo
        return False

    def on_hscale_value_changed(self, widget, event = None):
        if self.__disable_update:
            self.__disable_update = False
            return True

        if not self.__mplayer:
            debug('BasePlayerPage.on_forward_1()', 'no mplayer instance', error)
            return True

        self.__mplayer.to_stdin('seek %d 2\n' % widget.get_value())
        sleep(1)
        return True

    def on_forward_1(self, widget, event = None):
        if not self.__mplayer:
            debug('BasePlayerPage.on_forward_1()', 'no mplayer instance', error)
            return True

        self.__mplayer.to_stdin('seek 1\n')
        return True

    def on_forward_10(self, widget, event = None):
        if not self.__mplayer:
            debug('BasePlayerPage.on_forward_10()', 'no mplayer instance', error)
            return True

        self.__mplayer.to_stdin('seek 10\n')
        return True

    def on_forward_30(self, widget, event = None):
        if not self.__mplayer:
            debug('BasePlayerPage.on_forward_30()', 'no mplayer instance', error)
            return True

        self.__mplayer.to_stdin('seek 30\n')
        return True

    def on_backward_1(self, widget, event = None):
        if not self.__mplayer:
            debug('BasePlayerPage.on_backward_1()', 'no mplayer instance', error)
            return True

        self.__mplayer.to_stdin('seek -2\n')
        return True

    def on_backward_10(self, widget, event = None):
        if not self.__mplayer:
            debug('BasePlayerPage.on_backward_10()', 'no mplayer instance', error)
            return True

        self.__mplayer.to_stdin('seek -10\n')
        return True

    def on_backward_30(self, widget, event = None):
        if not self.__mplayer:
            debug('BasePlayerPage.on_backward_30()', 'no mplayer instance', error)
            return True

        self.__mplayer.to_stdin('seek -30\n')
        return True

    # -- helpers --

    def __set_action_text(self):
        txt = self.__action_text[1]
        if not self.__mplayer:
            txt = self.__action_text[0]

        button_action = self.__xml.get_widget('button_action')
        set_button_data(button_action, txt, self.__action_pixbuf)
        return

    def __start_player(self):
        cmd = self.build_player_command()
        if not cmd or not len(cmd):
            debug('BasePlayerPage.start_player()', 'no command supplied', error)
            return

        area = self.get_player_target_widget()
        if not area:
            debug('BasePlayerPage.start_player()', 'no player target widget supplied', error)
            return

        self.__action_callback = self.get_player_callback()
        if not self.__action_callback:
            debug('BasePlayerPage.start_player()', 'no player callback supplied', error)
            return

        self.__ready_callback = self.get_ready_callback()

        XID = area.window.xid
        left, top, width, height = area.get_allocation()

        cmdStr = cmd
        cmdStr += (' -wid %d -x %d -y %d -zoom -slave -nocache' % (XID, width, height))

        assert(self.__mplayer == None)
        if self.__mplayer:
            debug('BasePlayerPage.start_player()', 'mplayer instance still loaded in memory', error)
            return

        self.__mplayer = Command(cmdStr, self.__update_position)
        self.__enable_ui(True)
        self.set_player_stoplight('stop')

        self.__mplayer.run()
        return

    def __stop_player(self):
        if not self.__mplayer:
            debug('BasePlayerPage.stop_player()', 'no mplayer instance running')
            return

        self.__mplayer.stop()
        self.__update_position('V: 0.0 A-V:')
        self.__mplayer = None
        self.__action_callback = None
        self.__enable_ui(False)
        self.set_player_stoplight('go')

        if self.__ready_callback:
            self.__ready_callback()
        return

    def __update_position(self, line):
        if not self.__mplayer:  # player already stopped, e.g. externally
            return
        pos_start = line.find('V:')
        if pos_start < 0:
            self.__action_callback(line)
            return

        pos_end = line.find('A-V:')
        if pos_end < 0 or pos_end < pos_start:
            self.__action_callback(line)
            return

        position = to_float(line[pos_start + 2 : pos_end])
        if position < 0:
            self.__action_callback(line)
            return

        if int(position) == self.__current_position:
            return

        self.__current_position = int(position)
        self.__disable_update = True
            
        slider = self.__xml.get_widget('hscale_slider')
        slider.set_value(self.__current_position)

        self.__action_callback(line)
        return

    def __enable_ui(self, enabled):
        button_30_backward = self.__xml.get_widget('button_30_backward')
        button_10_backward = self.__xml.get_widget('button_10_backward')
        button_1_backward = self.__xml.get_widget('button_1_backward')
        button_30_forward = self.__xml.get_widget('button_1_forward')
        button_10_forward = self.__xml.get_widget('button_10_forward')
        button_1_forward = self.__xml.get_widget('button_30_forward')
        slider = self.__xml.get_widget('hscale_slider')

        button_30_backward.set_sensitive(enabled)
        button_10_backward.set_sensitive(enabled)
        button_1_backward.set_sensitive(enabled)
        button_30_forward.set_sensitive(enabled)
        button_10_forward.set_sensitive(enabled)
        button_1_forward.set_sensitive(enabled)
        slider.set_sensitive(enabled)

        self.__set_action_text()
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
