# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import CellRendererPixbuf, CellRendererText, TreeViewColumn
from gtk import TreeStore, SELECTION_MULTIPLE, TreeRowReference
from gtk import TREE_VIEW_DROP_BEFORE, TREE_VIEW_DROP_INTO_OR_BEFORE
from gtk import TREE_VIEW_DROP_INTO_OR_AFTER, TREE_VIEW_DROP_AFTER
from gtk.gdk import Pixbuf, ACTION_DEFAULT
from gtk.gdk import ACTION_MOVE, ACTION_COPY, BUTTON1_MASK
from gtk.gdk import pixbuf_new_from_file
from gobject import TYPE_STRING, TYPE_PYOBJECT
from winkirip.container_title import Title
from winkirip.lib import treepath_tuple_to_string, treepath_string_to_tuple
from winkirip.lib import debug, error, winki_path, build_path


COLUMN_LANG_PIXBUF = 0
COLUMN_LANG_TITLE = 1
COLUMN_LANG_OBJECT = 2

COLUMN_TREE_LANG = 0
COLUMN_TREE_TYPE = 1
COLUMN_TREE_TITLE = 2
COLUMN_TREE_OBJECT = 3

# containers are the first level in the tree, titles - the second and
# chapters/(audio/subtitle streams) - the third
CONT_PATH_LEN = 1
TITLE_PATH_LEN = 2
STREAM_PATH_LEN = 3

class BaseTwoTreesPage:
    def __init__(self):
        self.__source_treeview = None
        self.__dest_treeview = None
        return
        
    # -- public interface --

    def set_treeviews(self, source_treeview, dest_treeview):
        self.__source_treeview = source_treeview
        self.__dest_treeview = dest_treeview
        return

    def get_source_treeview(self):
        return self.__source_treeview

    def get_dest_treeview(self):
        return self.__dest_treeview

    def create_treeview(self, treeview, header_text):
        model = TreeStore(Pixbuf, TYPE_STRING, TYPE_PYOBJECT)
        treeview.set_model(model)
        treeview.set_rules_hint(True)

        renderer = CellRendererPixbuf()
        renderer.set_property('xalign', 0)
        column = TreeViewColumn('', renderer, pixbuf = COLUMN_LANG_PIXBUF)
        treeview.append_column(column)

        renderer = CellRendererText()
        column = TreeViewColumn(header_text, renderer, markup = COLUMN_LANG_TITLE)
        treeview.append_column(column)

        selection = treeview.get_selection()
        selection.set_mode(SELECTION_MULTIPLE)

        treeview.set_reorderable(True)
        return

    def setup_dnd(self, treeview, get_callback, received_callback):
        flags = ACTION_DEFAULT | ACTION_MOVE | ACTION_COPY
        treeview.enable_model_drag_dest([('text/plain', 0, 0)], flags)
        treeview.connect("drag-data-received", received_callback)

        treeview.enable_model_drag_source(BUTTON1_MASK, [('text/plain', 0, 0)], flags)
        treeview.connect("drag-data-get", get_callback)
        return

    def drag_data_get(self, treeview, selection):
        treeselection = treeview.get_selection()
        (model, pathlist) = treeselection.get_selected_rows()
        data = ''
        for path in pathlist:
            string_path = treepath_tuple_to_string(path)
            data += '%s\t%s\n' % (treeview.get_name(), string_path)
        selection.set('text/plain', 8, data)
        return True

    def drag_data_receive(self, treeview, x, y, selection):
        drop_info = treeview.get_dest_row_at_pos(x, y)

        dest_tv_name = treeview.get_name()
        paths, source_tv_name = self.__filter_paths(dest_tv_name, selection.data, drop_info)
        if not len(paths):
            return True

        self.__move_streams(paths, source_tv_name, dest_tv_name, drop_info)
        return True

    def move_selected(self, treeview):
        source_tv_name = treeview.get_name()
        dest_tv_name = self.__source_treeview.get_name()
        if self.__source_treeview.get_name() == source_tv_name:
            dest_tv_name = self.__dest_treeview.get_name()

        paths = []
        treeselection = treeview.get_selection()
        (model, paths) = treeselection.get_selected_rows()

        self.__move_streams(paths, source_tv_name, dest_tv_name, None)
        return

    # -- helpers for the language (audio/subtitle) pages --

    def fill_dest_container(self, project):
        """Fills the dest container with copies of the appropriate streams from
        all enabled titles
        """

        source_titles = project.get_enabled_titles()

        # multiple source containers are not handled by this class
        assert(len(project.get_source()) == 1)

        # the source tree should show exactly as much titles as the dest tree
        dest_container = project.get_dest()[0]
        assert(len(dest_container) == len(source_titles))

        for title_idx in range(len(source_titles)):
            dest_title = dest_container[title_idx]
            self.reset_title_streams(dest_title)
            streams = self.get_title_streams(source_titles[title_idx])
            for stream in streams:
                dest_stream = stream.copy()
                dest_stream.set_source(stream)
                self.add_title_stream(dest_title, dest_stream)
        return

    def reset_dest_container(self, project):
        """Removes all appropriate streams from the dest containers
        """

        for dest_container in project.get_dest():
            for dest_title in dest_container:
                self.reset_title_streams(dest_title)
        return

    def fill_model(self, title_list, treeview, iso_codes, default_name):
        """Fills the model of the passed treeview with the appropriate streams
        from the title_list
        """

        model = treeview.get_model()

        title_fname = build_path([winki_path, 'glade', 'media_title.png'])
        title_pixbuf = pixbuf_new_from_file(title_fname)

        for title in title_list:
            title_name = ('<span weight="bold">%s</span>' % title.get_name())
            iterator = model.append(None, [title_pixbuf, title_name, title])
            sid = 1
            for stream in self.get_title_streams(title):
                lang = iso_codes.find_iso_639_1(stream.get_language())
                if not lang:
                    debug('BaseTwoTreesPage.__fill_model()', 'unknown language %s' % stream.get_language())
                    continue

                stream_name = lang.get_name()
                if not stream_name or not len(stream_name):
                    stream_name = (default_name % sid)

                stream_name = ('<span weight="normal">%s</span>' % stream_name)
                model.append(iterator, [lang.get_flag(), stream_name, stream])
                sid += 1

        treeview.expand_all()
        return

    def sumbit_selected_streams(self, model, dest_containers):
        """Copy the streams the user selected from the dest treeview to the
        dest container
        """

        # at this point we should have only one dest container
        # todo: check PageSize.on_prev() just to be sure
        if len(dest_containers) != 1:
            debug('BaseTwoTreesPage.submit_selected_streams()', 'expected 1 dest container, got %d' % len(dest_containers), error)
            return False

        for row in model:
            flag, text, dest_title_row = row

            dest_title = None
            for title in dest_containers[0]:
                if title.get_id() == dest_title_row.get_id():
                    dest_title = title
                    break

            if not dest_title:
                debug('BaseTwoTreesPage.submit_selected_streams()', 'title id\'s does not match', error)
                return False

            # reset the target title and readd the streams
            self.reset_title_streams(dest_title)

            dest_title_iter = row.iter
            stream_iter = model.iter_children(dest_title_iter)
            while stream_iter:
                stream = model.get_value(stream_iter, COLUMN_LANG_OBJECT)
                self.add_title_stream(dest_title, stream)
                stream_iter = model.iter_next(stream_iter)

        return True

    def get_title_streams(self, title):
        # should be overwritten in the derived class
        assert(False)
        return

    def add_title_stream(self, title, stream):
        # should be overwritten in the derived class
        assert(False)
        return

    def reset_title_streams(self, title):
        # should be overwritten in the derived class
        assert(False)
        return

    # -- helpers implemented by the tree/language pages --

    def validate_move(self, source_tv_name, source_path, dest_tv_name, dest_path, source_items):
        # should be overwritten in the derived class
        assert(False)
        return False

    def is_tree(self):
        # should be overwritten in the derived class
        assert(False)
        return False

    def is_language(self):
        # should be overwritten in the derived class
        assert(False)
        return False

    def get_dest_iter(self, source_iter, source_model, dest_model, dest_pos, dest_path):
        # should be overwritten in the derived class
        assert(False)
        return None, None

    def get_values(self, model, iter, copy_object):
        # should be overwritten in the derived class
        assert(False)
        return []

    # -- helpers --

    def __filter_paths(self, dest_tv_name, data, drop_info):
        result = []

        source_items = data.split('\n')
        if not len(source_items):
            return result, None

        # ignore external sources for now
        source_name = self.__source_treeview.get_name()
        dest_name = self.__dest_treeview.get_name()
        tmp = source_items[0].split('\t')
        if len(tmp) != 2 or (tmp[0] != source_name and tmp[0] != dest_name):
            return result, None

        dest_path = None
        if drop_info:
            dest_path, dest_pos = drop_info

        source_tv_name = None
        for source_item in source_items:
            if not source_item or not len(source_item):
                continue

            source_tv_name, source_path = source_item.split('\t')

            if self.validate_move(source_tv_name, source_path, dest_tv_name, dest_path, source_items):
                result.append(source_path)

        return result, source_tv_name

    def __move_to_same_tv(self, source_paths, source_tv_name, drop_info):
        treeview = None
        if self.__source_treeview.get_name() == source_tv_name:
            treeview = self.__source_treeview
        else:
            treeview = self.__dest_treeview
        model = treeview.get_model()
        dest_path, dest_pos = drop_info
        dest_ref = TreeRowReference(model, dest_path)
        dest_iter = model.get_iter(dest_path)
        parent_iter = model.iter_parent(dest_iter)

        # collect references to the source rows
        source_paths = self.__paths_to_tuple(source_paths, dest_pos)
        source_refs = []
        for source_path in source_paths:
            # don't drop over the selection: this would invalidate our
            # precious dest_ref
            if source_path == dest_path:
                return

            source_refs.append(TreeRowReference(model, source_path))

        # move and collect references to the inserted rows
        select_refs = []
        for source_ref in source_refs:
            source_path = source_ref.get_path()
            source_iter = model.get_iter(source_path)
            value_list = self.get_values(model, source_iter, False)

            # if we are about to move a title - we have to move all associated streams too
            stream_refs = []
            if self.is_tree() and len(source_path) == TITLE_PATH_LEN:
                stream_iter = model.iter_children(source_iter)
                while stream_iter:
                    stream_refs.append(self.get_values(model, stream_iter, False))
                    stream_iter = model.iter_next(stream_iter)

            # disallow moving streams to titles
            if self.is_tree() and len(source_path) == STREAM_PATH_LEN and len(dest_ref.get_path()) == TITLE_PATH_LEN:
                return

            model.remove(source_iter)

            dest_path = dest_ref.get_path() 
            dest_iter = model.get_iter(dest_path)
            if dest_pos == TREE_VIEW_DROP_BEFORE or dest_pos == TREE_VIEW_DROP_INTO_OR_BEFORE:
                it = model.insert_before(parent_iter, dest_iter, value_list)
                select_refs.append(TreeRowReference(model, model.get_path(it)))
            else:
                it = model.insert_after(parent_iter, dest_iter, value_list)
                select_refs.append(TreeRowReference(model, model.get_path(it)))

            for stream_ref in stream_refs:
                model.append(it, stream_ref)

        # select the inserted rows
        selection = treeview.get_selection()
        selection.unselect_all()
        for select_ref in select_refs:
            selection.select_path(select_ref.get_path())

        # shift the focus
        treeview.grab_focus()
        return

    def __move_to_other_tv(self, source_paths, source_tv_name, drop_info):
        source_tv, dest_tv, source_model, dest_model = self.__get_treeview_objects(source_tv_name)

        dest_path = None
        dest_pos = None
        if drop_info:
            dest_path, dest_pos = drop_info
            iterator = dest_model.get_iter(dest_path)

            # dropping on a title results in appending to the title
            if self.is_language() and isinstance(dest_model.get_value(iterator, COLUMN_LANG_OBJECT), Title):
                dest_pos = None

            if self.is_tree() and isinstance(dest_model.get_value(iterator, COLUMN_TREE_OBJECT), Title):
                dest_pos = None

        # collect references to the source rows
        source_refs = []
        for source_path in source_paths:
            source_refs.append(TreeRowReference(source_model, source_path))
            
            # dropping into antoher title results in appending to the right one
            # if we move streams in "page_audio" or "page_subtitle";
            if self.is_language() and not self.__same_title(source_path, dest_path):
                dest_pos = None

        # move and collect references to the inserted rows
        select_refs = []
        for source_ref in source_refs:
            source_iter = source_model.get_iter(source_ref.get_path())

            dest_pos, dest_iter = self.get_dest_iter(source_iter, source_model, dest_model, dest_pos, dest_path)
            if not dest_iter:
                debug('BaseTwoTreesPage.__move_to_other_tv()', 'no destination iterator found: skipping')
                continue

            parent_iter = dest_model.iter_parent(dest_iter)
            value_list = self.get_values(source_model, source_iter, True) # this cludge with the "True" will bite back...

            if not dest_pos:
                it = dest_model.append(dest_iter, value_list)
                select_refs.append(TreeRowReference(dest_model, dest_model.get_path(it)))

            elif dest_pos == TREE_VIEW_DROP_BEFORE or dest_pos == TREE_VIEW_DROP_INTO_OR_BEFORE:
                it = dest_model.insert_before(parent_iter, dest_iter, value_list)
                select_refs.append(TreeRowReference(dest_model, dest_model.get_path(it)))

            else:
                it = dest_model.insert_after(parent_iter, dest_iter, value_list)
                select_refs.append(TreeRowReference(dest_model, dest_model.get_path(it)))

            # remove from the source tree if we move streams in "page_audio" or "page_subtitle"
            # or the user (re)moves objects from the dest treeview
            if self.is_language() or source_tv_name == 'treeview_dest':
                source_path = source_ref.get_path()
                source_model.remove(source_model.get_iter(source_path)) 
                dest_tv.expand_all()
                #if not parent_iter:
                #    dest_tv.expand_row(dest_model[0].path, True)
                #else:
                #    dest_tv.expand_row(dest_model.get_path(parent_iter), True)

        # unselect the source treeview
        selection = source_tv.get_selection()
        selection.unselect_all()

        # select the inserted rows
        selection = dest_tv.get_selection()
        selection.unselect_all()
        for select_ref in select_refs:
            selection.select_path(select_ref.get_path())

        # shift the focus
        dest_tv.grab_focus()
        return

    def __get_treeview_objects(self, source_tv_name):
        source_tv = None
        if self.__source_treeview.get_name() == source_tv_name:
            source_tv = self.__source_treeview
        else:
            source_tv = self.__dest_treeview
        source_model = source_tv.get_model()

        dest_tv = None
        if self.__source_treeview.get_name() == source_tv_name:
            dest_tv = self.__dest_treeview
        else:
            dest_tv = self.__source_treeview
        dest_model = dest_tv.get_model()
        return source_tv, dest_tv, source_model, dest_model

    def __move_streams(self, source_paths, source_tv_name, dest_tv_name, drop_info):
        if source_tv_name == dest_tv_name:
            self.__move_to_same_tv(source_paths, source_tv_name, drop_info)
        else:
            self.__move_to_other_tv(source_paths, source_tv_name, drop_info)
        return

    def __reverse_paths(self, paths):
        result = []
        for path in paths:
            result.append(treepath_string_to_tuple(path))

        result.sort()
        result.reverse()
        return result

    def __paths_to_tuple(self, paths, pos):
        result = []

        # reverse and convert
        if pos != TREE_VIEW_DROP_BEFORE and pos != TREE_VIEW_DROP_INTO_OR_BEFORE:
            result = self.__reverse_paths(paths)

        # just convert
        else:
            for path in paths:
                result.append(treepath_string_to_tuple(path))

        return result

    def __same_title(self, source_path, dest_path):
        source_tuple = source_path
        if not isinstance(source_tuple, tuple):
            source_tuple = treepath_string_to_tuple(source_path)

        dest_tuple = dest_path
        if not isinstance(dest_tuple, tuple):
            dest_tuple = treepath_string_to_tuple(dest_path)

        if len(source_tuple) != len(dest_tuple):
                return False

        # both paths point to the same title if all but the last elements equal
        for i in range (len(source_tuple) - 1):
            if source_tuple[i] != dest_tuple[i]:
                return False

        return True

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
