# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import TreeStore, CellRendererPixbuf, CellRendererText, TreeViewColumn
from gtk.gdk import pixbuf_new_from_file, Pixbuf
from gobject import TYPE_STRING, TYPE_PYOBJECT
from winkirip.page_base import BasePage
from winkirip.page_base_player import BasePlayerPage
from winkirip.container_title import Title
from winkirip.constructor_player import ConstructorPlayer
from winkirip.lib import winki_path, build_path, debug, error


COLUMN_PIXBUF = 0
COLUMN_TEXT = 1
COLUMN_TITLE = 2
COLUMN_CONTAINER = 3

class PageCrop(BasePage, BasePlayerPage):
    def __init__(self, xml, app, druid):
        BasePage.__init__(self, xml, app, druid)
        BasePlayerPage.__init__(self)

        dock_target = self.get_widget('player_dock')
        BasePlayerPage.dock(self, dock_target)
        BasePlayerPage.set_action_text(self, _('Crop _Detection'), _('_Get value'))

        scissors_fname = build_path([winki_path, 'glade', 'button_crop.png'])
        scissors_pixbuf = pixbuf_new_from_file(scissors_fname )
        BasePlayerPage.set_action_pixbuf(self, scissors_pixbuf)
        return
        
    # -- methods inherited from page_base_player.BasePlayerPage --

    def build_player_command(self):
        treeview = self.get_widget('treeview_titles')
        selection = treeview.get_selection()
        model, iterator, = selection.get_selected()
        if not model or not iterator:
            primary = _('No title selected') 
            secondary = _('Select a title from the list in order to perform crop detection on it')
            self.show_error(primary, secondary)
            return None

        path = model.get_path(iterator)
        selected_title = int(path[0])

        title = model[selected_title][COLUMN_TITLE]
        title_length = title.get_length()
        BasePlayerPage.set_range(self, title_length)

        video = title.get_video()
        if not len(video):
            debug('PageCrop.build_player_command()', 'no video in this title', error)
            return None

        source_video = video[0].get_source()
        if not source_video:
            debug('PageCrop.build_player_command()', 'cannot find source video', error)
            return None

        source_container_id = source_video.get_container_id()
        source_containers = self.get_project().get_source()
        source_container = None
        for cont in source_containers:
            if cont.get_id() == source_container_id:
                source_container = cont
                break

        if not source_container:
            debug('PageCrop.build_player_command()', 'cannot find source container', error)
            return None

        device = source_container.get_device()

        tool_checker = self.get_project().get_tool_checker()

        constr = ConstructorPlayer(tool_checker,
                                   self.get_project().get_config())
        constr.set_source(device, source_video.get_title_id())
        constr.set_cropdetect()
        constr.set_source_type(source_container.get_type())

        success, shell_cmd = constr.construct()
        if not success:
            debug('PageCrop.build_player_command()', 'command construction failed', error)
            return ''
        return shell_cmd

    def get_player_callback(self):
        return self.__parse_crop

    def get_ready_callback(self):
        return self.__submit_crop

    def get_player_target_widget(self):
        return self.get_widget('drawingarea_player')

    def set_player_stoplight(self, light):
        self.set_stoplight(light)
        return

    # -- methods inherited from page_base.BasePage --

    def get_icon(self):
        return 'page_crop.png'

    def get_title(self):
        return _('Crop Detection')

    def get_signal_list(self):
        return {'on_checkbutton_apply_for_all_toggled' : self.on_apply_all_toggled}

    def get_widget_list(self):
        return ['checkbutton_apply_for_all']

    def on_prepare(self):
        self.set_stoplight('stop')

        if not self.get_needs_init():
            return

        title_list = self.__extract_titles()
        self.__init_treeview(title_list)

        self.__update_ui()

        self.set_needs_init(False)
        return True

    def on_next(self):
        if BasePlayerPage.has_player(self):
            return False

        return self.update_project()

    def on_prev(self):
        return (not BasePlayerPage.has_player(self))

    def can_exit(self):
        return (not BasePlayerPage.has_player(self))

    def update_project(self):
        self.disable_buttons()

        self.__submit_crop()

        self.get_druid().invalidate_after('crop')
        self.enable_buttons()
        return True

    def show_help(self):
        return BasePage.show_help(self)

    # -- signal handlers --

    def on_apply_all_toggled(self, widget, signal = None):
        self.__submit_crop()
        return True

    def on_edit(self, renderer, path, text, model):
        it = model.get_iter(path)
        obj = model.get_value(it, COLUMN_TITLE)

        if not isinstance(obj, Title):
            return False

        title_name = ('<span weight="bold">%s</span>' % text)
        model.set(it, COLUMN_TEXT, title_name)

        obj.set_name(text)

        self.get_druid().invalidate_after('crop')
        return True

    def on_selection_changed(self, selection):
        model, iterator, = selection.get_selected()
        if not model or not iterator:
            debug('PageCrop.on_selection_changed()', 'no title selected')
            return False

        spin_w = self.get_widget('spinbutton_w')
        spin_h = self.get_widget('spinbutton_h')
        spin_x = self.get_widget('spinbutton_x')
        spin_y = self.get_widget('spinbutton_y')

        path = model.get_path(iterator)
        selected_title = int(path[0])
        obj = model[selected_title][COLUMN_TITLE]
        crop_values = obj.get_crop()
        if not crop_values or len(crop_values) != 4:
            crop_values = [0, 0, 0, 0]

        spin_w.set_text('%d' % crop_values[0])
        spin_h.set_text('%d' % crop_values[1])
        spin_x.set_text('%d' % crop_values[2])
        spin_y.set_text('%d' % crop_values[3])
        return True

    # -- public interface --

    # -- helpers --

    def __extract_titles(self):
        title_list = []

        dest = self.get_project().get_dest()
        for disc in dest:
            for title in disc:
                already_copied = False
                for item in title_list:
                    used_title, disc = item
                    if used_title.get_id() == title.get_id():
                        already_copied = True
                        break

                if not already_copied:
                    title_list.append([title, disc])

        return title_list

    def __init_treeview(self, title_list):
        treeview = self.get_widget('treeview_titles')
        model = treeview.get_model()
        if not model:
            self.__create_treeview()
        else:
            model.clear()

        self.__fill_model(title_list)
        return

    def __fill_model(self, title_list):
        treeview = self.get_widget('treeview_titles')
        model = treeview.get_model()

        title_fname = build_path([winki_path, 'glade', 'media_title.png'])
        title_pixbuf = pixbuf_new_from_file(title_fname)

        for item in title_list:
            title, container = item
            title_name = ('<span weight="bold">%s</span>' % title.get_name())
            model.append(None, [title_pixbuf, title_name, title, container])

        if len(title_list) > 0:
            selection = treeview.get_selection()
            selection.select_path(0)
        return

    def __create_treeview(self):
        model = TreeStore(Pixbuf, TYPE_STRING, TYPE_PYOBJECT, TYPE_PYOBJECT)

        treeview = self.get_widget('treeview_titles')
        treeview.set_model(model)
        treeview.set_rules_hint(True)

        renderer = CellRendererPixbuf()
        renderer.set_property('xalign', 0)
        column = TreeViewColumn('', renderer, pixbuf = COLUMN_PIXBUF)
        treeview.append_column(column)

        renderer = CellRendererText()
        renderer.set_property('editable', True)
        renderer.connect('edited', self.on_edit, model)
        column = TreeViewColumn(_('Title'), renderer, markup = COLUMN_TEXT)
        treeview.append_column(column)

        selection = treeview.get_selection()
        selection.connect('changed', self.on_selection_changed)
        return

    def __parse_crop(self, line):
        pos_start = line.find('-vf crop=')
        if pos_start < 0:
            debug('PageCrop.__parse_crop()', '-vf crop= not found')
            return

        pos_end = line.find(')', pos_start + 1)
        if pos_end < 0 or pos_end <= pos_start:
            debug('PageCrop.__parse_crop()', ') not found')
            return

        chunks = line[pos_start + len('-vf crop=') : pos_end].split(':')
        if len(chunks) != 4:
            debug('PageCrop.__parse_crop()', 'crop values != 4')
            return

        spin_w = self.get_widget('spinbutton_w')
        spin_h = self.get_widget('spinbutton_h')
        spin_x = self.get_widget('spinbutton_x')
        spin_y = self.get_widget('spinbutton_y')

        spin_w.set_text(chunks[0])
        spin_h.set_text(chunks[1])
        spin_x.set_text(chunks[2])
        spin_y.set_text(chunks[3])
        return

    def __submit_crop(self):
        checkbutton = self.get_widget('checkbutton_apply_for_all')
        treeview = self.get_widget('treeview_titles')

        selection = treeview.get_selection()
        model, iterator, = selection.get_selected()
        if not iterator and not checkbutton.get_active():
            debug('PageCrop.__sumbit_crop()', 'no title selected')
            return False

        spin_w = self.get_widget('spinbutton_w')
        spin_h = self.get_widget('spinbutton_h')
        spin_x = self.get_widget('spinbutton_x')
        spin_y = self.get_widget('spinbutton_y')

        w = int(spin_w.get_text())
        h = int(spin_h.get_text())
        x = int(spin_x.get_text())
        y = int(spin_y.get_text())

        if checkbutton.get_active():
            for row in model:
                obj = model.get_value(row.iter, COLUMN_TITLE)
                obj.set_crop([w, h, x, y])
        else:
            path = model.get_path(iterator)
            selected_title = int(path[0])
            obj = model[selected_title][COLUMN_TITLE]
            obj.set_crop([w, h, x, y])
        return True

    def __update_ui(self):
        checkbutton_for_all = self.get_widget('checkbutton_apply_for_all')

        treeview = self.get_widget('treeview_titles')
        model = treeview.get_model()

        if not model or len(model) < 2:
            checkbutton_for_all.set_sensitive(False)
        else:
            checkbutton_for_all.set_sensitive(True)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
