# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import ListStore, CellRendererText, TreeViewColumn
from gtk import FILE_CHOOSER_ACTION_OPEN, FILE_CHOOSER_ACTION_SELECT_FOLDER
from gobject import TYPE_STRING, TYPE_PYOBJECT
import re
from winkirip.lib import debug, error, normalize_path, is_file, is_readable
from winkirip.lib import escape_name, set_quotes, set_extension, split_path
from winkirip.lib import remove_quotes
from winkirip.page_base import BasePage
from winkirip.page_base_fileselect import BaseFileSelectorPage
from winkirip.container import Container, CONTAINER_DVD, CONTAINER_VCD
from winkirip.container import CONTAINER_FILE
from winkirip.scan_dvd import ScanDVD
from winkirip.scan_file import ScanFile
from winkirip.scan_devices import ScanDevices

COMBO_COLUMN_TEXT = 0
COMBO_COLUMN_DEVICE = 1
COMBO_COLUMN_TYPE = 2

TREE_COLUMN_TITLE = 0
TREE_COLUMN_DESCR = 1
TREE_COLUMN_OBJECT = 2

DEVICE_UNKNOWN = -1
DEVICE_DVD = 0
DEVICE_VCD = 1
DEVICE_FILE = 2
DEVICE_DVD_IMAGE = 3
DEVICE_DVD_FOLDER = 4
DEVICE_VCD_IMAGE = 5
DEVICE_VCD_FOLDER = 6

class PageSource(BasePage, BaseFileSelectorPage):
    """Druid page which takes care about the source selection"""

    def __init__(self, xml, app, druid):
        BasePage.__init__(self, xml, app, druid)
        BaseFileSelectorPage.__init__(self)
        self.__command = None
        self.__input_file = None
        return
        
    # -- methods inherited from page_base.BasePage --

    def get_icon(self):
        return 'page_source.png'

    def get_title(self):
        return _('Source')


    def get_signal_list(self):
        return {'on_button_add_dvd_clicked' : self.on_button_add_clicked,
                'on_button_add_vcd_clicked' : self.on_button_add_clicked,
                'on_button_add_file_clicked' : self.on_button_add_clicked,
                'on_button_remove_media_clicked' : self.on_button_remove_media_clicked,
                'on_entry_input_file_changed' : self.on_entry_input_files_changed,
                'on_button_input_file_clicked' : self.on_button_input_files_clicked,
                'on_button_cancel_scan_clicked' : self.on_cancel_scan_clicked,
                'on_button_open_clicked' : self.on_open_clicked,
                'on_combobox_changed' : self.on_combo_changed,
                'on_treeview1_button_press_event' : self.on_treeview_click,
                'on_button_rescan_clicked' : self.on_rescan_devices_clicked,
                'on_treeview1_key_press_event' : self.on_treeview_key}

    def get_widget_list(self):
        widgets = ['button_input_file', 'entry_input_file', 'button_add_dvd',
        'button_remove_media', 'button_add_vcd', 'button_add_file',
        'label_scan', 'progressbar_scan', 'button_cancel_scan', 'button_rescan']

        return widgets

    def on_prepare(self):
        self.set_stoplight('go')

        if not self.get_needs_init():
            return True

        self.__init_device_combos(False)
        self.__init_treeview()

        self.__enable_ui(True)
        self.__update_stoplight()

        self.set_needs_init(False)
        return self.__check_tools()

    def on_next(self):
        if not self.can_exit():
            return False

        if self.__get_container_count() < 1:
            primary = _('No media selected')
            secondary = _('You have to select at least one source media')
            self.show_error(primary, secondary)
            return False

        return self.update_project()

    def on_prev(self):
        #label_scan = self.get_widget('label_scan')
        #label_scan.set_text(text)

        return self.can_exit()

    def can_exit(self):
        return (not self.__command)

    def update_project(self):
        self.disable_buttons()

        self.get_project().reset_source()

        treeview = self.get_widget('treeview1')
        model = treeview.get_model()
        cid = 0
        for row in model:
            cont = model.get_value(row.iter, TREE_COLUMN_OBJECT)
            cont.set_id(cid)
            self.get_project().add_source(cont)
            cid += 1

        self.enable_buttons()
        return True

    # -- signal handlers --

    def on_edit_title(self, renderer, path, text, model):
        current_row = (int(path),)
        it = model.get_iter(current_row)
        model.set(it, TREE_COLUMN_TITLE, text)

        obj = model[current_row][TREE_COLUMN_OBJECT]
        obj.set_name(text)

        self.get_druid().invalidate_after('source')
        return True

    def on_button_add_clicked(self, widget, event = None):
        scan_type = ''
        if widget.get_name() == 'button_add_dvd':
            scan_type = 'dvd'
        elif widget.get_name() == 'button_add_vcd':
            scan_type = 'vcd'
        elif widget.get_name() == 'button_add_file':
            scan_type = 'file'
        else:
            # new scan button?
            return True

        if not self.__scan_media(scan_type, _('Scanning %s...')):
            debug('PageSource.on_button_add_clicked()', 'scan failed', error)
            primary = _('Unrecognized media')
            secondary = _('Media scan failed')
            self.show_error(primary, secondary)
            return True

        if widget.get_name() == 'button_add_dvd':
            combo = self.get_widget('combobox_dvd')
            self.__enable_row(combo, False, True)

        self.__update_button_remove()

        self.get_druid().invalidate_after('source')
        self.__update_stoplight()
        return True

    def on_button_remove_media_clicked(self, widget, event = None):
        treeview = self.get_widget('treeview1')
        selection = treeview.get_selection()
        model, selected_paths = selection.get_selected_rows()
        if not model or not selected_paths:
            primary = _('No media selected')
            secondary = _('Select the media to be removed from the list')
            self.show_error(primary, secondary)
            return True

        for sel_row in selected_paths:
            it = model.get_iter(sel_row)
            model.remove(it)

        self.get_druid().invalidate_after('source')
        self.__update_button_remove()
        self.__update_stoplight()

        treeview.grab_focus()
        return True

    def on_combo_changed(self, widget, event = None):
        self.get_druid().invalidate_after('source')

        button_open_dvd = self.get_widget('button_open_dvd')
        button_add_dvd = self.get_widget('button_add_dvd')

        dev_type = self.__get_dev_type(widget)
        if dev_type == DEVICE_DVD or dev_type == DEVICE_VCD:
            button_open_dvd.set_sensitive(False)
            button_add_dvd.set_sensitive(True)
        else:
            button_open_dvd.set_sensitive(True)
            button_add_dvd.set_sensitive(False)
            self.on_open_clicked(button_open_dvd)
        return True

    def on_entry_input_files_changed(self, widget, event = None):
        entry = self.get_widget('entry_input_file')
        self.__enable_row(entry, len(entry.get_text()) > 0, True)
        return True

    def on_button_input_files_clicked(self, widget, event = None):
        title = _('Select multimedia files')
        # todo: enable subtitle files too
        #filter_list = [_('Multimedia files'), _('Subtitle files'), _('All files')]
        filter_list = [_('Multimedia files'),  _('All files')]
        action = FILE_CHOOSER_ACTION_OPEN
        entry = self.get_widget('entry_input_file')
        cfg = self.get_project().get_config()

        input_file = self.__input_file
        if not input_file:
            input_file = cfg.get_last_used_path(CONTAINER_FILE)

        callback = self.__input_media_files_selected
        self.show_file_selection(input_file, title, filter_list, action, callback, entry, True)
        return True

    def on_cancel_scan_clicked(self, widget, event = None):
        if self.__command:
            self.__command.cancel()
            self.__command = None
        self.__enable_ui(True)
        return True

    def on_open_clicked(self, widget, event = None):
        title = _('Select folder containing DVD structure')
        filter_list = [_('Folders')]
        action = FILE_CHOOSER_ACTION_SELECT_FOLDER
        cfg = self.get_project().get_config()

        dev_type = DEVICE_DVD
        if widget.get_name() == 'button_open_dvd':
            combo = self.get_widget('combobox_dvd')
            dev_type = self.__get_dev_type(combo)
        else:
            # todo
            return False

        if dev_type == DEVICE_DVD_IMAGE:
            title = _('Select DVD Image File')
            filter_list = [_('Image files'), _('All files')]
            action = FILE_CHOOSER_ACTION_OPEN

        input_file = self.__input_file
        if not input_file:
            if dev_type == DEVICE_DVD or dev_type == DEVICE_DVD_IMAGE or dev_type == DEVICE_DVD_FOLDER:
                input_file = cfg.get_last_used_path(CONTAINER_DVD)
            else:
                input_file = cfg.get_last_used_path(CONTAINER_VCD)

        callback = self.__input_file_selected
        self.show_file_selection(input_file, title, filter_list, action, callback, combo)
        return True

    def on_rescan_devices_clicked(self, widget, event = None):
        self.__init_device_combos(True)
        return True

    def on_treeview_click(self, treeview, event):
        # ignore everything but the right mouse button
        if event.button != 3:
            return False

        selection = treeview.get_selection()
        (model, pathlist) = selection.get_selected_rows()
        if len(pathlist) > 1:
            # todo
            debug('PageSource.on_treeview_click()', 'multiple items selected: still not implemented :-)', error)
            return False

        vlist = []
        alist = []
        slist = []
        for path in pathlist:
            it = model.get_iter(path)
            cont = model.get_value(it, TREE_COLUMN_OBJECT)

            if len(cont) < 1:
                debug('PageSource.on_treeview_click()', 'no titles in this container', error)
                return False

            # show properties for the longest title
            longest_title = cont[0]
            if len(cont) > 1:
                max_len = 0.
                for title in cont:
                    if title.get_length() > max_len:
                        max_len = title.get_length()
                        longest_title = title

            vlist.extend(longest_title.get_video())
            alist.extend(longest_title.get_audio())
            slist.extend(longest_title.get_subtitles())

        if len(vlist) > 0 or len(alist) > 0 or len(slist) > 0:
            self.show_stream_properties(vlist, alist, slist)
        return False

    def on_treeview_key(self, treeview, event):
        return True # don't change

    # -- helpers --

    def __init_treeview(self):
        treeview = self.get_widget('treeview1')
        model = treeview.get_model()
        if not model:
            self.__create_treeview()
        else:
            model.clear()
        return

    def __create_treeview(self):
        model = ListStore(TYPE_STRING, TYPE_STRING, TYPE_PYOBJECT)

        treeview = self.get_widget('treeview1')
        treeview.set_model(model)
        treeview.set_rules_hint(True)

        renderer = CellRendererText()
        renderer.set_property('editable', True)
        renderer.connect('edited', self.on_edit_title, model)
        column = TreeViewColumn(_('Name'), renderer, text = TREE_COLUMN_TITLE)
        treeview.append_column(column)

        renderer = CellRendererText()
        column = TreeViewColumn(_('Description'), renderer, text = TREE_COLUMN_DESCR)
        treeview.append_column(column)
        return

    def __add_container(self, cont):
        treeview = self.get_widget('treeview1')
        model = treeview.get_model()

        iterator = model.append()
        model.set(iterator, TREE_COLUMN_TITLE, cont.get_name(), TREE_COLUMN_DESCR, self.__get_descr(cont), TREE_COLUMN_OBJECT, cont)

        selection = treeview.get_selection()
        selection.select_path(model.get_path(iterator))
        return

    def __get_descr(self, cont):
        result = ''

        if cont.get_type() == CONTAINER_DVD:
            result = self.__get_descr_dvd(cont)

        elif cont.get_type() == CONTAINER_FILE:
            result = self.__get_descr_file(cont)

        else:
            print cont.get_type()

        return result

    def __get_descr_dvd(self, cont):
        if len(cont) < 1:
            debug('PageSource.__get_descr_dvd()', 'no titles in this container', error)
            return _('Error: No titles in this container')

        result = _('DVD: ')

        if len(cont) > 1:
            result += (_('%d titles (Longest title: ') % len(cont))
        else:
            result += _('1 title (')

        result += self.__get_cont_descr(cont)
        result += _(')')
        return result

    def __get_descr_file(self, cont):
        if len(cont) < 1:
            debug('PageSource.__get_descr_file()', 'no titles in this container', error)
            return _('Error: No titles in this container')

        result = _('Multimedia file: ')
        result += self.__get_cont_descr(cont)

        return result

    def __get_cont_descr(self, cont):
        result = ''

        longest_title = cont[0]
        max_len = 0.
        for title in cont:
            if title.get_length() > max_len:
                longest_title = title
                max_len = title.get_length()

        vstreams = longest_title.get_video()
        if len(vstreams) == 1:
            result += _('1 video stream')
        elif len(vstreams) > 1:
            result += (_('%d video streams') % len(vstreams))

        astreams = longest_title.get_audio()
        if len(astreams) == 1:
            result += _(', 1 audio stream')
        elif len(astreams) > 1:
            result += (_(', %d audio streams') % len(astreams))

        sstreams = longest_title.get_subtitles()
        if len(sstreams) == 1:
            result += _(', 1 subtitle')
        elif len(sstreams) > 1:
            result += (_(', %d subtitles') % len(sstreams))

        return result

    def __input_file_selected(self, fname, combo):
        if not fname or not len(fname):
            return

        fname = escape_name(fname)
        self.__set_dev(combo, fname)
        self.__enable_row(combo, True, True)
        self.__input_file = fname

        cfg = self.get_project().get_config()
        if combo.get_name() == 'combobox_dvd':
            cfg.set_last_used_path(CONTAINER_DVD, fname)
        else:
            cfg.set_last_used_path(CONTAINER_VCD, fname)
        cfg.write()

        if combo.get_name() == 'combobox_dvd':
            button_file = self.get_widget('button_add_dvd')
            self.on_button_add_clicked(button_file)
        return

    def __input_media_files_selected(self, fnames, entry):
        if not fnames or not len(fnames):
            return

        self.__enable_row(entry, True, True)

        result = ''
        if len(fnames) == 1:
            result = escape_name(fnames[0])
        else:
            for fname in fnames:
                result += '%s ' % set_quotes(escape_name(fname))

        entry.set_text(result)
        self.__input_file = result

        cfg = self.get_project().get_config()
        cfg.set_last_used_path(CONTAINER_FILE, fnames[0])
        cfg.write()

        button_file = self.get_widget('button_add_file')
        self.on_button_add_clicked(button_file)
        return

    def __get_file_name(self):
        entry = self.get_widget('entry_input_file')
        path = normalize_path(entry.get_text())
        return escape_name(path)

    def __set_file_name(self, file_name):
        entry = self.get_widget('entry_input_file')
        entry.set_text(file_name)
        return

    def __check_tools(self):
        tchecker = self.get_project().get_tool_checker()
        has_mencoder = tchecker.has_tool('mencoder')
        has_mplayer = tchecker.has_tool('mplayer')
        has_lsdvd = tchecker.has_tool('lsdvd')

        button_dvd = self.get_widget('button_add_dvd')
        button_vcd = self.get_widget('button_add_vcd')
        button_file = self.get_widget('button_add_file')
        combo_dvd = self.get_widget('combobox_dvd')
        combo_vcd = self.get_widget('combobox_vcd')

        if not has_mencoder:
            button_dvd.set_sensitive(False)
            button_vcd.set_sensitive(False)
            button_file.set_sensitive(False)
            combo_dvd.set_sensitive(False)
            combo_vcd.set_sensitive(False)
            self.alert_missing_tool('mencoder')
            return False

        if not has_mplayer:
            button_dvd.set_sensitive(False)
            button_vcd.set_sensitive(False)
            button_file.set_sensitive(False)
            combo_dvd.set_sensitive(False)
            combo_vcd.set_sensitive(False)
            self.alert_missing_tool('mplayer')
            return False

        if not has_lsdvd:
            button_dvd.set_sensitive(False)
            combo_dvd.set_sensitive(False)
            self.alert_missing_tool('lsdvd')
            return False

        return True

    def __enable_ui(self, enable):
        button_add_dvd = self.get_widget('button_add_dvd')
        button_add_vcd = self.get_widget('button_add_vcd')
        button_add_file = self.get_widget('button_add_file')
        entry = self.get_widget('entry_input_file')
        button_open_dvd = self.get_widget('button_open_dvd')
        button_open_vcd = self.get_widget('button_open_vcd')
        button_open_file = self.get_widget('button_input_file')
        label_scan = self.get_widget('label_scan')
        progress = self.get_widget('progressbar_scan')
        button_scan = self.get_widget('button_cancel_scan')
        button_rescan_dev = self.get_widget('button_rescan')
        button_remove = self.get_widget('button_remove_media')
        combo_dvd = self.get_widget('combobox_dvd')
        combo_vcd = self.get_widget('combobox_vcd')

        button_add_dvd.set_sensitive(True)
        combo_dvd.set_sensitive(True)
        button_open_dvd.set_sensitive(False)

        button_add_vcd.set_sensitive(False)
        combo_vcd.set_sensitive(False)
        button_open_vcd.set_sensitive(False)

        button_add_file.set_sensitive(len(entry.get_text()) > 0)
        button_open_file.set_sensitive(True)
        entry.set_sensitive(True)

        button_remove.set_sensitive(False)

        label_scan.set_sensitive(not enable)
        button_scan.set_sensitive(not enable)
        button_rescan_dev.set_sensitive(enable)
        if enable:
            progress.set_fraction(0.0)
        progress.set_sensitive(not enable)
        return

    def __enable_row(self, widget, enable_add, enable_select):
        button_add = self.get_widget('button_add_dvd')
        button_open = self.get_widget('button_open_dvd')

        if widget.get_name() == 'combobox_vcd':
            button_add = self.get_widget('button_add_vcd')
            button_open = self.get_widget('button_open_vcd')
        elif widget.get_name() == 'entry_input_file':
            button_add = self.get_widget('button_add_file')
            button_open = self.get_widget('button_input_file')

        if enable_add:
            button_add.set_sensitive(True)
        else:
            button_add.set_sensitive(False)

        if enable_select:
            button_open.set_sensitive(True)
        else:
            button_open.set_sensitive(False)
        return

    def __update_button_remove(self):
        button = self.get_widget('button_remove_media')

        if self.__get_container_count() < 1:
            button.set_sensitive(False)
        else:
            button.set_sensitive(True)
        return

    def __update_stoplight(self):
        if self.__get_container_count() < 1:
            self.set_stoplight('stop')
        else:
            self.set_stoplight('go')
        return

    def __check_filename(self, path):
        primary = _('Could not access file')
        if not path or not len(path):
            debug('PageSource.__check_filename()', 'no file selected', error)
            secondary = _('You have to select a multimedia file to be used as a source. Type the full path to the file into the edit field or use the "Open" button to locate it.')
            self.show_error(primary, secondary)
            return False

        if not is_file(path):
            debug('PageSource.__check_filename()', 'not a file', error)
            secondary = _('%s is not a file. You have to select a file which contains at least one video stream.') % path
            self.show_error(primary, secondary)
            return False

        if not is_readable(path):
            debug('PageSource.__check_filename()', 'file not readable by user', error)
            secondary = _('%s is not readable by you. You should select a file for which you have sufficient access permissions.') % path
            self.show_error(primary, secondary)
            return False

        return True

    def __toggle_stoplight(self):
        entry = self.get_widget('entry_input_file')

        fname = entry.get_text()
        if len(fname) > 0 and is_file(fname):
            self.set_stoplight('go')
        else:
            self.set_stoplight('stop')
        return

    def __get_container_count(self):
        treeview = self.get_widget('treeview1')
        model = treeview.get_model()
        row_count = 0
        for row in model:
            row_count += 1

        return row_count

    def __scan_media(self, mtype, scan_template):
        self.__enable_ui(False)
        self.__show_errors = True
        self.__canceled = False

        label_scan = self.get_widget('label_scan')
        label_scan.set_text(scan_template % '')

        if mtype == 'dvd':
            if not self.__scan_dvd():
                label_scan.set_text('')
                return False

        elif mtype == 'vcd':
            if not self.__scan_vcd():
                return False

        elif mtype == 'file':
            if not self.__scan_files(label_scan, scan_template):
                return False

        else:
            debug('unknown source', error)
            assert(False)

        label_scan.set_text('')

        self.__command = None
        self.__enable_ui(True)
        return True

    def __has_filename(self, dev_type):
        if dev_type == DEVICE_DVD_IMAGE and not self.__input_file:
            primary = _('No file selected')
            secondary = _('You have to select a file which contains the source DVD image')
            self.show_error(primary, secondary)
            return False

        elif dev_type == DEVICE_DVD_FOLDER and not self.__input_file:
            primary = _('No folder selected')
            secondary = _('You have to select a folder which contains the source DVD structure')
            self.show_error(primary, secondary)
            return False

        return True

    def __scan_dvd(self):
        combo_dvd = self.get_widget('combobox_dvd')
        dev_type = self.__get_dev_type(combo_dvd)
        device = self.__get_dev(combo_dvd)

        if not self.__has_filename(dev_type):
            return False

        iso_codes = self.get_project().get_iso_codes()
        tchecker = self.get_project().get_tool_checker()

        result = Container()
        result.set_id(0) # will be fixed later by update_project()
        self.__command = ScanDVD(iso_codes, result, tchecker)
        self.__command.set_device(device)
        self.__command.set_progress_callback(self.__show_progress)

        if not self.__command.run():
            self.__enable_ui(True)
            self.__command = None
            return False

        if self.__command and not self.__command.is_canceled():
            result.set_type(CONTAINER_DVD)
            result.set_device(device)
            self.__add_container(result)
        return True

    def __scan_vcd(self):
        return True

    def __scan_files(self, label_scan, scan_template):
        entry = self.get_widget('entry_input_file')
        fnames = self.__split_file_list(entry.get_text())

        if not len(fnames):
            debug('PageSource.__scan_files()', 'No files specified', error)
            return False
        
        result = True
        for fname in fnames:
            d, f, e = split_path(fname)
            label_scan.set_text(scan_template % set_extension(f, e))

            if not self.__check_filename(fname):
                debug('PageSource.__scan_files()', 'invalid input file selected (%s)' % fname, error)
                result = False
            else:
                if not self.__scan_file(fname):
                    result = False

        if not result:
            self.__enable_ui(True)
            entry.grab_focus()
            label_scan.set_text('')

        return result

    def __scan_file(self, fname):
        iso_codes = self.get_project().get_iso_codes()
        tchecker = self.get_project().get_tool_checker()

        result = Container()
        result.set_id(0) # will be fixed later by update_project()
        d, f, e = split_path(fname)
        result.set_name(f)

        self.__command = ScanFile(iso_codes, result, tchecker)
        self.__command.set_device(fname)
        self.__command.set_progress_callback(self.__show_progress)

        if not self.__command.run():
            self.__command = None
            return False

        if self.__command and not self.__command.is_canceled():
            result.set_type(CONTAINER_FILE)
            result.set_device(fname)
            self.__add_container(result)

        #print result
        return True

    def __show_progress(self, prog):
        prog_bar = self.get_widget('progressbar_scan')
        prog_bar.set_fraction(prog)
        return

    def __init_device_list(self, force_rescan):
        tool_checker = self.get_project().get_tool_checker()
        scan = ScanDevices(tool_checker)
        cfg = self.get_project().get_config()

        from_config = scan.get_from_config(cfg)
        if not from_config or force_rescan:
            scan.run()

        dev_list = scan.get_device_list()

        if not from_config or force_rescan:
            cfg.set_cdrom_devices(dev_list)
            cfg.set_dvdrom_devices(dev_list)
            cfg.write()

        return dev_list

    def __init_device_combos(self, force_rescan):
        device_list = self.__init_device_list(force_rescan)

        combo_dvd_devices = self.get_widget('combobox_dvd')
        self.__init_combo(combo_dvd_devices, device_list, True)

        combo_vcd_devices = self.get_widget('combobox_vcd')
        self.__init_combo(combo_vcd_devices, device_list, False)
        return

    def __init_combo(self, combo, device_list, dvd_only):
        model = combo.get_model()
        if not model:
            model = ListStore(TYPE_STRING, TYPE_PYOBJECT, TYPE_PYOBJECT)
            combo.set_model(model)

            text_renderer = CellRendererText()
            text_renderer.set_property('xalign', 0)
            combo.pack_start(text_renderer, True)
            combo.add_attribute(text_renderer, 'text', COMBO_COLUMN_TEXT)
        else:
            model.clear()

        for device in device_list:
            text = '%s %s %s' % (device.get_manufacturer(), device.get_model(), device.get_version())
            if dvd_only:
                if device.is_dvd_reader():
                    model.append([text, device, DEVICE_DVD])
            else:
                model.append([text, device, DEVICE_VCD])

        if dvd_only:
            model.append([_('DVD Image File'), None, DEVICE_DVD_IMAGE])
            model.append([_('DVD Folder'), None, DEVICE_DVD_FOLDER])
        else:
            model.append([_('VCD Image File'), None, DEVICE_VCD_IMAGE])
            model.append([_('VCD Folder'), None, DEVICE_VCD_FOLDER])

        combo.set_active(0)
        return

    def __split_file_list(self, text):
        text = text.strip()
        if not text or not len(text):
            return []

        if text[0] == '"' and text[len(text) - 1] == '"':
            r = re.compile('".*?"', re.LOCALE)
            fnames = r.findall(text)
            return map(lambda x: remove_quotes(x), fnames)

        return [text]

    def __get_dev_type(self, combo):
        model = combo.get_model()
        active = combo.get_active()
        if active < 0:
            return DEVICE_UNKNOWN

        return model[active][COMBO_COLUMN_TYPE]

    def __get_dev(self, combo):
        model = combo.get_model()
        active = combo.get_active()
        if active < 0:
            return DEVICE_UNKNOWN

        return model[active][COMBO_COLUMN_DEVICE]

    def __set_dev(self, combo, value):
        model = combo.get_model()
        active = combo.get_active()
        if active < 0:
            return False

        model[active][COMBO_COLUMN_DEVICE] = value
        return True

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
