# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import events_pending, main_iteration
from string import capwords
from winkirip.container_title import Title
from winkirip.container_video import Video, VIDEO_FMT_PAL, VIDEO_FMT_NTSC
from winkirip.container_video import VIDEO_FMT_UNKNOWN, VIDEO_MPEG2
from winkirip.container_audio import Audio, AUDIO_AC3, AUDIO_PCM, AUDIO_MP2
from winkirip.container_audio import AUDIO_UNKNOWN, AUDIO_MP3, AUDIO_DTS
from winkirip.container_subtitle import Subtitle, SUBTITLE_VOBSUB
from winkirip.container_chapter import Chapter
from winkirip.lib import debug, error, hex2dec


class LsDVDParser:
    """This class parses the output produced by the command line tool "lsdvd"
    and creates a container object.

       LsDVDParser takes as input the output of 'lsdvd -x' and creates the
       appropriated data structures which describe this DVD disc. The result is
       placed into the self.__container object.

       The only public interface functions are:
        - run(): initiates the parsing process
        - get_container(): access to the data member object
    """

    def __init__(self, lines, iso_codes, cont):
        self.__lines = lines
        self.__iso_codes = iso_codes
        self.__index = 0
        self.__cur_line = ''
        self.__container = cont
        self.__title_id  = -1

        self.__dispatch = {'Disc Title' : self.__disc_title_found,
                         'Title' : self.__title_found,
                         'VTS' : self.__video_found,
                         'Palette' : self.__palette_found,
                         'Number of Angles' : self.__angle_count_found,
                         'Audio' : self.__audio_found,
                         'Chapter' : self.__chapter_found,
                         'Cell' : self.__cell_found,
                         'Subtitle' : self.__subtitle_found,
                         'Longest track' : self.__longest_track_found,
                         'libdvdread' : self.__dummy}

        self.__followup = {'Disc Title' : ['Disc Title', 'Title', 'Longest track'],
                          'Title' : ['Title', 'VTS', 'Number of Angles', 'Audio', 'Chapter', 'Cell', 'Subtitle', 'Longest track'],
                          'VTS' : ['Palette', 'Number of Angles', 'Audio', 'Chapter', 'Cell', 'Subtitle', 'Longest track'],
                          'Palette' : ['Number of Angles', 'Audio', 'Chapter', 'Cell', 'Subtitle', 'Longest track'],
                          'Number of Angles' : ['Audio', 'Chapter', 'Cell', 'Subtitle', 'Longest track'],
                          'Audio' : ['Audio', 'Chapter', 'Cell', 'Subtitle', 'Longest track'],
                          'Chapter' : ['Chapter', 'Cell', 'Subtitle', 'Longest track'],
                          'Cell' : ['Cell', 'Subtitle', 'Longest track', 'Title'],
                          'Subtitle' : ['Subtitle', 'Longest track'],
                          'Longest track' : []}
        return

    # -- public interface --

    def run(self):
        """Initiates the parsing process. Builds the DVD container representation.
        """

        token = self.__get_look_ahead()
        old_token = token
        while token:
            while events_pending():
                main_iteration(False)
            new_token = self.__dispatch_token(token, old_token)
            old_token = token
            token = new_token

        return

    def get_container(self):
        """Access to the DVD container representation.
        """

        return self.__container

    # -- helpers --

    def __can_follow(self, token, parent):
        """Decides if the token "token" is allowed to follow the token "parent".
        """
        allowed = []
        try:
            allowed = self.__followup[parent]
        except KeyError:
            debug('LsDVDParser.__can_follow()', 'key error: %s' % parent, error)
            pass

        for item in allowed:
            if item == token:
                return True

        return False

    def __dispatch_token(self, token, old_token):
        """Finds and calls the appropriate hanlder for "token".
        """

        #print 'dispatching [%s] after [%s]' % (token, old_token)
        if not self.__can_follow(token, old_token):
            #print '[%s] cannot folllow [%s]' % (token, old_token)
            return None

        func = None
        try:
            func = self.__dispatch[token]
        except KeyError:
            debug('LsDVDParser.__dispatch_token()', 'key error %s' % token, error)
            pass

        if not func:
            debug('LsDVDParser.__dispatch_token()', 'no func for %s -> %s' % (old_token, token), error)
            return

        self.__get_token()
        func()
        return self.__get_look_ahead()

    def __get_line(self):
        """Returns the next line from the "lsdvd -x" output.
        """
        if self.__index < len(self.__lines):
            return self.__cur_line
        return None

    def __get_token(self):
        """Returns the next token from the input stream (the "lsdvd -x" output).

            The token is just the line descriptor (e.g. chapter, title, etc.).
        """
        cur_line = ''
        pos = -1
        while (not cur_line or not len(cur_line)) and self.__index < len(self.__lines):
            cur_line = self.__lines[self.__index].strip()
            self.__index += 1

            # skip input lines which does not contain ':'
            pos = cur_line.find(':')
            if pos < 0:
                cur_line = ''

        # end of the (known) input reached
        if pos < 0:
            #assert(self.__index == len(self.__lines))
            if self.__index != len(self.__lines):
                #debug('LsDVDParser.__get_token()', 'end of input [%s] (pos = %d)' % (cur_line, pos), error)
                debug('LsDVDParser.__get_token()', 'index (%d) != len(lines) (%d)' % (self.__index, len(self.__lines)), error)
            return None

        self.__cur_line = cur_line
        return cur_line[ : pos]

    def __get_look_ahead(self):
        """The same as __get_token() but does not consume the token.
        """
        cur_line = ''
        index = self.__index
        pos = -1
        while (not cur_line or not len(cur_line)) and index < len(self.__lines):
            cur_line = self.__lines[index].strip()
            index += 1

            # skip input lines which does not contain ':'
            pos = cur_line.find(':')
            if pos < 0:
                #print 'skipping [%s] (pos = %d)' % (cur_line, pos)
                cur_line = ''

        # end of the (known) input reached
        if pos < 0:
            #assert(self.__index == len(self.__lines))
            if self.__index != len(self.__lines):
                #debug('LsDVDParser.__get_look_ahead()', 'end of input [%s] (pos = %d)' % (cur_line, pos), error)
                debug('LsDVDParser.__get_look_ahead()', 'index (%d) != len(lines) (%d)' % (self.__index, len(self.__lines)), error)
            return None

        return cur_line[ : pos]

    def __dummy(self):
        return

    def __disc_title_found(self):
        """Handler for lines starting with "Disc Title".
        """
        line = self.__get_line()
        items = line.split(':')
        assert(len(items) == 2)

        self.__container.set_name(self.__user_friendly_name(items[1]))

        self.__dispatch_token(self.__get_look_ahead(), 'Disc Title')
        return

    def __title_found(self):
        """Handler for lines starting with "Title".
        """
        self.__create_title()
        self.__dispatch_token(self.__get_look_ahead(), 'Title')
        return

    def __video_found(self):
        """Handler for lines starting with "VTS".
        """
        self.__create_video()
        self.__dispatch_token(self.__get_look_ahead(), 'VTS')
        return

    def __palette_found(self):
        """Handler for lines starting with "Palette".
        """
        self.__create_palette()
        self.__dispatch_token(self.__get_look_ahead(), 'Palette')
        return
        
    def __angle_count_found(self):
        """Handler for lines starting with "Number of Angles".
           
           This information is ignored.
        """
        self.__append_angles()
        self.__dispatch_token(self.__get_look_ahead(), 'Number of Angles')
        return
        
    def __audio_found(self):
        """Handler for lines starting with "Audio".
        """
        self.__create_audio()
        self.__dispatch_token(self.__get_look_ahead(), 'Audio')
        return
        
    def __chapter_found(self):
        """Handler for lines starting with "Chpater".
        """
        self.__modify_chapter()
        self.__dispatch_token(self.__get_look_ahead(), 'Chapter')
        return
        
    def __cell_found(self):
        """Handler for lines starting with "Cell".

           This information is ignored.
        """
        self.__dispatch_token(self.__get_look_ahead(), 'Cell')
        return
        
    def __subtitle_found(self):
        """Handler for lines starting with "Subtitle".
        """
        self.__create_subtitle()
        self.__dispatch_token(self.__get_look_ahead(), 'Subtitle')
        return

    def __longest_track_found(self):
        """Handler for lines starting with "Longest track".
        """
        items = self.__split_line(self.__cur_line)
        assert(len(items) == 1 and len(items[0]) == 2)
        #title = self.__container[int(items[0][1]) - 1]
        #title.set_enabled(True)
        
        self.__dispatch_token(self.__get_look_ahead(), 'Longest track')
        return

    def __workaround_sloppy_lsdvd_format(self, line, result):
        """Works around inconsistent lsdvd output.

           lsdvd separates the single tokens on one line with commas.
           But it does not separate "Length" from "Chapters" with comma in the
           "Title" line.
        """
        ch_start = line.find('Chapters')
        key, value = line[ch_start : ].split(':')
        result.append([key.strip(), value.strip()])

        reminder = line[ : ch_start]
        delim = reminder.find(':')
        key = reminder[ : delim]
        value = reminder[delim  + 1: ]
        result.append([key.strip(), value.strip()])
        return

    def __split_line(self, line):
        """Splits the line into key=value pairs.
        """
        result = []
        items = line.split(',')
        for item in items:
            pos = item.find(':')
            key = item[ : pos].strip()
            value = item[pos + 1 : ].strip()

            if key == 'Length' and value.find('Chapters') > -1:
                self.__workaround_sloppy_lsdvd_format(item, result)
                continue
                
            result.append([key, value])

        return result

    def __create_title(self):
        """Builds the appropriate container object.
        """
        line = self.__get_line()
        items = self.__split_line(line)
        assert(len(items) == 6)

        field = {'title' : 0, 'chapters' : 1, 'length' : 2, 'cells' : 3, 'audio streams' : 4, 'subpictures' : 5}
        mean = {'key' : 0, 'value' : 1}

        title_id = int(items[field['title']][mean['value']])
        #chapter_count = int(items[field['chapters']][mean['value']])

        title = Title()
        self.__container.add_title(title)
        title.set_id(title_id)
        return

    def __create_video(self):
        """Builds the appropriate container object.
        """
        line = self.__get_line()
        items = self.__split_line(line)
        assert(len(items) == 8)

        field = {'vts' : 0, 'ttn' : 1, 'fps' : 2, 'format' : 3, 'aspect' : 4, 'width' : 5, 'height' : 6, 'df' : 7}
        mean = {'key' : 0, 'value' : 1}

        #kind = int(items[field['vts']][mean['value']])
        vid = int(items[field['ttn']][mean['value']])
        fps = float(items[field['fps']][mean['value']])
        format = items[field['format']][mean['value']]
        aspect = items[field['aspect']][mean['value']]
        width = int(items[field['width']][mean['value']])
        height = int(items[field['height']][mean['value']])

        video = Video()
        video.set_id(vid)
        video.set_type(VIDEO_MPEG2) # todo
        video.set_format(format)
        video.set_aspect(aspect)
        video.set_fps(fps)
        video.set_width(width)
        video.set_height(height)
        video.set_format(self.__get_video_format(format))
        self.__container.add_video(video)
        return

    def __create_palette(self):
        self.__get_line()
        return
# lsdvd does not deliver credible palette information...
#       items = self.__split_line(line)
#       assert(len(items) == 1)
#
#       palette = items[0][1].split()
#       assert(len(palette) == 16)
#
#       for i in range(len(palette)):
#           if len(palette[i]) > 6:
#               print 'truncating %s to %s - todo: is this a bug?' % (palette[i], palette[i][ : 6])
#               palette[i] = palette[i][ : 6]
#
#       palette.reverse()
#
#       self.__container.set_palette(palette)
#       return

    def __append_angles(self):
        """Builds the appropriate container object.
        """
        # just consume the line
        self.__get_line()

        # lsdvd and mplayer seem to have different opinions about the angle count...

        #items = self.__split_line(line)
        #assert(len(items) == 1)
        #field = {'count' : 0}
        #mean = {'key' : 0, 'value' : 1}
        #count = int(items[field['count']][mean['value']])
        return

    def __create_audio(self):
        """Builds the appropriate container object.
        """
        line = self.__get_line()
        items = self.__split_line(line)
        assert(len(items) == 9)

        field = {'audio' : 0, 'lang' : 1, 'format' : 2, 'freq' : 3, 'quant' : 4, 'channels' : 5, 'ap' : 6, 'content' : 7, 'id' : 8}
        mean = {'key' : 0, 'value' : 1}

        language = items[field['lang']][mean['value']]
        format = items[field['format']][mean['value']]
        frequency = items[field['freq']][mean['value']]
        channels = int(items[field['channels']][mean['value']])
        stream_id = items[field['id']][mean['value']]

        aformat = self.__get_audio_format(format)
        if aformat == AUDIO_UNKNOWN:
            return

        audio = Audio()
        audio.set_id(hex2dec(stream_id))
        audio.set_language(self.__get_language_name(language))
        audio.set_format(aformat)
        audio.set_frequency(frequency)
        audio.set_number_channels(channels)
        self.__container.add_audio(audio)
        return

    def __create_subtitle(self):
        """Builds the appropriate container object.
        """
        line = self.__get_line()
        items = self.__split_line(line)
        assert(len(items) == 5) # bug in lsdvd (the line ends with comma)

        field = {'subtitle' : 0, 'lang' : 1, 'content' : 2, 'id' : 3}
        mean = {'key' : 0, 'value' : 1}

        sid = int(items[field['subtitle']][mean['value']])
        language = items[field['lang']][mean['value']]

        subtitle = Subtitle()
        subtitle.set_id(sid - 1)
        subtitle.set_language(self.__get_language_name(language))
        # todo: are other formats possible?
        subtitle.set_format(SUBTITLE_VOBSUB)
        self.__container.add_subtitle(subtitle)
        return

    def __modify_chapter(self):
        """Sets the audio and video length for a specified chapter.
        """
        line = self.__get_line()
        items = self.__split_line(line)
        assert(len(items) == 3)

        field = {'chapter' : 0, 'length' : 1, 'start cell' : 2}
        mean = {'key' : 0, 'value' : 1}

        cid = int(items[field['chapter']][mean['value']])
        length = items[field['length']][mean['value']]
        seconds = self.__to_seconds(length)

        chapter = Chapter()
        chapter.set_length(seconds)
        chapter.set_id(cid)
        self.__container.add_chapter(chapter)
        return

    def __to_seconds(self, time):
        """Converts time string formatted as "hh:mm:ss" to time in seconds
        """
        hours, minutes, seconds = time.split(':')
        sec, ms = seconds.split('.')
        return int(hours) * 3600 + int(minutes) * 60 + int(sec)

    def __get_language_name(self, lang):
        pos = lang.find(' - ')
        short_name = lang[ : pos]

        # make sure the language text is valid
        lobj = self.__iso_codes.find_iso_639_1(short_name.strip())
        return lobj.get_iso_639_1()

    def __get_video_format(self, format):
        if format == 'PAL':
            return VIDEO_FMT_PAL

        elif format == 'NTSC':
            return VIDEO_FMT_NTSC

        return VIDEO_FMT_UNKNOWN

    def __get_audio_format(self, fmt):
        if fmt == 'ac3':
            return AUDIO_AC3

        elif fmt == 'lpcm':
            return AUDIO_PCM

        elif fmt == 'mp2':
            return AUDIO_MP2

        # todo: what's this? mp2? mp3?
        elif fmt == 'mpeg1':
            return AUDIO_MP3

        elif fmt == 'dts':
            return AUDIO_DTS

        debug('LsDVDParser.__get_audio_format()', 'Unexpected format : %s' % fmt, error)
        return AUDIO_UNKNOWN

    def __user_friendly_name(self, text):
        spaces = text.replace('_', ' ')
        capital = capwords(spaces)
        return capital.strip()

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
