# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from winkirip.lib import debug, error, to_int, to_float
from winkirip.container_title import Title
from winkirip.container_audio import Audio, AUDIO_PCM, AUDIO_AC3, AUDIO_OGG
from winkirip.container_audio import AUDIO_MP2, AUDIO_MP3, AUDIO_FLAC, AUDIO_QTIME
from winkirip.container_audio import AUDIO_WMA, AUDIO_AAC, AUDIO_UNKNOWN
from winkirip.container_video import Video, VIDEO_MPEG1, VIDEO_MPEG2
from winkirip.container_video import VIDEO_MPEG4, VIDEO_XVID, VIDEO_THEORA
from winkirip.container_video import VIDEO_REAL, VIDEO_DIVX, VIDEO_WMV, VIDEO_QTIME
from winkirip.container_video import VIDEO_H264, VIDEO_FLASH, VIDEO_UNKNOWN
from winkirip.container_chapter import Chapter
import re


class FileParser:
    """Generic parser for mplayer output.

    It assumes that we deal with a simple file containing one video and one
    audio stream only.

    For that matter it creates one title in the container with one video and
    one audio objects if appropriate.
    """


    def __init__(self, lines, iso_codes,  cont):
        self.__lines = lines
        self.__container = cont
        self.__iso_codes = iso_codes
        self.__len = -1
        self.__additional = None
        self.__keys = {'ID_AUDIO_BITRATE=' : self.__audio_bitrate,
                       'ID_AUDIO_RATE=' : self.__audio_frequency,
                       'ID_AUDIO_NCH=' : self.__audio_nch,
                       'ID_AUDIO_FORMAT=' : self.__audio_format,
                       'ID_AUDIO_CODEC=' : self.__audio_codec,
                       'ID_AUDIO_ID=' : self.__audio_id,
                       'ID_LENGTH=' : self.__length,
                       'ID_VIDEO_ID=' : self.__video_id,
                       'ID_VIDEO_FORMAT=' : self.__video_format,
                       'ID_VIDEO_BITRATE=' : self.__video_bitrate,
                       'ID_VIDEO_WIDTH=' : self.__video_width,
                       'ID_VIDEO_HEIGHT=' : self.__video_height,
                       'ID_VIDEO_FPS=' : self.__video_fps,
                       'ID_VIDEO_ASPECT=' : self.__video_aspect,
                       'ID_VIDEO_CODEC=' : self.__video_codec,
                       'VIDEO: ' : self.__video_supplement}
        return

    # -- public interface --

    def run(self):
        self.__container.remove_titles()
        title = Title()
        self.__container.add_title(title)

        result = True

        for line in self.__lines:
            line = line.replace('\n', '').strip()

            for key in self.__keys:
                if line.find(key) > -1:
                    try:
                        result &= self.__keys[key](line, key)
                    except KeyError:
                        debug('FileParser.run()', 'undefined function for key %s' % key, error)

        # add one chapter and apply length to the available streams
        chapter = Chapter()
        chapter.set_id(0)
        chapter.set_length(self.__len)
        title.add_chapter(chapter)

        if self.__len > -1:
            if len(self.__container[0].get_audio()) > 0:
                audio = self.__container[0].get_audio_at(0)
                audio.set_length(self.__len)

            if len(self.__container[0].get_video()) > 0:
                video = self.__container[0].get_video_at(0)
                video.set_length(self.__len)

        # supplement the video information
        self.__apply_additional_video_info()
        return result

    def get_container(self):
        """Access to the file container representation.
        """

        return self.__container

    # -- helpers --

    def __get_audio(self):
        if len(self.__container[0].get_audio()) == 0:
            audio = Audio()
            self.__container[0].add_audio(audio)

        return self.__container[0].get_audio_at(0)

    def __get_video(self):
        if len(self.__container[0].get_video()) == 0:
            video = Video()
            self.__container[0].add_video(video)

        return self.__container[0].get_video_at(0)

    def __audio_bitrate(self, line, key):
        brate = line[len(key) : ]
        bitrate = to_int(brate)
        if bitrate < 0:
            debug('FileParser.__audio_bitrate()', 'failed conversion for [%s]' % brate, error)
            return False

        audio = self.__get_audio()
        audio.set_kbits(to_int(bitrate / 1000.0))
        return True

    def __audio_frequency(self, line, key):
        freq = line[len(key) : ]
        frequency = to_int(freq)
        if frequency < 0:
            debug('FileParser.__audio_frequency()', 'failed conversion for [%s]' % freq, error)
            return False

        audio = self.__get_audio()
        audio.set_frequency(frequency)
        return True

    def __audio_nch(self, line, key):
        nch = line[len(key) : ]
        nr_chan = to_int(nch)
        if nr_chan < 0:
            debug('FileParser.__audio_nch()', 'failed conversion for [%s]' % nch, error)
            return False

        audio = self.__get_audio()
        audio.set_number_channels(nr_chan)
        return True

    def __audio_format(self, line, key):
        codecs = {'1' : AUDIO_PCM,
                  '2' : AUDIO_PCM,
                  '17' : AUDIO_PCM,
                  'samr' : AUDIO_PCM,
                  'ima4' : AUDIO_PCM,
                  'raw' : AUDIO_PCM,
                  'vrbs' : AUDIO_OGG,
                  '80' : AUDIO_MP3,
                  '85' : AUDIO_MP3,
                  '353' : AUDIO_WMA,
                  '10' : AUDIO_WMA,
                  '8192' : AUDIO_AC3,
                  '28781' : AUDIO_AAC,
                  'mp4a' : AUDIO_AAC,
                  'qdm1' : AUDIO_QTIME,
                  'qdm2' : AUDIO_QTIME,
                  'qdmc' : AUDIO_QTIME}

        fmt = line[len(key) : ].lower()

        audio = self.__get_audio()
        aformat = AUDIO_UNKNOWN

        try:
            aformat = codecs[fmt]
            audio.set_format(aformat)

        except KeyError:
            debug('FileParser.__audio_format()', 'unknown audio format [%s]' % line, error)

        #return (aformat != AUDIO_UNKNOWN)
        return True


    # fallback function in case __audio_format does not deliver result
    def __audio_codec(self, line, key):
        codec = line[len(key) : ].lower()
        fmt = ''

        if codec.find('vorbis') > -1:
            fmt = AUDIO_OGG
        elif codec.find('pcm') > -1:
            fmt = AUDIO_PCM
        elif codec.find('wma') > -1:
            fmt = AUDIO_WMA
        elif codec.find('mp3') > -1:
            fmt = AUDIO_MP3
        elif codec.find('a52') > -1:
            fmt = AUDIO_AC3
        else:
            debug('FileParser.__audio_codec()', 'ignoring unknown audio codec [%s]' % codec)
            return True

        audio = self.__get_audio()
        afmt = audio.get_format()

        if afmt == AUDIO_UNKNOWN:
            audio.set_format(fmt)
        return True

    def __audio_id(self, line, key):
        sid = line[len(key) : ]
        aid = to_int(sid)
        if aid < 0:
            debug('FileParser.__audio_id()', 'failed conversion for [%s]' % sid, error)
            return False

        audio = self.__get_audio()
        audio.set_id(aid)
        return True

    def __length(self, line, key):
        l = line[len(key) : ]
        length = to_float(l)
        if length < 0:
            debug('FileParser.__length()', 'failed conversion for [%s]' % l, error)
            return False

        self.__len = length
        return True

    def __video_id(self, line, key):
        sid = line[len(key) : ]
        vid = to_int(sid)
        if vid < 0:
            debug('FileParser.__video_id()', 'failed conversion for [%s]' % sid, error)
            return False

        video = self.__get_video()
        video.set_id(vid)
        return True

    def __video_format(self, line, key):
        video = self.__get_video()
        codecs = {'0x10000001' : VIDEO_MPEG1,
                  '0x10000002' : VIDEO_MPEG2,
                  'xvid' : VIDEO_XVID,
                  'divx' : VIDEO_DIVX,
                  'div3' : VIDEO_DIVX,
                  'dx50' : VIDEO_DIVX,
                  'mp4v' : VIDEO_MPEG4,
                  'mp42' : VIDEO_MPEG4,
                  'mp43' : VIDEO_MPEG4,
                  's263' : VIDEO_MPEG4,
                  'h263' : VIDEO_MPEG4,
                  'mjpeg' : VIDEO_MPEG4,
                  'mjpg' : VIDEO_MPEG4,
                  'jpg' : VIDEO_MPEG4,
                  'jpeg' : VIDEO_MPEG4,
                  'mpg4' : VIDEO_MPEG4,
                  'cvid' : VIDEO_MPEG4,
                  'svq1' : VIDEO_QTIME,
                  'svq2' : VIDEO_QTIME,
                  'svq3' : VIDEO_QTIME,
                  's264' : VIDEO_H264,
                  'h264' : VIDEO_H264,
                  'avc1' : VIDEO_H264,
                  '0x10000005' : VIDEO_H264,
                  'wmv1' : VIDEO_WMV,
                  'wmv2' : VIDEO_WMV,
                  'wmv3' : VIDEO_WMV,
                  'rv40' : VIDEO_REAL,
                  'flv1' : VIDEO_FLASH}

        fmt = line[len(key) : ].lower()
        vformat = VIDEO_UNKNOWN

        try:
            vformat = codecs[fmt]
            video.set_type(vformat)

        except KeyError:
            # todo: IV50, DVSD, ...
            debug('FileParser.__video_format()', 'unknown video format [%s]' % line, error)
            print fmt

        #return (vformat != VIDEO_UNKNOWN)
        return True

    def __video_bitrate(self, line, key):
        brate = line[len(key) : ]
        bitrate = to_int(brate)
        if bitrate < 0:
            debug('FileParser.__video_bitrate()', 'failed conversion for [%s]' % brate, error)
            return False

        video = self.__get_video()
        video.set_bitrate(bitrate / 1000.0)
        return True

    def __video_width(self, line, key):
        w = line[len(key) : ]
        width = to_int(w)
        if width < 0:
            debug('FileParser.__video_width()', 'failed conversion for [%s]' % w, error)
            return False

        video = self.__get_video()
        video.set_width(width)
        return True

    def __video_height(self, line, key):
        h = line[len(key) : ]
        height = to_int(h)
        if height < 0:
            debug('FileParser.__video_height()', 'failed conversion for [%s]' % h, error)
            return False

        video = self.__get_video()
        video.set_height(height)
        return True

    def __video_fps(self, line, key):
        f = line[len(key) : ]
        fps = to_float(f)
        if fps < 0:
            debug('FileParser.__video_fps()', 'failed conversion for [%s]' % f, error)
            return False

        video = self.__get_video()
        video.set_fps(fps)
        return True

    def __video_aspect(self, line, key):
        asp = line[len(key) : ]
        aspect = to_float(asp)
        if aspect < 0:
            debug('FileParser.__video_aspect()', 'failed conversion for [%s]' % asp, error)
            return False

        # no aspect information
        if abs(aspect) < 1e-30:
            return True

        video = self.__get_video()
        video.set_aspect(aspect)
        return True

    # fallback function in case __video_format does not deliver result
    def __video_codec(self, line, key):
        codec = line[len(key) : ]
        fmt = ''

        if codec.find('divx') > -1:
            fmt = VIDEO_DIVX
        elif codec.find('mpeg') > -1:
            fmt = VIDEO_MPEG1
        elif codec.find('jpeg') > -1:
            fmt = VIDEO_MPEG4
        elif codec.find('jpg') > -1:
            fmt = VIDEO_MPEG4
        elif codec.find('vid') > -1:
            fmt = VIDEO_MPEG4
        elif codec.find('svq') > -1:
            fmt = VIDEO_QTIME
        else:
            debug('FileParser.__video_codec()', 'ignoring unknown video codec [%s]' % codec)
            return True

        video = self.__get_video()
        vfmt = video.get_type()

        if vfmt == VIDEO_UNKNOWN:
            video.set_type(fmt)

        return True

    def __video_supplement(self, line, key):
        self.__additional = line[len(key) : ].strip()
        return True

    def __apply_additional_video_info(self):
        if not self.__additional:
            return

        video = self.__get_video()
        mpeg_video = (video.get_type() == VIDEO_MPEG1 or video.get_type() == VIDEO_MPEG2)

        # try to fix the missing aspect information for mpeg1/2 streams
        if not video.get_aspect() and mpeg_video:
            r = re.compile('\(aspect (\d+)\)')
            m = r.findall(self.__additional)
            if len(m) == 1 and to_int(m[0]) > -1:
                aspect_index = to_int(m[0])
                self.__apply_aspect(aspect_index, video)

        return

    def __apply_aspect(self, idx, video):
        # stealed from a post to the mplayer mailing list (thank you again Nico Sabbi)
        lst = [0, 1, 0.6735, 0.7031, 0.7615, 0.8055, 0.8437, 0.8935, 0.9375, 0.9815, 1.0255, 1.0695, 1.1250, 1.1575, 1.2015]

        if idx < 0 or idx >= len(lst):
            return

        if abs(lst[idx]) < 1e-3 or abs(video.get_height()) < 1e-3:
            return

        ratio = to_float(video.get_width()) / (lst[idx] * video.get_height())
        video.set_aspect(ratio)
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
