# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from winkirip.lib import debug, error, to_int, to_float
from winkirip.container_title import Title
from winkirip.container_audio import Audio, AUDIO_PCM, AUDIO_AC3, AUDIO_OGG
from winkirip.container_audio import AUDIO_MP2, AUDIO_MP3, AUDIO_FLAC
from winkirip.container_audio import AUDIO_WMA, AUDIO_AAC
from winkirip.container_video import Video, VIDEO_MPEG1, VIDEO_MPEG2
from winkirip.container_video import VIDEO_MPEG4, VIDEO_XVID, VIDEO_THEORA
from winkirip.container_video import VIDEO_REAL, VIDEO_DIVX, VIDEO_WMV
from winkirip.container_video import VIDEO_H264, VIDEO_FLASH 
from winkirip.container_subtitle import Subtitle, SUBTITLE_VOBSUB
from winkirip.container_subtitle import SUBTITLE_SRT, SUBTITLE_SUB
from winkirip.container_chapter import Chapter
import re

class MatroskaParser:
    def __init__(self, lines, iso_codes, cont, file_len):
        self.__lines = lines
        self.__container = cont
        self.__iso_codes = iso_codes
        return

    # -- public interface --

    def run(self):
        self.__container.remove_titles()
        title = Title()
        self.__container.add_title(title)

        tracks, additional = self.__split_tracks()
        if len(tracks) < 1:
            debug('MatroskaParser.run()', 'no tracks found', error)
            return False

        file_len = self.__get_flen(additional)
        if file_len < 0.:
            # try to find this info in the whole mplayer output
            file_len = self.__get_mplayer_flen()
            if file_len < 0.:
                debug('MatroskaParser.run()', 'unknown file length', error)
                return False

        if not self.__create_streams(tracks, file_len):
            return False

        if not self.__update_stream_info(additional):
            return False

        # add one chapter
        chapter = Chapter()
        chapter.set_id(0)
        chapter.set_length(file_len)
        title.add_chapter(chapter)
        return True

    def get_container(self):
        """Access to the file container representation.
        """

        return self.__container

    # -- helpers --

    def __split_tracks(self):
        tracks = []
        track = {}
        additional = []
        header = True

        for line in self.__lines:
            line = line.replace('\n', '').strip()
            if line[ : 5] == '[mkv]':
                line, header_part = self.__strip_mkv_line(line[5 : ])
                if not line or len(line) < 1:
                    continue

                # track description
                if header_part:
                    if header:
                        additional.append(line)

                    # new track starts here
                    if line.lower().find('a track...') > -1:
                        if not header:
                            tracks.append(track)
                        # clear the "first" flag and start collecting lines
                        else:
                            header = False
                        track = {}
                        continue

                    items = line.split(':')
                    if len(items) == 2:
                        track[items[0]] = items[1]
                    #else:
                    #    print items

                # additional info
                else:
                    additional.append(line)
            
        tracks.append(track)
        return tracks, additional

    def __strip_mkv_line(self, line):
        header_line = False
        start_line = line.find('+')
        if start_line > -1:
            line = line[start_line + 1 : ].strip()
            header_line = True
        return line, header_line


    def __create_streams(self, tracks, file_len):
        result = True

        for track in tracks:
            if self.__is_video(track):
                result &= self.__add_video(track, file_len)

            elif self.__is_audio(track):
                result &= self.__add_audio(track, file_len)

            elif self.__is_subtitle(track):
                result &= self.__add_subtitle(track, file_len)
            
            else:
                debug('MatroskaParser.__create_streams()', 'unknown track type', error)
                result = False

        return result

    def __update_stream_info(self, additional):
        result = True
        regex = re.compile('Track\s+ID\s+(\d+):\s(.*?)\s+\((.*?)\),.*?id\s+(\d+)(.*?lang\s+(.*))*')

        for line in additional:
            if line.find('Track ID') > -1:
                result &= self.__update_stream(line.strip(), regex)

        return result

    def __update_stream(self, line, regex):
        result = False
        match = regex.match(line)
        if match:
            result = True
            tid, stype, fmt, sid, lang = match.group(1, 2, 3, 4, 6)

            if stype == 'video':
                vlist = self.__container[0].get_video()
                result &= self.__set_stream_attrib(int(tid), int(sid), fmt, lang, vlist)

            elif stype == 'audio':
                alist = self.__container[0].get_audio()
                result &= self.__set_stream_attrib(int(tid), int(sid), fmt, lang, alist)

            elif stype == 'subtitles':
                slist = self.__container[0].get_subtitles()
                result &= self.__set_stream_attrib(int(tid), int(sid), fmt, lang, slist)

            else:
                result = False
        else:
            debug('ParserMatroska.__update_stream()', 'can\'t match %s' % line, error)

        return result

    def __set_stream_attrib(self, tid, sid, fmt, lang, lst):
        for stream in lst:
            if stream.get_id() == tid:
                stream.set_id(sid)
                return True

        debug('ParserMatroska.__set_stream_attrib()', 'can\'t update stream tid %s' % tid, error)
        return False

    def __get_mplayer_flen(self):
        flen = -1.

        for line in self.__lines:
            if line.find('ID_LENGTH=') > -1:
                items = line.split('=')
                if len(items) != 2:
                    return flen

                flen = to_float(items[1])
                break

        return flen

    def __get_flen(self, lines):
        flen = -1.

        for line in lines:
            items = line.split(':')
            if len(items) == 2:
                if items[0] == 'duration':
                    flen = to_float(items[1].replace('s', ''))
                    break
        return flen

    def __get_int_value(self, word, dict):
        value = self.__find(word, dict)
        if not value:
            return -1
        return to_int(value)

    def __get_float_value(self, word, dict):
        value = self.__find(word, dict)
        if not value:
            return -1
        return to_float(value)

    def __get_string_value(self, word, dict):
        value = self.__find(word, dict)
        if not value:
            return ''
        return value

    def __find(self, key, dict):
        value = None
        try:
            value = dict[key].strip()
        except KeyError:
            pass
        return value

    def __is_video(self, dict):
        value = self.__find('Track type', dict)
        if value and value == 'Video':
            return True
        return False

    def __is_audio(self, dict):
        value = self.__find('Track type', dict)
        if value and value == 'Audio':
            return True
        return False

    def __is_subtitle(self, dict):
        value = self.__find('Track type', dict)
        if value and value == 'Subtitle':
            return True
        return False

    def __add_video(self, dict, file_len):
        video = Video()

        # tnumber will be replaced later with the ID as mplayer sees it
        tnumber = self.__get_int_value('Track number', dict)
        if tnumber > -1:
            video.set_id(tnumber)

        codec_id = self.__get_string_value('Codec ID', dict)
        self.__video_set_codec(video, codec_id)

        duration = self.__get_string_value('Default duration', dict)
        self.__video_set_fps(video, duration)

        width = self.__get_int_value('Pixel width', dict)
        if width > -1:
            video.set_width(width)

        height = self.__get_int_value('Pixel height', dict)
        if height > -1:
            video.set_height(height)

        dwidth = self.__get_int_value('Display width', dict)
        dheight = self.__get_int_value('Display height', dict)
        if dwidth > -1 and dheight > 0:
            video.set_aspect(float(dwidth) / float(dheight))

        video.set_length(file_len)
        self.__container[0].add_video(video)
        return True

    def __add_audio(self, dict, file_len):
        audio = Audio()

        # tnumber will be replaced later with the ID as mplayer sees it
        tnumber = self.__get_int_value('Track number', dict)
        if tnumber > -1:
            audio.set_id(tnumber)

        codec_id = self.__get_string_value('Codec ID', dict)
        self.__audio_set_codec(audio, codec_id)

        lang = self.__get_string_value('Language', dict)
        self.__set_stream_language(audio, lang)

        freq = self.__get_float_value('Sampling frequency', dict)
        if freq > -1:
            audio.set_frequency(freq)

        nr_chan = self.__get_int_value('Channels', dict)
        if nr_chan > -1:
            audio.set_number_channels(nr_chan)

        audio.set_length(file_len)
        self.__container[0].add_audio(audio)
        return True

    def __add_subtitle(self, dict, file_len):
        subtitle = Subtitle()

        # tnumber will be replaced later with the ID as mplayer sees it
        tnumber = self.__get_int_value('Track number', dict)
        if tnumber > -1:
            subtitle.set_id(tnumber)

        codec_id = self.__get_string_value('Codec ID', dict)
        self.__subtitle_set_codec(subtitle, codec_id)

        lang = self.__get_string_value('Language', dict)
        self.__set_stream_language(subtitle, lang)

        subtitle.set_length(file_len)
        self.__container[0].add_subtitle(subtitle)
        return True


    def __video_set_codec(self, video, codec_id):
        if len(codec_id) < 1:
            return

        items = codec_id.split('/')
        if len(items) == 3:
            codec = items[0].lower()
            if codec ==  'v_mpeg4':
                video.set_type(VIDEO_MPEG4)

            elif codec ==  'v_ms':
                video.set_type(VIDEO_DIVX)

            else:
                debug('MatroskaParser.__video_set_codec()', 'unknown video type: %s' % codec_id, error)
        return

    def __video_set_fps(self, video, duration):
        if len(duration) < 1:
            return

        regex = re.compile('.*?\(\s*=\s*(\d+\.\d+)\s*fps\)')
        match = regex.match(duration)
        if match:
            fps = to_float(match.group(1))
            if fps > -1:
                video.set_fps(fps)
        return

    def __audio_set_codec(self, audio, codec_id):
        if len(codec_id) < 1:
            return

        codec = codec_id.lower()
        if codec ==  'a_vorbis':
            audio.set_format(AUDIO_OGG)

        elif codec.find('a_mpeg') > -1:
            items = codec.split('/')
            if len(items) == 2:
                if items[1] == 'l3':
                    audio.set_format(AUDIO_MP3)
                elif items[1] == 'l2':
                    audio.set_format(AUDIO_MP2)
                else:
                    debug('MatroskaParser.__audio_set_codec()', 'unknown audio type: %s' % codec_id, error)
            else:
                debug('MatroskaParser.__audio_set_codec()', 'unknown audio type: %s' % codec_id, error)

        else:
            debug('MatroskaParser.__audio_set_codec()', 'unknown audio type: %s' % codec_id, error)
        return
        
    def __subtitle_set_codec(self, subtitle, codec_id):
        if len(codec_id) < 1:
            return

        codec = codec_id.lower()
        if codec ==  's_vobsub':
            subtitle.set_format(SUBTITLE_VOBSUB)

        elif codec.find('s_text') > -1:
            subtitle.set_format(SUBTITLE_SRT)
            items = codec.split('/')
            if len(items) == 2:
                subtitle.set_encoding(items[1].lower())

        else:
            debug('MatroskaParser.__subtitle_set_codec()', 'unknown subtitle type: %s' % codec_id, error)
        return
        
    def __set_stream_language(self, stream, lang_str):
        if len(lang_str) < 1:
            return

        lang = self.__iso_codes.find_iso_639_2(lang_str)
        stream.set_language(lang.get_iso_639_1())
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
