# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from winkirip.lib import debug, error, to_int


class MplayerDVDParser:

    def __init__(self, lines, iso_codes, title, cont):
        self.__lines = lines
        self.__title  = title
        self.__container = cont
        self.__iso_codes = iso_codes
        self.__title_count_found = False
        self.__title_len_found = False
        return

    # -- public interface --

    def run(self):
        this_title_chapters = 'ID_DVD_TITLE_%d_CHAPTERS=' % self.__title.get_id()
        spos = 0
        acount = 0
        scount = 0

        for line in self.__lines:
            line = line.replace('\n', '')

            # if lsdvd and mplayer disagree about the number of titles on the DVD just return whith error
            if not self.__title_count_found and line.find('ID_DVD_TITLES=') > -1:
                if len(self.__container) != to_int(line[14 : ]):
                    debug('MplayerDVDParser.run()', 'unhandled title count mismatch', error)
                    return False
                self.__title_count_found = True

            # if the number of chapters does not match: take mplayer's guess (rare case but happens)
            elif not self.__title_len_found and line.find(this_title_chapters) > -1:
                num_chapters = to_int(line[len(this_title_chapters) : ])
                if len(self.__title.get_chapters()) != num_chapters:
                    debug('MplayerDVDParser.run()', 'expected %d chapters, got %d: fixing' % (len(self.__title.get_chapters()), num_chapters), error)
                    self.__fix_chapters(num_chapters)
                self.__title_len_found = True

            # update the audio index and language (lsdvd gets them sometimes wrong)
            elif line.find('audio stream: ') > -1 and line.find('Found') < 0:
                if not self.__update_audio(line):
                    return False
                acount += 1

            # update the subtitle index and language (lsdvd gets them sometimes wrong)
            elif line.find('subtitle ( sid ): ') > -1:
                if not self.__update_subtitle(line, spos):
                    return False
                spos += 1
                scount += 1

        result = True
        result &= self.__fix_audio_count(acount)
        result &= self.__fix_subtitle_count(scount)
        return result

    # -- helpers --

    def __fix_chapters(self, num_chapters):
        chaps = self.__title.get_chapters()

        result = []
        # use just the first num_chapters chapters
        if len(chaps) > num_chapters:
            result = []
            for i in range(num_chapters):
                result.append(chaps[i])

        else:
            # todo: this ignores the additional chapters seen by mplayer
            result.extend(chaps)

        # add just the "good" ones
        self.__title.reset_chapters()
        for ch in result:
            self.__title.add_chapter(ch)
        return

    def __update_audio(self, line):
        astream_start = line.find('audio stream: ')
        aformat_start = line.find('audio format: ') # either this or the next line are produced by different mplayer versions
        format_start = line.find('format: ') # either this or the previous line are produced by different mplayer versions
        lang_start = line.find('language: ')
        aid_start = line.find('aid: ')

        if astream_start < 0 or format_start < 0 or lang_start < 0 or aid_start < 0:
            debug('MplayerDVDParser.__update_audio()', 'strange formatted string used by mplayer', error)
            return False

        format_len = len('format: ')
        if aformat_start > 0:
            format_start = aformat_start
            format_len = len('audio format: ')

        astream = to_int(line[astream_start + len('audio stream: ') : format_start])
        aid = to_int(line[aid_start + len('aid: ') : len(line) - 1])

        if astream < 0 or aid < 0:
            debug('MplayerDVDParser.__update_audio()', 'cannot read astream or aid', error)
            return False

        fmt = line[format_start + format_len : lang_start].strip()
        lang = line[lang_start + len('language: ') : aid_start].strip()

        # make sure the language text is valid
        lobj = self.__iso_codes.find_iso_639_1(lang)
        lang = lobj.get_iso_639_1()

        astreams = self.__title.get_audio()
        if astream >= len(astreams):
            debug('MplayerDVDParser.__update_audio()', 'wrong index', error)
            return False

        audio = self.__title.get_audio_at(astream)
        audio.set_id(aid)
        audio.set_language(lang)
        return True

    def __update_subtitle(self, line, spos):
        sstream_start = line.find('subtitle ( sid ): ')
        lang_start = line.find('language: ')

        if sstream_start < 0 or lang_start < 0:
            debug('MplayerDVDParser.__update_subtitle()', 'strange formatted string used by mplayer', error)
            return False

        sid = to_int(line[sstream_start + len('subtitle ( sid ): ') : lang_start])
        lang = line[lang_start + len('language: ') : ]

        # make sure the language text is valid
        lobj = self.__iso_codes.find_iso_639_1(lang)
        lang = lobj.get_iso_639_1()

        sstreams = self.__title.get_subtitles()
        if spos >= len(sstreams):
            debug('MplayerDVDParser.__update_subtitles()', 'wrong index', error)
            return False

        subtitle = self.__title.get_subtitle_at(spos)
        subtitle.set_id(sid)
        subtitle.set_language(lang)
        return True

    def __remove_duplicates(self, lst):
        uniq = []
        skip = False

        for stream in lst:
            for elem in uniq:
                if elem.get_id() == stream.get_id():
                    skip = True
                    break

            if skip:
                skip = False
                continue

            uniq.append(stream)

        return uniq

    def __fix_audio_count(self, acount):
        astreams = self.__title.get_audio()

        # filter streams with the same ID (I didn't beleaved my eyes as I saw that is possible...)
        astreams = self.__remove_duplicates(astreams)

        result = []
        # use just the first acount streams
        if len(astreams) > acount:
            result = []
            for i in range(acount):
                result.append(astreams[i])

        else:
            # todo: this ignores the additional streams seen by mplayer
            result.extend(astreams)

        # add just the "good" ones
        self.__title.reset_audio()
        for astream in result:
            self.__title.add_audio(astream)

        return True

    def __fix_subtitle_count(self, scount):
        sstreams = self.__title.get_subtitles()

        # filter streams with the same ID (I didn't beleaved my eyes as I saw that is possible...)
        sstreams = self.__remove_duplicates(sstreams)

        result = []
        # use just the first acount streams
        if len(sstreams) > scount:
            for i in range(scount):
                result.append(sstreams[i])

        else:
            # todo: this ignores the additional streams seen by mplayer
            result.extend(sstreams)

        # add just the "good" ones
        self.__title.reset_subtitles()
        for sstream in result:
            self.__title.add_subtitle(sstream)

        return True

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
