# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

class Section:
    def __init__(self):
        self.__options = {}
        return

    def has_option(self, key):
        try:
            self.__options[key]
        except KeyError:
            return False
        return True

    def get_option(self, key):
        value = ''
        try:
            value = self.__options[key]
        except KeyError:
            pass
        return value

    def set_option(self, key, value):
        self.__options[key] = value
        return

    def remove_option(self, key):
        try:
            del self.__options[key]
        except:
            pass
        return

    def get_options(self):
        return self.__options.keys()

class RCConfig:
    def __init__(self):
        self.__sections = {}
        return

    def has_section(self, key):
        try:
            self.__sections[key]
        except KeyError:
            return False
        return True

    def get_section(self, key):
        value = None
        try:
            value = self.__sections[key]
        except KeyError:
            pass
        return value

    def add_section(self, key):
        try:
            self.__sections[key]
        except KeyError:
            self.__sections[key] = Section()
        return self.__sections[key]

    def remove_section(self, key):
        try:
            del self.__sections[key]
        except:
            pass
        return

    def get_sections(self):
        return self.__sections.keys()

    def set_option(self, section, option, value):
        sect = self.add_section(section)
        sect.set_option(option, value)
        return

class RCVersion:

        # -- public interface --

        def update_from_parser(self, parser):
            # overwrite the defaults with the config file values only for the
            # options, valid for the current version
            rc_config = self.__get_default_values()
            sections = rc_config.get_sections()
            for section in sections:
                if not parser.has_section(section):
                    continue

                sect = rc_config.get_section(section)
                options = sect.get_options()
                for option in options:
                    if not parser.has_option(section, option):
                        continue

                    sect.set_option(option, parser.get(section, option))

            # copy the device information from the config file if present
            if parser.has_section('settings'):
                dev_list = []
                if parser.has_option('settings', 'cd-rom devices'):
                    raw = parser.get('settings', 'cd-rom devices')
                    dev_list.extend(raw.split())

                if parser.has_option('settings', 'dvd-rom devices'):
                    raw = parser.get('settings', 'dvd-rom devices')
                    dev_list.extend(raw.split())

                for device in dev_list:
                    dev_name = 'device %s' % device
                    if parser.has_section(dev_name):
                        rc_config.add_section(dev_name)
                        options = parser.options(dev_name)
                        for option in options:
                            value = parser.get(dev_name, option)
                            rc_config.set_option(dev_name, option, value)

            return rc_config

        def update_parser(self, parser, rc_config):
            sections = rc_config.get_sections()
            for section in sections:
                if not parser.has_section(section):
                    parser.add_section(section)

                sect = rc_config.get_section(section)
                options = sect.get_options()
                for option in options:
                    value = sect.get_option(option)
                    parser.set(section, option, value)
            return parser

        # -- helpers --

        def __get_default_values(self):
            rc_config = RCConfig()

            rc_config.set_option('settings', 'output folder', '~/movie')
            rc_config.set_option('settings', 'default source', 'dvd')
            rc_config.set_option('settings', 'default encoding', 'multi pass')
            rc_config.set_option('settings', 'default file encoding', 'matroska')
            rc_config.set_option('settings', 'default size file', '700')
            rc_config.set_option('settings', 'default size dvd', '4370')
            rc_config.set_option('settings', 'default size vcd', '700')
            rc_config.set_option('settings', 'default size svcd', '700')
            rc_config.set_option('settings', 'preferred audio', 'en')
            rc_config.set_option('settings', 'preferred subtitles', 'en')
            rc_config.set_option('settings', 'user mode', 'default')
            rc_config.set_option('settings', 'dvd-rom devices', '')
            rc_config.set_option('settings', 'cd-rom devices', '')
            rc_config.set_option('settings', 'druid position', '100 100 800 600')
            rc_config.set_option('settings', 'settings position', '100 100 600 500')
            rc_config.set_option('settings', 'properties position', '100 100 600 400')
            rc_config.set_option('settings', 'chapter rip', 'false')
            rc_config.set_option('settings', 'exit winki', 'false')
            rc_config.set_option('settings', 'show popups', 'true')
            rc_config.set_option('settings', 'update rip ui', 'true')
            rc_config.set_option('settings', 'test rip', 'false')
            rc_config.set_option('program locations', 'mplayer', '')
            rc_config.set_option('program locations', 'mencoder', '')
            rc_config.set_option('program locations', 'ffmpeg', '')
            rc_config.set_option('program locations', 'oggenc', '')
            rc_config.set_option('program locations', 'mkvmerge', '')
            rc_config.set_option('program locations', 'avimerge', '')
            rc_config.set_option('program locations', 'mplex', '')
            rc_config.set_option('program locations', 'tcrequant', '')
            rc_config.set_option('program locations', 'tcextract', '')
            rc_config.set_option('program locations', 'paldump', '')
            rc_config.set_option('program locations', 'lindex', '')
            rc_config.set_option('program locations', 'title', '')
            rc_config.set_option('program locations', 'dvdauthor', '')
            rc_config.set_option('program locations', 'lsdvd', '')
            rc_config.set_option('program locations', 'mkisofs', '')
            rc_config.set_option('program locations', 'cdrecord', '')
            rc_config.set_option('program locations', 'hal-device', '')
            rc_config.set_option('program locations', 'MP4Box', '')
            rc_config.set_option('program locations', 'lame', '')
            rc_config.set_option('program locations', 'mp2enc', '')
            rc_config.set_option('program locations', 'vcdimager', '')
            rc_config.set_option('program locations', 'spuunmux', '')
            rc_config.set_option('program locations', 'spumux', '')
            rc_config.set_option('program locations', 'faac', '')
            rc_config.set_option('misc', 'nice value', '19')
            rc_config.set_option('misc', 'lower threshold mpeg-2', '2400')
            rc_config.set_option('misc', 'upper threshold mpeg-2', '9500')
            rc_config.set_option('misc', 'lower threshold mpeg-4', '800')
            rc_config.set_option('misc', 'upper threshold mpeg-4', '2500')
            rc_config.set_option('misc', 'last used dvd path', '.')
            rc_config.set_option('misc', 'last used vcd path', '.')
            rc_config.set_option('misc', 'last used file path', '.')
            rc_config.set_option('misc', 'mplayer preview args', '')
            rc_config.set_option('video', 'file preset', 'none')
            rc_config.set_option('video', 'dvd preset', 'none')
            rc_config.set_option('video', 'file format', 'MPEG-4')
            rc_config.set_option('video', 'file aspect', 'original')
            rc_config.set_option('video', 'file resolution', '720x576')
            rc_config.set_option('video', 'file shrink method', '1 pass')
            rc_config.set_option('video', 'dvd resolution', '720x576')
            rc_config.set_option('video', 'dvd format', 'MPEG-2')
            rc_config.set_option('video', 'dvd aspect', 'original')
            rc_config.set_option('video', 'dvd shrink method', 'requantize')
            rc_config.set_option('audio', 'file format', 'OGG')
            rc_config.set_option('audio', 'dvd format', 'AC3')
            rc_config.set_option('audio', 'ogg quality', '128')
            rc_config.set_option('audio', 'mp3 quality', '128')
            rc_config.set_option('audio', 'mp2 quality', '224')
            rc_config.set_option('audio', 'ac3 quality', '224')
            rc_config.set_option('audio', 'aac quality', '128')
            rc_config.set_option('estimation', 'ogg quality', '64 96 128 160 192 224 256 320')
            rc_config.set_option('estimation', 'mp3 quality', '64 96 128 160 192 224 256 320')
            rc_config.set_option('estimation', 'mp2 quality', '64 96 128 160 192 224 256 320')
            rc_config.set_option('estimation', 'ac3 quality', '64 96 112 128 224 384 512')
            rc_config.set_option('estimation', 'bytes per ogg second', '14000')
            rc_config.set_option('estimation', 'bytes per pcm second', '200000')
            rc_config.set_option('estimation', 'bytes per vobsub second', '3500')
            rc_config.set_option('estimation', 'bytes per vcd second', '144500')
            rc_config.set_option('estimation', 'preferred mpeg-2 bitrate', '4000')
            rc_config.set_option('estimation', 'minimum mpeg-2 bitrate', '2000')
            rc_config.set_option('estimation', 'preferred mpeg-4 bitrate', '1200')
            rc_config.set_option('estimation', 'minimum mpeg-4 bitrate', '800')
            rc_config.set_option('estimation', 'preferred xvid bitrate', '1200')
            rc_config.set_option('estimation', 'preferred x264 bitrate', '1200')
            rc_config.set_option('estimation', 'minimum xvid bitrate', '800')
            rc_config.set_option('estimation', 'minimum x264 bitrate', '800')
            rc_config.set_option('estimation', 'matroska container penalty', '0.0005')
            rc_config.set_option('estimation', 'svcd container penalty', '0.14')
            return rc_config

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
