# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from winkirip.scan import Scan
from winkirip.command import Command
from winkirip.parser_proc import ProcParser
from winkirip.parser_cdrecord import CDRecordParser
from winkirip.parser_hal_device import HALDeviceParser
from winkirip.device_info import DeviceInfo

class ScanDevices(Scan):

    def __init__(self, tool_checker):
        Scan.__init__(self, None, tool_checker)
        self.__lines = []
        self.__device_list = []
        return

    # -- methods inherited from scan.Scan --

    def run(self):
        self.__device_list = []

        if self.has_tool('hal-device'):
            return self.__scan_hal()

        return self.__scan_cdrecord()

    def reset(self):
        Scan.reset(self)
        self.__lines = []
        return

    # -- public interface --

    def get_device_list(self):
        return self.__device_list

    def get_from_config(self, cfg):
        self.__device_list = []

        dev_files = []
        dev_files.extend(cfg.get_dvdrom_devices())
        dev_files.extend(cfg.get_cdrom_devices())

        # remove duplicates from dev_files
        uniq = {}
        for dfile in dev_files:
            uniq[dfile] = 0
        dev_files = uniq.keys()

        result = True
        for dev_file in dev_files:
            dev_info = cfg.get_device_info('device %s' % dev_file)
            if not dev_info or len(dev_info) != 5:
                result = False
                break

            drive = DeviceInfo()
            drive.set_device_file(dev_file)
            drive.set_speed(int(dev_info['speed']))
            drive.set_manufacturer(dev_info['manufacturer'])
            drive.set_capabilities(int(dev_info['caps']))
            drive.set_model(dev_info['model'])
            drive.set_version(dev_info['version'])

            self.__device_list.append(drive)

        return result

    # -- helpers --

    def __scan_hal(self):
        prog = self.get_tool_location('hal-device')
        cmd = Command(prog, self.__accumulate_lines)
        cmd.set_bufsize(5)
        cmd.run()
        if cmd and cmd.get_status() != 0:
            self.reset()
            return False

        hparser = HALDeviceParser(self.__device_list, self.__lines)
        hparser.run()
        self.__device_list = hparser.get_device_list()

        return (not self.is_canceled())

    def __scan_cdrecord(self):
        if not self.has_tool('cdrecord'):
            return False

        if not self.__get_proc_info() or self.is_canceled():
            return False

        pparser = ProcParser(self.__lines)
        pparser.run()
        self.__device_list = pparser.get_device_list()

        for device in self.__device_list:
            self.reset()

            if not self.__get_device_info(device) or self.is_canceled():
                return False

            cparser = CDRecordParser(device, self.__lines)
            cparser.run()

        return True

    def __get_proc_info(self):
        cmd = Command('LC_ALL=C cat /proc/sys/dev/cdrom/info', self.__accumulate_lines)
        cmd.set_bufsize(5)
        cmd.run()
        if cmd and cmd.get_status() != 0:
            self.reset()
            return False

        return True

    def __get_device_info(self, device_file):
        prog = self.get_tool_location('cdrecord')
        cmd_text = '%s -scanbus dev=/dev/%s' % (prog, device_file.get_device_file())
        cmd = Command(cmd_text, self.__accumulate_lines)
        cmd.set_bufsize(5)
        cmd.run()
        if cmd and cmd.get_status() != 0:
            self.reset()
            return False

        return True

    def __accumulate_lines(self, line):
        self.__lines.append(line)
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
