# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from winkirip.scan import Scan
from winkirip.parser_dvd import LsDVDParser
from winkirip.parser_mplayer_dvd import MplayerDVDParser
from winkirip.command import Command
from winkirip.device_info import DeviceInfo
from winkirip.lib import debug, error, set_quotes

from winkirip import lib


class ScanDVD(Scan):

    def __init__(self, iso_codes, container, tool_checker):
        Scan.__init__(self, container, tool_checker)
        self.__lines = []
        self.__device_file_name = None
        self.__iso_codes = iso_codes
        return

    # -- methods inherited from scan.Scan --

    def run(self):
        if not self.__lsdvd():
            return False

        # arbitrary value at the beginning
        # (we don't know how many titles the DVD is going to have)
        self.__set_progress(1, 20)

        lsdvd_parser = LsDVDParser(self.__lines, self.__iso_codes, self.get_container())
        lsdvd_parser.run()

        if self.is_canceled():
            return True

        # lsdvd lies about some titles having more audio/subtitle languages
        # so we try to fix this information using mplayer
        self.reset()
        if not self.__mplayer():
            return False

        # copy the title length info to the audio, video and subtitle objects
        self.__apply_lengths()
        return True


    def reset(self):
        Scan.reset(self)
        self.__lines = []
        return

    def set_device(self, device):
        if isinstance(device, DeviceInfo):
            self.__device_file_name = device.get_full_device_file()
        else:
            self.__device_file_name = device
        return

    # -- helpers --

    def __lsdvd(self):
        if not self.has_tool('lsdvd'):
            debug('ScanDVD.__lsdvd()', 'cannot find lsdvd executable', error)
            return False

        prog = self.get_tool_location('lsdvd')

        cmd_line = '%s -x' % prog
        if self.__device_file_name:
            cmd_line += ' ' + lib.quote_path(self.__device_file_name)
        cmd = Command(cmd_line, self.__accumulate_lines)
        cmd.set_bufsize(20)
        cmd.run()
        if cmd and cmd.get_status() != 0:
            self.reset()
            return False

        return True

    def __mplayer(self):
        if not self.has_tool('mplayer'):
            debug('ScanDVD.__mplayer()', 'cannot find mplayer executable', error)
            return False

        prog = self.get_tool_location('mplayer')

        cont = self.get_container()
        title_count = len(cont)
        title_index = 1
        for title in cont:
            cmd_line = ('%s -v -identify -frames 0 dvd://%d' % (prog, title_index))
            if self.__device_file_name:
                cmd_line += (' -dvd-device %s'
                             % lib.quote_path(self.__device_file_name))
            title_index += 1

            self.__set_progress(title_index - 1, title_count)

            cmd = Command(cmd_line, self.__accumulate_lines)
            cmd.set_bufsize(5)
            cmd.run()
            if cmd and cmd.get_status() != 0:
                self.reset()
                return False

            if self.is_canceled():
                return True

            mplayer_parser = MplayerDVDParser(self.__lines, self.__iso_codes, title, cont)
            if not mplayer_parser.run():
                self.reset()
                return False

            self.reset()

        return True

    def __apply_lengths(self):
        cont = self.get_container()
        for title in cont:
            length = title.get_length()

            vstreams = title.get_video()
            for video in vstreams:
                video.set_length(length)

            astreams = title.get_audio()
            for audio in astreams:
                audio.set_length(length)

            sstreams = title.get_subtitles()
            for subtitle in sstreams:
                subtitle.set_length(length)
        return

    def __set_progress(self, cur, count):
        res = float(cur) / float(count)

        if res < 0.0:
            res = 0.0
        elif res > 1.0:
            res = 1.0

        self.set_progress(res)
        return

    def __accumulate_lines(self, line):
        self.__lines.append(line)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
