# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from winkirip.lib import debug, error, split_path, set_quotes, set_extension, to_float
from winkirip.scan import Scan
from winkirip.parser_file import FileParser
from winkirip.parser_subtitle import SrtParser, SubParser, VobsubParser
from winkirip.parser_matroska import MatroskaParser
from winkirip.command import Command
from winkirip.container import FILE_UNKNOWN, FILE_MATROSKA, FILE_AVI
from winkirip.container import FILE_MPEG, FILE_OGM, FILE_WMV, FILE_SUBTITLE
from winkirip.container_subtitle import SUBTITLE_UNKNOWN, SUBTITLE_VOBSUB, SUBTITLE_SRT
from winkirip.container_subtitle import SUBTITLE_SUB, SUBTITLE_IMGFILE
from winkirip.container_audio import AUDIO_PCM, AUDIO_AC3, AUDIO_OGG
from winkirip.container_audio import AUDIO_MP2, AUDIO_MP3, AUDIO_FLAC

class ScanFile(Scan):

    def __init__(self, iso_codes, container, tool_checker):
        Scan.__init__(self, container, tool_checker)
        self.__lines = []
        self.__device_file_name = None
        self.__iso_codes = iso_codes
        self.__type = FILE_UNKNOWN
        self.__sub_type = SUBTITLE_UNKNOWN
        self.__file_len = 0.
        return

    # -- methods inherited from scan.Scan --

    def run(self):
        if not self.__scan():
            debug('ScanFile.run()', 'media scan failed', error)
            return False

        # arbitrary value at the beginning
        self.__set_progress(1, 20)

        if self.__type == FILE_SUBTITLE:
            if self.__sub_type == SUBTITLE_SRT:
                file_parser = SrtParser(self.__lines, self.__iso_codes, self.get_container())

            elif self.__sub_type == SUBTITLE_SUB:
                file_parser = SubParser(self.__lines, self.__iso_codes, self.get_container())

            elif self.__sub_type == SUBTITLE_VOBSUB:
                file_parser = VobsubParser(self.__lines, self.__iso_codes, self.get_container())

            else:
                debug('ScanFile.run()', 'unknown subtitle format', error)
                return False

        elif self.__type == FILE_MATROSKA:
            file_parser = MatroskaParser(self.__lines, self.__iso_codes, self.get_container(), self.__file_len)

        else:
            file_parser = FileParser(self.__lines, self.__iso_codes, self.get_container())

        if not file_parser.run():
            # mplayer does not spit the usual info for some mkv files
            # so try to scan the file with mplayer
            if self.__type == FILE_MATROSKA:
                file_parser = FileParser(self.__lines, self.__iso_codes, self.get_container())
                if not file_parser.run():
                    self.reset()
                    debug('ScanFile.run()', 'media parse failed', error)
                    return False
            else:
                self.reset()
                debug('ScanFile.run()', 'media parse failed', error)
                return False

        #self.__set_audio_video_file_names()
        return True


    def reset(self):
        Scan.reset(self)
        self.__lines = []
        return

    def set_device(self, device):
        self.__device_file_name = device
        return

    # -- helpers --

    def __scan(self):
        if not self.__device_file_name:
            debug('ScanFile.__scan()', 'no file name supplied', error)
            return False

        # todo: endable subtitle scanning after improving the parsing
        #if self.__is_subtitle_file():
        #    return self.__scan_subtitle()

        path, name, ext = split_path(self.__device_file_name)
        if ext.lower() == '.mkv':
            self.__type = FILE_MATROSKA

        return self.__scan_mplayer()

    def __is_subtitle_file(self):
        dirs, name, ext = split_path(self.__device_file_name)
        if ext and ext == '.srt' and self.__is_text_file():
            self.__type = FILE_SUBTITLE
            self.__sub_type = SUBTITLE_SRT
            return True

        if ext and ext == '.sub' and self.__is_text_file():
            self.__type = FILE_SUBTITLE
            self.__sub_type = SUBTITLE_SUB
            return True

        return False

    def __is_text_file(self):
        cmd_line = 'LC_ALL=C file %s' % set_quotes(self.__device_file_name)
        cmd = Command(cmd_line, self.__accumulate_lines)
        cmd.set_bufsize(5)
        cmd.run()
        if cmd and cmd.get_status() != 0:
            self.reset()
            debug('ScanFile.__is_text_file()', 'detection via "file" failed: 0', error)
            return False

        if len(self.__lines) < 1:
            debug('ScanFile.__is_text_file()', 'detection via "file" failed: 1', error)
            return False
        
        # todo: this is a weak test - the string 'text' may have been contained
        # in the file name
        if self.__lines[0].lower().find('text') > -1:
            # todo: try to extract file's encoding from this line
            self.__lines = []
            return True

        return False

    def __scan_subtitle(self):
        fname = self.__device_file_name
        self.__lines = []

        # vobsub
        if self.__type != SUBTITLE_SUB and self.__type != SUBTITLE_SRT:
            fname = set_extension(self.__device_file_name, 'idx')
            self.__type = FILE_SUBTITLE
            self.__sub_type = SUBTITLE_VOBSUB

        try:
            f = open(fname, 'r')
            raw = f.read()
            f.close()
        except IOError:
            self.__type = FILE_UNKNOWN
            debug('ScanFile.__scan_subtitles()', 'IO error', error)
            return False

        lines = raw.split('\n')
        for line in lines:
            self.__lines.append(line)

        return True

    def __scan_mplayer(self):
        if not self.has_tool('mplayer'):
            debug('ScanFile.__scan_mplayer()', 'cannot find mplayer executable', error)
            return False

        prog = self.get_tool_location('mplayer')

        cmd_line = '%s -v -identify -frames 0 "%s"' % (prog, self.__device_file_name)
        cmd = Command(cmd_line, self.__accumulate_lines)
        cmd.set_bufsize(5)
        cmd.run()
        if cmd and cmd.get_status() != 0:
            self.reset()
            debug('ScanFile.__scan_mplayer()', 'mplayer scan failed', error)
            return False

        return True

    def __set_audio_video_file_names(self):
        cont = self.get_container()
        for title in cont:
            video_list = title.get_video()
            map(lambda video: video.set_filename(self.__device_file_name), video_list)

            audio_list = title.get_audio()
            map(lambda audio: audio.set_filename(self.__device_file_name), audio_list)
        return

    def __set_progress(self, cur, count):
        res = float(cur) / float(count)

        if res < 0.0:
            res = 0.0
        elif res > 1.0:
            res = 1.0

        self.set_progress(res)
        return

    def __accumulate_lines(self, line):
        self.__lines.append(line)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
