# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import ListStore
from winkirip.lib import build_path, winki_path, check_widgets, debug, error
from winkirip.lib import textcombo_setup, textcombo_get_value, to_int, to_float
from winkirip.container import CONTAINER_UNKNOWN, FILE_MATROSKA, FILE_AVI, FILE_MP4
from winkirip.container_video import VIDEO_MPEG1, VIDEO_MPEG2, VIDEO_MPEG4
from winkirip.container_video import VIDEO_XVID, VIDEO_H264, VIDEO_THEORA
from winkirip.container_video import VIDEO_REAL, VIDEO_DIVX, VIDEO_WMV
from winkirip.container_video import VIDEO_UNKNOWN, VIDEO_FLASH, VIDEO_QTIME
from winkirip.container_audio import Audio, AUDIO_PCM, AUDIO_AC3, AUDIO_OGG
from winkirip.container_audio import AUDIO_UNKNOWN, AUDIO_QTIME
from winkirip.container_audio import AUDIO_MP2, AUDIO_MP3, AUDIO_FLAC, AUDIO_AAC, AUDIO_WMA
from winkirip.container_subtitle import Subtitle, SUBTITLE_UNKNOWN, SUBTITLE_VOBSUB
from winkirip.container_subtitle import SUBTITLE_SRT, SUBTITLE_SUB, SUBTITLE_IMGFILE
from winkirip.section_ui import PropFrame, VideoSection, AudioSection, SubtitleSection

class SelectProperties:
    def __init__(self, iso_codes):
        self.__iso_codes = iso_codes
        self.__wnd = None
        self.__vobjects = []
        self.__aobjects = []
        self.__sobjects = []
        self.__store_position_func = None
        self.__restore_position_func = None
        self.__ignore_update = False
        self.__update_callback = None

        self.__aspects = [(_('16/9 (widescreen)'), '16/9'), (_('4/3 (TV)'), '4/3'), (_('2.11/1 (DVB)'), '2.11')]
        self.__encodings = [(_('Arabic ISO-8859-6'), 'iso8859-6'),
                            (_('Baltic ISO-8859-13'), 'iso8859-13'),
                            (_('Baltic CP-1257'), 'cp1257'),
                            (_('Central European ISO-8859-2'), 'iso8859-2'),
                            (_('Central European CP-1250'), 'cp1250'),
                            (_('Cyrillic ISO-8859-5'), 'iso8859-5'),
                            (_('Cyrillic CP-1251'), 'cp1251'),
                            (_('Greek ISO-8859-7'), 'iso8859-7'),
                            (_('Greek CP-1253'), 'cp1253'),
                            (_('Hebrew ISO-8859-8'), 'iso8859-8'),
                            (_('Hebrew CP-1255'), 'cp1255'),
                            (_('Nordic ISO-8859-10'), 'iso8859-10'),
                            (_('Romanian ISO-8859-16'), 'iso8859-16'),
                            (_('South European ISO-8859-3'), 'iso8859-3'),
                            (_('Turkish CP-1254'), 'cp1254'),
                            (_('Turkish ISO-8859-9'), 'iso8859-9'),
                            (_('Unicode UTF-8'), 'utf8'),
                            (_('Unicode UTF-16'), 'utf16'),
                            (_('Vietnamese CP-1258'), 'cp1258'),
                            (_('Western ISO-8859-15'),'iso8859-15'),
                            (_('Western CP-1252'), 'cp1252')]
        return

    # --- public interface ---

    def show(self, vlist, alist, slist, update_callback):
        self.__update_callback = update_callback

        # reset the cached objects
        self.__vobjects = []
        self.__aobjects = []
        self.__sobjects = []

        # the window itself
        self.__wnd = PropFrame(self)
        self.__wnd.create_window()

        # the data sections
        for video in vlist:
            video_section = VideoSection()
            self.__wnd.add_section(video_section)
            self.__init_video(video, video_section)

        for audio in alist:
            audio_section = AudioSection()
            self.__wnd.add_section(audio_section)
            self.__init_audio(audio, audio_section)

        for subtitle in slist:
            subtitle_section = SubtitleSection()
            self.__wnd.add_section(subtitle_section)
            self.__init_subtitle(subtitle, subtitle_section)

        # the bottom row with the close button
        self.__wnd.add_bottom_section()

        if self.__restore_position_func:
            restore_height = False
            self.__restore_position_func(self.__wnd.get_window(), 'properties', restore_height)

        self.__wnd.show()
        return

    def hide(self):
        if self.__store_position_func:
            self.__store_position_func(self.__wnd.get_window(), 'properties')

        self.__submit_video()
        self.__submit_audio()
        self.__submit_subtitles()

        if self.__update_callback:
            self.__update_callback()
        return

    def set_store_position(self, func):
        self.__store_position_func = func
        return

    def set_restore_position(self, func):
        self.__restore_position_func = func
        return

    # --- signal handlers ---

    def on_audio_lang_changed(self, widget):
        if self.__ignore_update:
            return True

        for item in self.__aobjects:
            obj, widgets = item
            if widgets.get_lang() == widget:
                img_widget = widgets.get_lang_image()
                self.__setup_lang(widget.get_text(), img_widget)
        return True

    def on_subtitle_lang_changed(self, widget):
        if self.__ignore_update:
            return True

        for item in self.__sobjects:
            obj, widgets = item
            if widgets.get_lang() == widget:
                img_widget = widgets.get_lang_image()
                self.__setup_lang(widget.get_text(), img_widget)
        return True

    def on_subtitle_format_changed(self, widget, event = None):
        if self.__ignore_update:
            return True

        for item in self.__sobjects:
            obj, widgets = item
            if widgets.get_format() == widget:
                combo_encoding = widgets.get_encoding()
                label_encoding = widgets.get_label_encoding()
                format = textcombo_get_value(widget)

                if format != SUBTITLE_SRT and format != SUBTITLE_SUB:
                    label_encoding.set_sensitive(False)
                    combo_encoding.set_sensitive(False)
                else:
                    label_encoding.set_sensitive(True)
                    combo_encoding.set_sensitive(True)
        return True

    # --- helpers ---

    def __object_match(self, vlist, alist, slist):
        if len(vlist) != len(self.__vobjects) or len(alist) != len(self.__aobjects) or len(slist) != len(self.__sobjects):
            return False

        for i in range(len(self.__vobjects)):
            obj, widgets = self.__vobjects[i]
            if obj != vlist[i]:
                return False

        for i in range(len(self.__aobjects)):
            obj, widgets = self.__aobjects[i]
            if obj != alist[i]:
                return False

        for i in range(len(self.__sobjects)):
            obj, widgets = self.__sobjects[i]
            if obj != slist[i]:
                return False

        return True

    def __init_video(self, video, video_section):
        combo_format = video_section.get_format()
        formats = [(_('MPEG-1'), VIDEO_MPEG1),
                  (_('MPEG-2'), VIDEO_MPEG2),
                  (_('MPEG-4'), VIDEO_MPEG4),
                  (_('XviD'), VIDEO_XVID),
                  (_('Theora'), VIDEO_THEORA),
                  (_('x264'), VIDEO_H264),
                  (_('Real'), VIDEO_REAL),
                  (_('DivX'), VIDEO_DIVX),
                  (_('WMV'), VIDEO_WMV),
                  (_('Flash'), VIDEO_FLASH),
                  (_('QuickTime Video'), VIDEO_QTIME),
                  (_('Unknown video format'), VIDEO_UNKNOWN)]
        format = video.get_type()
        textcombo_setup(combo_format, formats, format)

        combo_aspect = video_section.get_aspect()
        aspect = video.get_aspect()
        if not aspect:
            aspect = ''
        else:
            if not isinstance(aspect, str):
                aspect = self.__num2str(aspect)
        self.__setup_textcombo(combo_aspect, self.__aspects, aspect)

        spin_len = video_section.get_len()
        spin_len.set_value(video.get_length())

        spin_width = video_section.get_width()
        spin_width.set_value(video.get_width())

        spin_height = video_section.get_height()
        spin_height.set_value(video.get_height())

        entry_fps =  video_section.get_fps()
        entry_fps.set_text('%s' % self.__num2str(video.get_fps()))

        self.__vobjects.append((video, video_section))
        return

    def __init_audio(self, audio, audio_section):
        self.__ignore_update = True

        combo_format = audio_section.get_format()
        formats = [(_('PCM'), AUDIO_PCM),
                   (_('AC3'), AUDIO_AC3),
                   (_('OGG'), AUDIO_OGG),
                   (_('MP2'), AUDIO_MP2),
                   (_('MP3'), AUDIO_MP3),
                   (_('FLAC'), AUDIO_FLAC),
                   (_('WMA'), AUDIO_WMA),
                   (_('AAC'), AUDIO_AAC),
                   (_('QuickTime Audio'), AUDIO_QTIME),
                   (_('Unknown audio format'), AUDIO_UNKNOWN)]
        format = audio.get_format()
        textcombo_setup(combo_format, formats, format)

        img_lang = audio_section.get_lang_image()
        entry_lang = audio_section.get_lang()
        entry_lang.connect('changed', self.on_audio_lang_changed)
        self.__setup_lang(audio.get_language(), img_lang, entry_lang)

        spin_len = audio_section.get_len()
        spin_len.set_value(audio.get_length())

        spin_bitrate = audio_section.get_bitrate()
        spin_bitrate.set_value(audio.get_kbits())

        spin_nrch = audio_section.get_nrchan()
        spin_nrch.set_value(audio.get_number_channels())

        spin_freq = audio_section.get_freq()
        spin_freq.set_value(audio.get_frequency())

        self.__ignore_update = False

        self.__aobjects.append((audio, audio_section))
        return

    def __init_subtitle(self, subtitle, subtitle_section):
        self.__ignore_update = True

        combo_format = subtitle_section.get_format()
        formats = [(_('VobSub'), SUBTITLE_VOBSUB),
                   (_('SRT'), SUBTITLE_SRT),
                   (_('SUB'), SUBTITLE_SUB),
                   (_('Image files'), SUBTITLE_IMGFILE),
                   (_('Unknown subtitle format'), SUBTITLE_UNKNOWN)]
        format = subtitle.get_format()
        combo_format.connect('changed', self.on_subtitle_format_changed)
        textcombo_setup(combo_format, formats, format)

        img_lang = subtitle_section.get_lang_image()
        entry_lang = subtitle_section.get_lang()
        entry_lang.connect('changed', self.on_subtitle_lang_changed)
        self.__setup_lang(subtitle.get_language(), img_lang, entry_lang)

        enc = subtitle.get_encoding()
        if not enc:
            enc = ''
        combo_encoding = subtitle_section.get_encoding()
        self.__setup_textcombo(combo_encoding, self.__encodings, enc)

        # enable encoding controls only for text subtitles
        if format != SUBTITLE_SRT and format != SUBTITLE_SUB:
            label_encoding = subtitle_section.get_label_encoding()
            label_encoding.set_sensitive(False)
            combo_encoding.set_sensitive(False)

        self.__ignore_update = False

        self.__sobjects.append((subtitle, subtitle_section))
        return

    def __submit_video(self):
        for item in self.__vobjects:
            video, widgets = item

            combo_format = widgets.get_format()
            video.set_type(textcombo_get_value(combo_format))

            combo_aspect = widgets.get_aspect()
            self.__submit_video_aspect(combo_aspect, video)

            spin_len = widgets.get_len()
            length = to_float(spin_len.get_value())
            if length > 0.:
                video.set_length(length)

            spin_width = widgets.get_width()
            width = to_int(spin_width.get_value())
            if width > 0:
                video.set_width(width)

            spin_height = widgets.get_height()
            height = to_int(spin_height.get_value())
            if height > 0:
                video.set_height(height)

            entry_fps = widgets.get_fps()
            fps = to_float(entry_fps.get_text())
            if fps > 0:
                video.set_fps(fps)
        return

    def __submit_audio(self):
        for item in self.__aobjects:
            audio, widgets = item

            combo_format = widgets.get_format()
            audio.set_format(textcombo_get_value(combo_format))

            entry_lang = widgets.get_lang()
            lang = self.__get_lang(entry_lang.get_text())
            audio.set_language(lang.get_iso_639_1())

            spin_len = widgets.get_len()
            length = to_float(spin_len.get_value())
            if length > 0.:
                audio.set_length(length)

            spin_bitrate = widgets.get_bitrate()
            bitrate = to_int(spin_bitrate.get_value())
            if bitrate > 0.:
                audio.set_kbits(bitrate)

            spin_nrch = widgets.get_nrchan()
            nrchan = to_int(spin_nrch.get_value())
            if nrchan > 0:
                audio.set_number_channels(nrchan)

            spin_freq = widgets.get_freq()
            freq = to_int(spin_freq.get_value())
            if freq > 0:
                audio.set_frequency(freq)
        return

    def __submit_subtitles(self):
        for item in self.__sobjects:
            subtitle, widgets = item

            combo_format = widgets.get_format()
            subtitle.set_format(textcombo_get_value(combo_format))

            entry_lang = widgets.get_lang()
            lang = self.__get_lang(entry_lang.get_text())
            subtitle.set_language(lang.get_iso_639_1())

            combo_enc = widgets.get_encoding()
            self.__submit_subtitle_encoding(combo_enc, subtitle)
        return

    def __setup_textcombo(self, combo, lst, default):
        model = combo.get_model()
        if not model:
            model = ListStore(str)
            combo.set_model(model)
            combo.set_text_column(0)
        else:
            model.clear()

        pos = 0
        default_idx = -1
        for item in lst:
            value, key = item
            combo.append_text(value)
            if key == default:
                default_idx = pos
            pos += 1

        if not default:
            return

        if default_idx < 0:
            combo.get_child().set_text(default)
        else:
            combo.set_active(default_idx)
        return

    def __setup_lang(self, lang_str, img_widget, entry_widget = None):
        if not lang_str:
            lang_str = 'xx'

        lang = self.__get_lang(lang_str)
        img_widget.set_from_pixbuf(lang.get_flag())

        if entry_widget:
            entry_widget.set_text(lang.get_name())
        return

    def __get_lang(self, lang_str):
        # this is a bit cumbersome: first we get a reference to the "unknown" language
        # for comparison because the find_... functions return it as a fall back
        lang_unk = self.__iso_codes.find_name('unknown')

        # is this a valid lanugage name?
        lang = self.__iso_codes.find_name(lang_str)
        if lang.get_name() == lang_unk.get_name():
            # it isn't: is this a valid iso 639-1 name?
            lang = self.__iso_codes.find_iso_639_1(lang_str)
            if lang.get_name() == lang_unk.get_name():
                # it isn't: is this a valid iso 639-2 name?
                lang = self.__iso_codes.find_iso_639_2(lang_str)
        return lang

    def __num2str(self, num):
        # int?
        if (num - int(num)) < 1e-30:
            return '%d' % num

        # X.Y0000 ? 
        rest1 = num * 10.0
        if (rest1 - int(rest1)) < 1e-30:
            return '%.1f' % num

        return '%.2f' % num

    def __submit_video_aspect(self, combo, video):
        # get the text and replace it with a standard
        # value if appropriate (the combo text is localized)
        aspect = combo.get_child().get_text()
        for item in self.__aspects:
            value, key = item
            if value == aspect:
                aspect = key
                break

        if aspect:
            # 42/3.14 or whatever
            if aspect.find('/') > -1:
                items = aspect.split('/')
                if len(items) == 2:
                    x = to_float(items[0])
                    y = to_float(items[1])
                    if x > 0. and y > 0.:
                        video.set_aspect('%s/%s' % (self.__num2str(x), self.__num2str(y)))
            # float number
            else:
                num = to_float(aspect)
                if num > 0.:
                    video.set_aspect('%s' % self.__num2str(num))
        return

    def __submit_subtitle_encoding(self, combo, subtitle):
        encoding = combo.get_child().get_text()
        for item in self.__encodings:
            value, key = item
            if value == encoding:
                encoding = key
                break

        if encoding and len(encoding) > 0:
            subtitle.set_encoding(encoding)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
