# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from math import fabs
from winkirip.lib import debug, error

class ChapterSplitter:
    def __init__(self, chapter_list):
        self.__chapter_list = chapter_list
        return

    # -- public interface --

    def split(self, nr_chunks):
        result = []

        # invalid input
        if nr_chunks < 1:
            debug('ChapterSplitter.split', 'splitting in %d chunks requested' % nr_chunks, error)
            result = [0]

        # one chunk requested: put everything into it
        elif nr_chunks == 1:
            result = [0]

        # more chunks requested than chapters available:
        # put every chapter into separate chunk
        elif nr_chunks >= (len(self.__chapter_list) - 1):
            for index in range(len(self.__chapter_list)):
                result.append(index)

        # more chapters available than chunks requested
        else:
            # try to split into the requested number of chunks
            time_optimum = self.__get_optimum_size(nr_chunks)
            result, rest = self.__do_split(time_optimum, 0.0)

            # if unsuccessful: average the error and distribute it to all
            # chunks
            if len(result) > nr_chunks:
                result, rest = self.__do_split(time_optimum, rest / float(nr_chunks))

            # if we still cannot split properly - just append the last segment
            # to the segment bevore it
            if len(result) != nr_chunks:
                result.pop()

        if len(result) != nr_chunks:
            debug('ChapterSplitter.split()', 'requested %d chunks; delivered %d' % (nr_chunks, len(result)), error)
        return result

    def __get_optimum_size(self, nr_chunks):
        sum_len = 0.0

        for item in self.__chapter_list:
            chapter = item.get_chapter()
            sum_len += chapter.get_length()
        return sum_len / float(nr_chunks)

    def __do_split(self, time_optimum, tolerance):
        result = [0]
        chunk_sum = 0.0

        for i in range(len(self.__chapter_list)):
            chapter = self.__chapter_list[i].get_chapter()
            if (chunk_sum + chapter.get_length()) > (time_optimum + tolerance) and i > 1:
                result.append(i)
                chunk_sum = chapter.get_length()
            else:
                chunk_sum += chapter.get_length()

        return result, chunk_sum


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
