# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from gtk import events_pending, main_iteration
from winkirip.command import Command
from winkirip.parser_encoder_caps import EncoderCapsParser
from winkirip.lib import build_path, winki_path, lower_strip, debug, error

TOOL_EXTERNAL = 0
TOOL_MENCODER = 1

class Tool:
    def __init__(self, name = None, location = None, available = False, tooltype = TOOL_EXTERNAL):
        self.__name = name
        self.__location = location
        self.__available = available
        self.__type = tooltype
        return

    # -- public interface --

    def get_name(self):
        return self.__name

    def set_name(self, name):
        self.__name = name
        return

    def get_location(self):
        return self.__location

    def set_location(self, location):
        self.__location = location
        return

    def get_available(self):
        return self.__available

    def set_available(self, available):
        self.__available = available
        return

    def get_type(self):
        return self.__type

    def set_type(self, tooltype):
        self.__type = tooltype
        return

    def __repr__(self):
        avail = 'not available'
        if self.__available:
            avail = 'available'

        ttype = 'external'
        if self.__type != TOOL_EXTERNAL:
            ttype = 'internal'

        return '%s:%s:%s:%s' % (self.__name, self.__location, avail, ttype)


class ToolChecker:
    def __init__(self, config):
        self.__tools = []
        self.__config = config
        self.__cmd_result = ''
        return

    # -- public interface --

    def load(self):
        external_tools = ['mplayer', 'mencoder', 'lsdvd', 'oggenc', 'ffmpeg',
        'lame', 'mp2enc', 'vcdimager', 'mkvmerge', 'mplex', 'dvdauthor' ,
        'tcrequant', 'tcextract', 'paldump', 'spuunmux', 'spumux', 'title',
        'lindex', 'avimerge', 'mkisofs', 'cdrecord', 'hal-device', 'MP4Box',
        'faac']
        internal_tools = ['toolame', 'twolame', 'mp3lame', 'xvid', 'lavc', 'x264']

        for tool_name in external_tools:
            tool_path = self.__get_config_path(tool_name)
            tool = self.__create_external_tool(tool_name, tool_path)
            self.__tools.append(tool)

        tools = self.__create_internal_tools(internal_tools)
        for tool in tools:
            self.__tools.append(tool)

        return

    def get_tool(self, tool_name, tool_type = TOOL_EXTERNAL):
        result = None

        for tool in self.__tools:
            if tool.get_name().lower() == tool_name.lower() and tool.get_type() == tool_type:
                result = tool
                break

        return result

    def get_tool_location(self, tool_name, lang_neutral = True):
        tool = self.get_tool(tool_name)
        if not tool or tool.get_type() != TOOL_EXTERNAL:
            return ''

        if not lang_neutral:
            return tool.get_location()

        return 'LC_ALL=C ' + tool.get_location()

    def set_tool_location(self, tool_name, location):
        tool = self.get_tool(tool_name)
        if not tool or tool.get_type() != TOOL_EXTERNAL:
            return False

        tool.set_location(location)
        return self.__config.set_tool_location(tool_name, location)

    def has_tool(self, tool_name, tool_type = TOOL_EXTERNAL):
        tool = self.get_tool(tool_name, tool_type)
        if not tool:
            return False

        return tool.get_available()

    # -- helpers --

    def __get_config_path(self, tool_name):
        # look if the user specified a path to the tool
        path = self.__config.get_tool_location(tool_name)

        # if not - look for the tool in the default path
        if not path or not len(path):
            path = self.__find_tool(tool_name)
            
        return path

    def __find_tool(self, tool_name):
        self.__cmd_result = ''

        shell_cmd = 'LC_ALL=C which %s' % tool_name
        cmd = Command(shell_cmd, self.__get_path)
        cmd.bufsize = 5
        cmd.run()
        if cmd and cmd.get_status() != 0:
            return ''

        return self.__cmd_result

    def __get_path(self, line):
        self.__cmd_result += line
        return

    def __append_line(self, line):
        self.__cmd_result.append(line)
        return

    def __create_external_tool(self, tool_name, tool_path):
        available = (len(tool_path) > 0)
        if not available:
            tool_path = None

        return Tool(tool_name, tool_path, available, TOOL_EXTERNAL)

    def __create_internal_tools(self, tool_list):
        mencoder = 'mencoder'
        if self.has_tool('mencoder'):
            mencoder = self.get_tool_location('mencoder')

        self.__cmd_result = []

        shell_cmd = '%s -ovc help -oac help' % mencoder
        cmd = Command(shell_cmd, self.__append_line)
        cmd.bufsize = 20
        cmd.run()

        parser = EncoderCapsParser(self.__cmd_result)

        result = []
        for tool_name in tool_list:
            availability = False
            if parser.run(tool_name):
                availability = True
            tool = Tool(name = tool_name, available = availability, tooltype = TOOL_MENCODER)
            result.append(tool)

        return result

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
