# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.container import CONTAINER_UNKNOWN, CONTAINER_DVD, CONTAINER_FILE
from winkirip.container_video import Video, VIDEO_XVID, VIDEO_MPEG2
from winkirip.container_video import VIDEO_MPEG4
from winkirip.container_audio import Audio
from winkirip.container_subtitle import Subtitle
from winkirip.worker_scale_progress import WorkerScaleProgress
from winkirip.lib import debug, error, to_int, append_to_filename, set_extension
from winkirip.lib import file_size, number_lines, build_path, does_path_exist
from os import unlink

EMPTY_FILE = 0
EMPTY_XML_FILE = 1

class Worker(WorkerScaleProgress):
    def __init__(self, context, create_callback, ui_name):
        WorkerScaleProgress.__init__(self)
        self.__context = context
        self.__create_callback = create_callback
        self.__commands = []
        self.__name = ui_name
        self.__ui_command_count = 0
        return

    # -- public interface --

    def get_context(self):
        return self.__context

    def get_source_container(self, cont_id):
        source = self.__context.get_project().get_source()
        match_list = filter(lambda container: container.get_id() == cont_id, source)
        if len(match_list) < 1:
            return None

        # make sure container IDs are unique
        assert(len(match_list) == 1)
        return match_list[0]

    def get_dest_container(self, cont_id):
        dest = self.__context.get_project().get_dest()
        match_list = filter(lambda container: container.get_id() == cont_id, dest)
        if len(match_list) < 1:
            return None

        # make sure container IDs are unique
        assert(len(match_list) == 1)
        return match_list[0]

    def get_dest_container_type(self, cont_id):
        cont = self.get_dest_container(cont_id)
        if not cont:
            return CONTAINER_UNKNOWN
        return cont.get_type()

    def get_source_container_type(self, cont_id):
        cont = self.get_source_container(cont_id)
        if not cont:
            return CONTAINER_UNKNOWN
        return cont.get_type()

    def get_source_title(self, cont_id, title_id):
        src_cont = self.get_source_container(cont_id)
        if not src_cont:
            return None

        return self.__get_title(src_cont, title_id)

    def get_dest_title(self, cont_id, title_id):
        dest_cont = self.get_dest_container(cont_id)
        if not dest_cont:
            debug('Worker.get_dest_title()', 'No dest title with CID=%d TID=%d' % (cont_id, title_id), error)
            assert(False)
            return None

        return self.__get_title(dest_cont, title_id)

    def get_start_end_chapter(self, title):
        if not title:
            debug('Worker.get_start_end_chapter()', 'no title supplied', error)
            return -1, -1

        ch_start = title.get_first_chapter_id()
        ch_end = title.get_last_chapter_id()
        return ch_start, ch_end

    def get_lang_name(self, iso_codes, language):
        ui_lang_name = _('Unknown')
        lang = iso_codes.find_iso_639_1(language)
        if lang:
            ui_lang_name = lang.get_name()
        return ui_lang_name

    def get_audio_list(self, source_container_id):
        result = []

        # pick just the streams which belong to this source media
        dest = self.get_context().get_dest()
        for container in dest:
            for title in container:
                audio_list = title.get_audio()
                for audio in audio_list:
                    source_audio = audio.get_source()
                    source_id = source_audio.get_container_id()
                    if source_id != source_container_id:
                        continue

                    result.append(audio)

        return result

    def get_video_list(self, source_container_id):
        result = []

        # pick just the streams which belong to this source media
        dest = self.get_context().get_dest()
        for container in dest:
            for title in container:
                video_list = title.get_video()
                for video in video_list:
                    source_video = video.get_source()
                    source_id = source_video.get_container_id()
                    if source_id != source_container_id:
                        continue

                    result.append(video)
        return result

    def get_subtitle_list(self, source_container_id):
        result = []

        # pick just the streams which belong to this source media
        dest = self.get_context().get_dest()
        for container in dest:
            for title in container:
                subtitle_list = title.get_subtitles()
                for subtitle in subtitle_list:
                    source_subtitle = subtitle.get_source()
                    source_id = source_subtitle.get_container_id()
                    if source_id != source_container_id:
                        continue

                    result.append(subtitle)
        return result

    def get_commands(self):
        return self.__commands

    def create_commands(self):
        if not self.__create_callback:
            return False

        self.__commands = []
        return self.__create_callback(self.__commands)

    def get_ui_name(self):
        return self.__name

    # helpers used by the derived classes

    def compute_scale_expand(self, video, crop_values, aspect_dst, resolution):
        result = ''

        if not isinstance(video, Video):
            debug('Worker.compute_scale_expand()', 'video object type error', error)
            return '', ''

        aspect_src = video.get_aspect()
        if aspect_dst == 'original':
            aspect_dst = aspect_src

        width_source = video.get_width()
        height_source = video.get_height()

        if resolution == 'original':
            width_target = width_source
            height_target = height_source
        else:
            w, h = resolution.split('x')
            width_target = to_int(w)
            height_target = to_int(h)
            if width_target < 0 or height_target < 0:
                debug('Worker.compute_scale_expand()', 'resolution not set', error)
                return aspect_dst, result

        height_crop = height_source
        width_crop = width_source
        shift_x = -1
        shift_y = -1
        if crop_values and len(crop_values) == 4 and crop_values[0] > 0 and crop_values[1] > 0:
            width_crop, height_crop, shift_x, shift_y = crop_values
        else:
            print crop_values
            debug('Worker.compute_scale_expand()', 'invalid crop', error)
            return aspect_dst, 'scale=%d:-2,expand=%d:%d,' % (width_target, width_target, height_target)

        aspect_target = 16.0 / 9.0
        if aspect_dst == '4/3':
            aspect_target = 4.0 / 3.0

        aspect_source = 16.0 / 9.0
        if aspect_src == '4/3':
            aspect_source = 4.0 / 3.0

        width_expand = (width_target  * aspect_source) / aspect_target
        height_expand = height_target

        # let the width consume the whole screen and adapt the height
        width_scale = width_expand
        height_scale = (((((float(height_crop) * 100.0) / float(height_source)) * height_target) / 100.0) / aspect_source) * aspect_target

        # make all values multiple of 16
        width_crop = self.__align(width_crop)
        height_crop = self.__align(height_crop)
        width_scale = self.__align(width_scale)
        height_scale = self.__align(height_scale)
        width_expand = self.__align(width_expand)
        height_expand = self.__align(height_expand)

        result = ''
        if width_crop != width_source or height_crop != height_source:
            # todo: shift_x/y might be incorrect because width/height might have been altered
            # not by much though
            if shift_x > -1 and shift_y > -1:
                result = 'crop=%d:%d:%d:%d,' % (width_crop, height_crop, shift_x, shift_y)
            else:
                result = 'crop=%d:%d,' % (width_crop, height_crop)

        if width_scale != width_expand or height_scale != height_expand:
            result += ('scale=%d:%d,expand=%d:%d,' % (width_scale, height_scale, width_expand, height_expand))
        # if width_scale == width_expand and height_scale == height_expand, but
        # we still have to resize
        elif width_crop != width_scale or height_crop != height_scale:
            result += ('scale=%d:%d,' % (width_scale, height_scale))

        return aspect_dst, result

    def calc_vbitrate(self, disc_size, admin_size, occupied, seconds, vsize, preferred_bitrate):
        if occupied < 0. or seconds < 0. or vsize < 0.:
            debug('Worker.calc_vbitrate()', 'uninitialized value: occupied = %f, seconds = %f, vsize = %f' % (occupied, seconds, vsize), error)
            assert(False)

        # no size limit
        if disc_size < 0:
            return preferred_bitrate, False
        else:
            vbitrate = self.__vbitrate(disc_size, admin_size, occupied, seconds)

        # mplayer and mencoder interpret values > 16000 as bits/second instead
        # of kbits/second
        if vbitrate > 16000:
            vbitrate = 16000

        # the space available for the video is larger than the video stream
        shrink_needed = True
        if vsize > 0 and (disc_size - admin_size - occupied) > vsize:
            shrink_needed = False

        return vbitrate, shrink_needed

    def unlink_temp(self, file_list):
        for fname in file_list:
            if not fname or not len(fname):
                continue

            try:
                unlink(fname)
            except OSError:
                debug('Worker.unlink_temp()', 'cannot unlink %s' % fname, error)

        return


    def expand_chapter_list(self, ch_start, ch_end, outname):
        cfg = self.get_context().get_project().get_config()

        # don't bother with chapter ripping if a test rip should be performed
        if not self.__change_names() or cfg.get_test_rip():
            return [(ch_start, ch_end, outname)]

        return [(ch, ch, append_to_filename(outname, '-%d' % ch)) for ch in range(ch_start, ch_end + 1)]

    def get_name_for_chapters(self, ch_start, ch_end, outname):
        if not self.__change_names():
            return outname

        if ch_start == ch_end:
            return append_to_filename(outname, '-%d' % ch_start)

        assert(False)
        return outname

    def get_chapter_len(self, start_ch, end_ch, title, allow_zero = False):
        chap_len = 0.

        chap_list = title.get_chapters()
        for chapter in chap_list:
            if chapter.get_id() > start_ch and chapter.get_id() <= end_ch:
               chap_len += chapter.get_length()

        if abs(chap_len) < 1e-30 and not allow_zero:
            debug('Worker.get_chapter_len()', 'chapter len is zero - setting to 1', error)
            chap_len = 1.

        return chap_len

    def remove_stream_if_empty_file(self, stream, check_kind):
        cont_id = stream.get_container_id()
        title_id = stream.get_title_id()
        title = self.get_dest_title(cont_id, title_id)

        if check_kind == EMPTY_FILE:
            filter_func = self.__check_empty
        elif check_kind == EMPTY_XML_FILE:
            filter_func = self.__check_empty_xml
        else:
            debug('Worker.remove_stream_if_empty_file()', 'unknown function type', error)
            return False

        # remove streams which does not contain data
        if isinstance(stream, Video):
            result = filter(filter_func, title.get_video())

            title.reset_video()
            for vstream in result:
                title.add_video(vstream)

        elif isinstance(stream, Audio):
            result = filter(filter_func, title.get_audio())

            title.reset_audio()
            for astream in result:
                title.add_audio(astream)

        elif isinstance(stream, Subtitle):
            result = filter(filter_func, title.get_subtitles())

            title.reset_subtitles()
            for sstream in result:
                title.add_subtitle(sstream)

        else:
            debug('Worker.remove_stream_if_empty_file()', 'unknown stream type', error)
            return False

        return True

    def create_stream_name(self, cid, tid, dest_dir, ext, sid = -1):
        if sid > -1:
            fname = set_extension('disc%d_title%d_%d' % (cid, tid, sid), ext)
        else:
            fname = set_extension('disc%d_title%d' % (cid, tid), ext)
        return build_path([dest_dir, fname])

    # -- helpers --

    def __align(self, number):
        if number % 16 != 0:
            number = to_int(number / 16) * 16
        return to_int(number)

    def __vbitrate(self, disc_size, admin_size, occupied, seconds):
        return (disc_size - admin_size - occupied) / seconds / 125.0

    def __get_title(self, container, title_id):
        match_list = filter(lambda title: title.get_id() == title_id, container)
        if len(match_list) < 1:
            debug('Worker.__get_title()', 'No title with TID=%d in\n%s' % (title_id, container), error)
            assert(False)
            return None

        # make sure title IDs are unique inside a container
        assert(len(match_list) == 1)
        return match_list[0]

    def __check_empty(self, stream):
        fname = stream.get_filename()

        if isinstance(stream, Subtitle):
            fname = set_extension(fname, 'sub')

        # chapter rip: append number to the name
        if self.__change_names():
            cont_id = stream.get_container_id()
            title_id = stream.get_title_id()
            title = self.get_dest_title(cont_id, title_id)
            chapter_list = title.get_chapters()
            for chapter in chapter_list:
                tname =  append_to_filename(fname, '-%d' % chapter.get_id())
                if not does_path_exist(tname) or file_size(tname) < 1:
                    return False
        # regular rip
        else:
            if not does_path_exist(fname) or file_size(fname) < 1:
                return False

        return True

    def __check_empty_xml(self, stream):
        dest_dir = self.get_context().get_project().get_dest_dir()
        xml_fname = build_path([dest_dir, stream.get_filename()])

        # chapter rip: append number to the name
        if self.__change_names():
            cont_id = stream.get_container_id()
            title_id = stream.get_title_id()
            title = self.get_dest_title(cont_id, title_id)
            chapter_list = title.get_chapters()
            for chapter in chapter_list:
                tname =  append_to_filename(xml_fname, '-%d' % chapter.get_id())
                if number_lines(tname) < 5: # todo dirty check: lines < 5
                    return False
        # regular rip
        else:
            if number_lines(xml_fname) < 5: # todo dirty check: lines < 5
                return False

        return True

    def __change_names(self):
        cfg = self.get_context().get_project().get_config()
        if not cfg.get_chapter_rip() or self.get_dest_container_type(0) != CONTAINER_DVD or self.get_source_container_type(0) != CONTAINER_DVD:
            return False
        return True


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
