# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker_stream import WorkerStream
from winkirip.command_ui import UICommand
from winkirip.container import CONTAINER_DVD, CONTAINER_FILE, CONTAINER_UNKNOWN
from winkirip.container_audio import AUDIO_PCM, AUDIO_MOBILE
from winkirip.constructor_audio_dump import ConstructorAudioPcm, ConstructorAudioCopy
from winkirip.lib import debug, error, to_float
import re

class WorkerAudio(WorkerStream):
    def __init__(self, context, source_id):
        WorkerStream.__init__(self, context, source_id, _('Extract audio'))

        self.__outname = None

        self.__audio_len = 1.
        self.__prev_audio_len = 0.

        self.__regex = re.compile('A:\ *(\d+?\.\d+?)\ +V:\ *(\d+?\.\d+?)')
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def init_stream_list(self, src_cid):
        return self.get_audio_list(src_cid)

    def prepare_ui_commands(self):
        return

    def create_main_commands(self, audio, audio_idx):
        iso_codes = self.get_context().get_iso_codes()

        fname, lname = self.get_names(audio, iso_codes)

        ui_command = UICommand()
        ui_command.set_parse_callback(self.__parse_output)
        ui_command.set_can_progress(self.can_progress())
        ui_command.set_data([audio, fname])

        return [ui_command]

    def create_post_command(self, audio, audio_idx):
        ui_command = UICommand()
        ui_command.set_command(self.exec_remove_empty_file)
        ui_command.set_data([audio])
        return ui_command

    def prepare_exec(self, command_data, command):
        assert(len(command_data) == 2)
        audio, outname = command_data

        self.use_stream(audio)
        self.__outname = outname
        return True

    def get_outname(self):
        return self.__outname

    def construct(self, start_ch, end_ch, outname):
        cfg = self.get_context().get_config()

        constr = self.get_constructor()
        if not constr:
            debug('WorkerAudio.construct()', 'cannot create constructor', error)
            return None

        constr.set_test_rip(cfg.get_test_rip(), self.get_src_title().get_length())
        constr.set_source(self.get_device(self.get_src_cont()), self.get_src_tid())
        constr.set_source_type(self.get_src_cont_type())
        constr.set_chapters(start_ch, end_ch)
        constr.set_outname(outname)
        constr.set_aid(self.get_stream_id())

        success, shell_cmd = constr.construct()
        if not success:
            debug('WorkerAudio.construct()', 'command construction failed', error)
            self.set_error(_('Command construction failed'))
            return None

        self.__audio_len = self.get_chapter_len(start_ch - 1, end_ch, self.get_src_title())
        self.__prev_audio_len = self.get_chapter_len(0, start_ch - 1, self.get_src_title(), True)

        return shell_cmd

    def extract(self, shell_cmd, command):
        return command.run_command(shell_cmd)

    def update_failure(self):
        debug('WorkerAudio.update_failure()', 'execution failed', error)
        return

    def update_success(self):
        self.update_audio(self.get_stream(), self.__outname)
        return

    # -- common functions for the derived classes --

    def get_constructor(self):
        # should be implemented by derived classes
        assert(False)
        return None

    def can_progress(self):
        return True

    def get_names(self, audio, iso_codes):
        # should be implemented by derived classes
        assert(False)
        return '', ''

    def update_audio(self, audio, fname):
        # should be implemented by derived classes
        assert(False)
        return '', ''

    # -- helpers --

    def __parse_output(self, line):
        progress = 0.0
        estimation = ''

        match = self.__regex.match(line)
        if match:
            items = match.group(1, 2)
            progress = to_float(items[1])
            if progress < 0:
                progress = 0.
            else:
                progress = self.scale_progress((progress - self.__prev_audio_len) / self.__audio_len)
        return progress, estimation


class WorkerPCM(WorkerAudio):
    def __init__(self, context, source_id):
        WorkerAudio.__init__(self, context, source_id)
        return

    # -- methods inherited from worker_audio.WorkerAudio --

    def get_constructor(self):
        tool_checker = self.get_context().get_tool_checker()
        cfg = self.get_context().get_config()
        dest_format = cfg.get_audio_format_file()

        constr = ConstructorAudioPcm(tool_checker)
        if dest_format == AUDIO_MOBILE:
            constr.set_resample('mobile')
        return constr

    def get_names(self, audio, iso_codes):
        disc_id = audio.get_container_id()
        title_id = audio.get_title_id()

        aid = audio.get_id()
        fname = self.create_stream_name(disc_id + 1, title_id + 1, 'audio', 'wav', aid)
        lname = self.get_lang_name(iso_codes, audio.get_language())

        return fname, lname

    def update_audio(self, audio, fname):
        audio.set_filename(fname)
        #audio.set_id(apos + 1)
        audio.set_format(AUDIO_PCM)
        return


class WorkerCopy(WorkerAudio):
    def __init__(self, context, source_id):
        WorkerAudio.__init__(self, context, source_id)
        return

    # -- methods inherited from worker_audio.WorkerAudio --

    def get_constructor(self):
        tool_checker = self.get_context().get_tool_checker()

        constr = ConstructorAudioCopy(tool_checker)
        return constr

    def can_progress(self):
        return False

    def get_names(self, audio, iso_codes):
        disc_id = audio.get_container_id()
        title_id = audio.get_title_id()

        aid = audio.get_id()
        atype = audio.format2str()

        fname = self.create_stream_name(disc_id + 1, title_id + 1, 'audio', atype, aid)
        lname = self.get_lang_name(iso_codes, audio.get_language())

        return fname, lname

    def update_audio(self, audio, fname):
        audio.set_filename(fname)
        #audio.set_id(apos + 1)
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
