# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker_stream import WorkerStream
from winkirip.command_ui import UICommand
from winkirip.lib import set_extension, debug, error, to_float, sec2str
from winkirip.container import CONTAINER_DVD
from winkirip.container_audio import AUDIO_UNKNOWN, AUDIO_PCM, AUDIO_AC3
from winkirip.container_audio import AUDIO_OGG, AUDIO_MP2, AUDIO_MP3
from winkirip.container_audio import AUDIO_AAC, AUDIO_MOBILE
from constructor_audio import ConstructorOgg, ConstructorMp3, ConstructorMp2
from constructor_audio import ConstructorAc3, ConstructorAac
from os import symlink
from copy import deepcopy
import re

class WorkerAudioCompress(WorkerStream):
    def __init__(self, context, source_id):
        WorkerStream.__init__(self, context, source_id, _('Compress audio'))

        self.__in_file = ''
        self.__out_file = ''
        self.__uncompressed = []
        self.__audio_len = 1.
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def init_stream_list(self, src_cid):
        return self.get_audio_list(src_cid)

    def prepare_ui_commands(self):
        return

    def create_post_command(self, audio, audio_idx):
        ui_command = UICommand()
        ui_command.set_command(self.unlink_uncompressed)
        ui_command.set_data([audio])
        return ui_command

    def prepare_exec(self, command_data, command):
        assert(len(command_data) == 1)
        audio, = command_data

        self.use_stream(audio)

        self.__in_file = audio.get_filename()
        self.__out_file = set_extension(self.__in_file, self.get_audio_ext())

        if self.__is_same_format(audio):
            # if the source audio file is a stand alone audio file
            # create a link to it in the dest audio directory
            # todo: not tested
            source_audio = audio.get_source()
            if source_audio.get_filename():
                symlink(source_audio.get_filename(), self.__out_file)

        return True

    def get_outname(self):
        return self.__in_file

    def construct(self, start_ch, end_ch, fname_ch):
        cfg = self.get_context().get_config()
        self.__audio_len = self.get_chapter_len(start_ch - 1, end_ch, self.get_dst_title())

        out_fname_ch = set_extension(fname_ch, self.get_audio_ext())

        constr = self.get_constructor()
        constr.set_outname(out_fname_ch)
        constr.set_inname(fname_ch)

        constr.set_test_rip(cfg.get_test_rip(), self.get_dst_title().get_length())

        success, shell_cmd = constr.construct()
        if not success:
            debug('WorkerAudioCompress.construct()', 'command construction failed', error)
            self.set_error(_('Command construction failed'))
            return None

        if not self.__is_same_format(self.get_stream()):
            self.__uncompressed.append(fname_ch)

        return shell_cmd

    def extract(self, shell_cmd, command):
        if self.__is_same_format(self.get_stream()):
            return True

        return command.run_command(shell_cmd, from_stderr = self.use_stderr())

    def update_failure(self):
        debug('WorkerAudioCompress.update_failure()', 'execution failed', error)
        return

    def update_success(self):
        self.get_stream().set_filename(self.__out_file)
        self.get_stream().set_format(self.get_audio_format())
        return

    # -- common functions for the derived classes --

    def get_audio_len(self):
        return self.__audio_len

    def unlink_uncompressed(self, command_data, command):
        if len(self.__uncompressed) < 1:
            return True

        fnames = ''
        for fname in self.__uncompressed:
            fnames += (' %s' % fname)
        self.log_append('rm %s' % fnames)

        self.log_append('\n')
        self.unlink_temp(self.__uncompressed)
        return True

    def use_stderr(self):
        return True

    def correct_bitrate(self, bitrate_in, valid_bitrates):
        audio = self.get_stream()
        abitrate = audio.get_kbits()
        if abitrate == 0:
            debug('WorkerAudioCompress.correct_bitrate()',
                  'FIXME: undetermined source audio bitrate, proceeding with '
                  'requested encoding bitrate %d' % bitrate_in, error)
            return bitrate_in
        if abitrate < bitrate_in:
            list_valid = deepcopy(valid_bitrates)
            list_valid.reverse()
            for allowed in list_valid:
                if allowed < abitrate:
                    return allowed
            debug('WorkerAudioCompress.correct_bitrate()', 'cannot find valid bitrate lower than %d' % abitrate, error)
            self.log_append('# the requested audio bitrate (%d) cannot be used. the source has very low bitrate (%d by minimum %d for this codec). using default and hoping for the best...' % (bitrate_in, abitrate, valid_bitrates[0]))
        return -1

    def get_audio_format(self):
        # should be implemented by derived classes
        assert(False)
        return AUDIO_UNKNOWN

    def get_audio_ext(self):
        # should be implemented by derived classes
        assert(False)
        return ''

    def get_constructor(self):
        # should be implemented by derived classes
        assert(False)
        return None

    # -- helpers --

    def __is_same_format(self, audio):
        result = False

        # current and required format matches: nothing to do
        if self.get_audio_format() == audio.get_format():
            result = True

        # current and required format matches (quick hack)
        if self.get_audio_format() == AUDIO_MOBILE and audio.get_format() == AUDIO_PCM:
            result = True

        return result

class WorkerOGG(WorkerAudioCompress):
    def __init__(self, context, source_id):
        WorkerAudioCompress.__init__(self, context, source_id)
        self.__ogg_regex = re.compile('\[\s*(\d+\.\d+)%\]\s*\[\s*(.*?)\s*remaining\]')
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def create_main_commands(self, audio, audio_idx):
        ui_command = UICommand()
        ui_command.set_parse_callback(self.__parse_ogg_output)
        ui_command.set_can_progress(True)
        ui_command.set_can_estimate(True)
        ui_command.set_data([audio])
        return [ui_command]

    # -- methods inherited from worker_stream.WorkerAudioCompress --

    def get_audio_format(self):
        return AUDIO_OGG

    def get_audio_ext(self):
        return 'ogg'

    def get_constructor(self):
        tool_checker = self.get_context().get_tool_checker()
        cfg = self.get_context().get_config()
        abitrate = self.correct_bitrate(cfg.get_ogg_quality(), cfg.get_ogg_values())

        constr = ConstructorOgg(tool_checker)
        constr.set_bitrate(abitrate)
        constr.set_resample('48000')
        return constr

    # -- helpers --

    def __parse_ogg_output(self, line):
        progress = 0.0
        estimation = ''

        line_start = line.find('[')
        if line_start > -1:
            match = self.__ogg_regex.match(line[line_start : ])
            if match:
                prog, est = match.group(1, 2)
                progress = to_float(prog)
                if progress < 0:
                    progress = 0.
                else:
                    progress = self.scale_progress(progress / 100.)

                show_ms = False
                estimation = self.__ogg_estim2sec(est)
                estimation = sec2str(self.scale_estimation(estimation), show_ms)
        return progress, estimation

    def __ogg_estim2sec(self, text):
        items = text.replace('m', ' ').replace('s', ' ').split()
        if len(items) == 2:
            m = int(items[0])
            s = int(items[1])
            return m * 60 + s
        return 0


class WorkerMP2(WorkerAudioCompress):
    def __init__(self, context, source_id):
        WorkerAudioCompress.__init__(self, context, source_id)
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def create_main_commands(self, audio, audio_idx):
        ui_command = UICommand()
        ui_command.set_parse_callback(self.__parse_mp2_output)
        ui_command.set_can_progress(True)
        ui_command.set_data([audio])
        return [ui_command]

    # -- methods inherited from worker_stream.WorkerAudioCompress --

    def get_audio_format(self):
        return AUDIO_MP2

    def get_audio_ext(self):
        return 'mp2'

    def get_constructor(self):
        tool_checker = self.get_context().get_tool_checker()
        cfg = self.get_context().get_config()
        cont_type = self.get_dst_cont().get_type()

        constr = ConstructorMp2(tool_checker)
        if cont_type == CONTAINER_DVD:
            constr.set_bitrate(384)
            constr.set_resample('48000')
        else:
            abitrate = self.correct_bitrate(cfg.get_mp2_quality(), cfg.get_mp2_values())
            constr.set_bitrate(abitrate)
        return constr

    # -- helpers --

    def __parse_mp2_output(self, line):
        progress = 0.

        line_start = line.find('--DEBUG: [mp2enc]')
        if line_start > -1:
            items = line.split()
            if len(items) == 5 and items[3] == 'seconds':
                progress = to_float(items[2])
                if progress < 0:
                    progress = 0.
                else:
                    progress = self.scale_progress(progress / self.get_audio_len())
        return progress, ''


class WorkerMP3(WorkerAudioCompress):
    def __init__(self, context, source_id):
        WorkerAudioCompress.__init__(self, context, source_id)
        self.__mp3_regex = re.compile('.*?\(\s*(\d+)%\)')
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def create_main_commands(self, audio, audio_idx):
        ui_command = UICommand()
        ui_command.set_parse_callback(self.__parse_mp3_output)
        ui_command.set_can_progress(True)
        ui_command.set_can_estimate(True)
        ui_command.set_data([audio])
        return [ui_command]

    # -- methods inherited from worker_stream.WorkerAudioCompress --

    def get_audio_format(self):
        return AUDIO_MP3

    def get_audio_ext(self):
        return 'mp3'

    def get_constructor(self):
        tool_checker = self.get_context().get_tool_checker()
        cfg = self.get_context().get_config()
        abitrate = self.correct_bitrate(cfg.get_mp3_quality(), cfg.get_mp3_values())

        constr = ConstructorMp3(tool_checker)
        constr.set_bitrate(abitrate)
        constr.set_resample('48')
        return constr

    # -- helpers --

    def __parse_mp3_output(self, line):
        estimation = ''
        progress = 0.

        line_start = line.find('(')
        if line_start > -1:
            match = self.__mp3_regex.match(line[line_start : ])
            if match:
                prog = match.group(1)
                progress = to_float(prog)
                if progress < 0:
                    progress = 0.
                else:
                    progress = self.scale_progress(progress / 100.)

                items = line[line_start : ].split()
                if len(items) > 6:
                    show_ms = False
                    estimation = self.__mp3_estim2sec(items[len(items) - 1])
                    estimation = sec2str(self.scale_estimation(estimation), show_ms)
        return progress, estimation

    def __mp3_estim2sec(self, text):
        items = text.replace(':', ' ').split()
        if len(items) == 2:
            m = int(items[0])
            s = int(items[1])
            return m * 60 + s

        elif len(items) == 3:
            h = int(items[0])
            m = int(items[1])
            s = int(items[2])
            return h * 3600 + m * 60 + s
        return 0


class WorkerAC3(WorkerAudioCompress):
    def __init__(self, context, source_id):
        WorkerAudioCompress.__init__(self, context, source_id)
        self.__ac3_regex = re.compile('.*?time=(\d+\.\d+)')
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def create_main_commands(self, audio, audio_idx):
        ui_command = UICommand()
        ui_command.set_parse_callback(self.__parse_ac3_output)
        ui_command.set_can_progress(True)
        ui_command.set_data([audio])
        return [ui_command]

    # -- methods inherited from worker_stream.WorkerAudioCompress --

    def get_audio_format(self):
        return AUDIO_AC3

    def get_audio_ext(self):
        return 'ac3'

    def get_constructor(self):
        tool_checker = self.get_context().get_tool_checker()
        cfg = self.get_context().get_config()
        cont_type = self.get_dst_cont().get_type()

        constr = ConstructorAc3(tool_checker)
        if cont_type == CONTAINER_DVD:
            constr.set_bitrate(384)
            constr.set_resample('48000')
        else:
            abitrate = self.correct_bitrate(cfg.get_ac3_quality(), cfg.get_ac3_values())
            constr.set_bitrate(abitrate)
        return constr

    # -- helpers --

    def __parse_ac3_output(self, line):
        progress = 0.

        line_start = line.find('size=')
        if line_start > -1:
            match = self.__ac3_regex.match(line[line_start : ])
            if match:
                item = match.group(1)
                progress = to_float(item)
                if progress < 0:
                    progress = 0.
                else:
                    progress = self.scale_progress(progress / self.get_audio_len())
        return progress, ''

class WorkerAAC(WorkerAudioCompress):
    def __init__(self, context, source_id):
        WorkerAudioCompress.__init__(self, context, source_id)
        self.__aac_regex = re.compile('\s*\(\s*(\d+)%\)|')
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def create_main_commands(self, audio, audio_idx):
        ui_command = UICommand()
        ui_command.set_parse_callback(self.__parse_aac_output)
        ui_command.set_can_progress(True)
        ui_command.set_can_estimate(True)
        ui_command.set_data([audio])
        return [ui_command]

    # -- methods inherited from worker_stream.WorkerAudioCompress --

    def get_audio_format(self):
        return AUDIO_AAC

    def get_audio_ext(self):
        return 'aac'

    def get_constructor(self):
        tool_checker = self.get_context().get_tool_checker()
        cfg = self.get_context().get_config()
        abitrate = self.correct_bitrate(cfg.get_aac_quality(), cfg.get_aac_values())

        constr = ConstructorAac(tool_checker)
        constr.set_bitrate(abitrate)
        return constr

    # -- helpers --

    def __parse_aac_output(self, line):
        estimation = ''
        progress = 0.

        line_start = line.find('(')
        if line_start > -1:
            match = self.__aac_regex.match(line[line_start : ])
            if match:
                prog = match.group(1)
                progress = to_float(prog)
                if progress < 0:
                    progress = 0.
                else:
                    progress = self.scale_progress(progress / 100.)

                items = line[line_start : ].split('|')
                if len(items) > 4:
                    show_ms = False
                    estimation = self.__aac_estim2sec(items[len(items) - 1])
                    estimation = sec2str(self.scale_estimation(estimation), show_ms)
        return progress, estimation

    def __aac_estim2sec(self, text):
        items = text.replace('.', ' ').split()
        if len(items) == 2:
            s = int(items[0])
            return s
        return 0


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
