# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.command_ui import UICommand
from winkirip.lib import sec2str, fix_name, fix_fname, file_size, to_int, debug, error
from winkirip.writer_xml import WriterXML
from winkirip.constructor_mpegvideo import ConstructorMpegVideo
from winkirip.container import CONTAINER_FILE
from winkirip.alert import AlertError
from os.path import join
import re

class WorkerAuthorDVD(Worker):
    def __init__(self, context):
        Worker.__init__(self, context, self.__create, _('Author DVD'))
        self.__exec_log_text = ''
        self.__dvdauthor_regex = re.compile('.*?VOBU.*?at\s+(\d+)MB(.*?(\d+)%)*')
        self.__video_files_size = 1.
        return

    # -- helpers --

    def __create(self, command_list):
        ui_command = UICommand()
        ui_command.set_command(self.__exec_create_xml)
        ui_command.set_exec_log_callback(self.__exec_log)
        ui_command.set_parse_callback(self.__parse_dvdauthor_output)
        ui_command.set_can_progress(True)
        command_list.append(ui_command)

        dest = self.get_context().get_dest()
        if not self.init_command_progress(len(dest)):
            debug('WorkerCreateAuthorDVD.__create()', 'cannot init progress commands', error)
            return False

        warning_issued = False
        for disc_nr in range(len(dest)):
            #cont = self.get_context().get_dest()[disc_nr]

            fname = 'disc.xml'
            if len(dest) > 1:
                fname = 'disc%d.xml' % (disc_nr + 1)

            # todo
            # dvdauthor does not like streams created by tcextract (this is
            # the case we haven't requantified the extracted stream and haven
            # mixed it with subtitles or audio) or created by tcextract and
            # requantifyed by tcrequant. The funny thing is - mplex fixes such
            # streams. It is very unlikely that the user does not want a dvd
            # without audio, so I'll postpone this problem yet...

            #ui_command = UICommand(self.__exec_filter_video, ui_text, True)
            #ui_command.set_exec_log_callback(self.__exec_log)
            #ui_command.set_data([disc_nr, title_nr])
            #command_list.append(ui_command)

            ui_command = UICommand()
            ui_command.set_command(self.__exec_author)
            ui_command.set_exec_log_callback(self.__exec_log)
            ui_command.set_parse_callback(self.__parse_dvdauthor_output)
            ui_command.set_can_progress(True)
            ui_command.set_data([fname])
            command_list.append(ui_command)

            for title in dest[disc_nr]:
                if warning_issued:
                    break

                vlist = title.get_video()
                for video in vlist:
                    vres = self.__get_resolution(video)
                    warning_issued = self.__check_video_resolution(vres)
                    if warning_issued:
                        break

        return True

    def __exec_author(self, command_data, command):
        assert(len(command_data) == 1)
        fname, = command_data

        tool_checker = self.get_context().get_tool_checker()
        if not tool_checker.has_tool('dvdauthor'):
            self.__exec_log_text = ('# dvdauthor not found\n')
            command.set_error(_('No %s installation found') % 'dvdauthor')
            return False

        program = tool_checker.get_tool_location('dvdauthor')
        cmd_line = '%s -x %s' % (program, fname)
        self.__exec_log_text = '%s\n\n' % cmd_line

        self.next_command_progress()
        if not self.init_chapter_progress(1):
            debug('WorkerCreateAuthorDVD.__exec_author()', 'cannot init chapter progress', error)
            return False
        self.next_chapter_progress()

        # ok this function gets called multiple times but is not that expensive
        # and should return always the same result
        self.__calc_dest_size()

        # for displaying only one warning
        return command.run_command(cmd_line, from_stderr = True)

    def __calc_dest_size(self):
        fsize = 0
        dest = self.get_context().get_dest()
        for cont in dest:
            for title in cont:
                vlist = title.get_video()
                for video in vlist:
                    fname = video.get_filename()
                    fsize += file_size(fname)

        # size in mb
        if abs(fsize) > 1e-3:
            self.__video_files_size = fsize / 1024.0 / 1024
        return

    def __exec_create_xml(self, command_data, command):
        self.__exec_log_text = ''

        dest = self.get_context().get_dest()
        for disc_nr in range(len(dest)):
            cont = self.get_context().get_dest()[disc_nr]

            fname = 'disc.xml'
            out_dir = 'DISC'
            if cont.get_name():
                out_dir = fix_fname(cont.get_name()).upper()

            if len(dest) > 1:
                out_dir += ('%d' % (disc_nr + 1))
                fname = 'disc%d.xml' % (disc_nr + 1)

            writer = WriterXML(fname)
            self.__start_xml(writer, out_dir)
            self.__start_menu_xml(writer)
            self.__end_menu_xml(writer)
            self.__start_titleset_xml(writer)

            writer.increment()

            result = True
            for title_nr in range(len(cont)):
                if title_nr == 0:
                    result &= self.__video_options_xml(writer, cont[title_nr])
                    # a bug? in dvdauthor forces us to use only one title's audio & subs
                    # todo: pick the title with most audio/subtitle streams
                    result &= self.__audio_options_longest_list_xml(writer, cont)
                    result &= self.__subtitle_options_longest_list_xml(writer, cont)
                #result &= self.__audio_options_xml(writer, cont[title_nr])
                #result &= self.__subtitle_options_xml(writer, cont[title_nr])
                result &= self.__title_xml(writer, cont[title_nr])

            self.__end_titleset_xml(writer)
            self.__end_xml(writer)

            writer.write()
            self.__exec_log_text += ('# created %s\n' % fname)

        return result

    def __start_xml(self, writer, output_dir):
        writer.stag('dvdauthor', {'dest' : output_dir})
        return

    def __end_xml(self, writer):
        writer.decrement()
        writer.etag('dvdauthor')
        return

    def __start_menu_xml(self, writer):
        # no menus right now
        writer.increment()
        writer.tag('vmgm', {})
        return

    def __end_menu_xml(self, writer):
        # no menus right now
        return

    def __start_titleset_xml(self, writer):
        writer.stag('titleset', {})

        writer.increment()
        writer.stag('titles', {})
        return

    def __end_titleset_xml(self, writer):
        writer.decrement()
        writer.etag('titles')

        writer.decrement()
        writer.etag('titleset')
        return

    def __video_options_xml(self, writer, title):
        vstreams = title.get_video()
        if len(vstreams) < 1:
            return False

        # todo: video[0]
        vstream = vstreams[0]

        vformat = vstream.get_format_str()
        vaspect = vstream.get_aspect('dvdauthor')

        vres = self.__get_resolution(vstream)
        wscr = 'nopanscan'

        opts = {'widescreen' : wscr, 'resolution' : vres, 'aspect' : vaspect, 'format' : vformat}

        if vaspect == '4:3':
            del(opts['widescreen'])

        writer.tag('video', opts)

        return True

    def __get_resolution(self, vstream):
        cfg = self.get_context().get_config()
        resol = cfg.get_video_resolution_dvd()
        if resol.find('x') < 0:
            sx = '%d' % vstream.get_width()
            sy = '%d' % vstream.get_height()
        else:
            sx, sy = cfg.get_video_resolution_dvd().split('x')
        vres = ('%sx%s' % (sx, sy))
        return vres

    def __check_video_resolution(self, resol):
        #cfg = self.get_context().get_config()
        #if not cfg.get_show_popups():
        #    return

        if not resol in ['720x576', '704x576', '352x576', '352x288']:
            primary = _('Creating non standard DVD with video resolution %s') % resol
            secondary = _('You decided to use the original resolution for the video. However it is non standard for a DVD. You may have troubles playing this DVD with some players. Allowed video resulutions are 720x576, 704x576, 352x576 and 352x288 pixels')
            dlg_error = AlertError()
            dlg_error.show(primary, secondary, modal = False)
            return True
        return False

    def __audio_options_xml(self, writer, title):
        streams = title.get_audio()
        for audio in streams:
            alang = audio.get_language()
            writer.tag('audio', {'lang' : alang})

        return True

    def __subtitle_options_xml(self, writer, title):
        streams = title.get_subtitles()
        for subtitle in streams:
            slang = subtitle.get_language()
            writer.tag('subpicture', {'lang' : slang})

        return True

    def __audio_options_longest_list_xml(self, writer, cont):
        streams = []
        for title in cont:
            if len(title.get_audio()) > len(streams):
                streams = title.get_audio()

        for audio in streams:
            alang = audio.get_language()
            writer.tag('audio', {'lang' : alang})

        return True

    def __subtitle_options_longest_list_xml(self, writer, cont):
        streams = []
        for title in cont:
            if len(title.get_subtitles()) > len(streams):
                streams = title.get_subtitles()

        for subtitle in streams:
            slang = subtitle.get_language()
            writer.tag('subpicture', {'lang' : slang})

        return True

    def __title_xml(self, writer, title):
        palette_file = title.get_palette_file()
        vstreams = title.get_video()
        if len(vstreams) < 1:
            return False

        if not palette_file:
            writer.stag('pgc', {})
        else:
            writer.stag('pgc', {'palette' : palette_file})
        writer.increment()

        ch_start = title.get_first_chapter_id()
        ch_end = title.get_last_chapter_id()
        chapt_info = self.expand_chapter_list(ch_start, ch_end, vstreams[0].get_filename())

        ch_pos = 0
        for start_ch, end_ch, vname in chapt_info:
            chapters = self.__create_chapter_times(title, start_ch, end_ch, ch_pos)
            ch_pos += 1
            writer.tag('vob', {'chapters' : chapters, 'file' : vname})

        writer.decrement()
        writer.etag('pgc')

        return True

    def __create_chapter_times(self, title, start_ch, end_ch, ch_pos):
        total = 0.
        time_str = sec2str(total)

        chapters = title.get_chapters()
        if len(chapters) < 1:
            return None

        # chapter rip
        if start_ch == end_ch:
            for i in range(ch_pos):
                total += chapters[i].get_length()
                time_str = sec2str(total)

            time_str += (',%s' % sec2str(total + chapters[ch_pos].get_length()))
        return time_str

        # title rip
        for chapter in chapters:
            total += chapter.get_length()
            time_str += (',%s' % sec2str(total))

        return time_str

    def __exec_filter_video(self, command_data, command):
        assert(len(command_data) == 2)
        disc_nr, title_nr = command_data

        cont = self.get_context().get_dest()[disc_nr]
        title = cont[title_nr]

        #fname = 'video/disc%d_title%d_filtered.m2v' % (disc_nr + 1, title_nr + 1)
        fname = self.create_stream_name(disc_nr + 1, title_nr + 1, 'video', 'filtered.m2v')
        tool_checker = self.get_context().get_project().get_tool_checker()

        constr = ConstructorMpegVideo(tool_checker)
        constr.set_source(title.get_video_at(0).get_filename())
        constr.set_source_type(CONTAINER_FILE)
        constr.set_outname(fname)

        success, shell_cmd = constr.construct()
        if not success:
            command.set_error(_('You can get help in the winki forum located at http://www.winki-the-ripper.de'))
            return False

        self.__exec_log_text = ('%s\n\n' % shell_cmd)

        result = command.run_command(shell_cmd)
        if result:
            title.get_video_at(0).set_filename(fname)

        return result

    def __parse_dvdauthor_output(self, line):
        progress = 0.0
        estimation = ''

        line_start = line.find('VOBU')
        if line_start > -1:
            match = self.__dvdauthor_regex.match(line[line_start : ])
            if match:
                mb, blah, percent = match.group(1, 2, 3)

                # first pass
                if not percent:
                    mb = to_int(mb)
                    if mb < 0:
                        mb = 0.
                    else:
                        progress = self.scale_progress(mb / self.__video_files_size / 2.)

                # second pass
                else:
                    percent = to_int(percent)
                    if percent < 0:
                        percent = 0.
                    else:
                        progress = self.scale_progress((percent / 100. / 2.) + 0.5)

        return progress, estimation


    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            return

        fobj.write('%s' % self.__exec_log_text)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
