# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker_stream import WorkerStream
from winkirip.command_ui import UICommand
from winkirip.container import CONTAINER_DVD, CONTAINER_FILE, FILE_AVI
from winkirip.container import FILE_MATROSKA, FILE_MP4, FILE_MOBILE
from winkirip.container_video import VIDEO_H264
from winkirip.container_subtitle import SUBTITLE_SRT, SUBTITLE_SUB, SUBTITLE_VOBSUB
from winkirip.lib import debug, error, move, fix_fname, build_path, mkdir
from winkirip.lib import set_extension, to_float, to_int
from winkirip.constructor_container import ConstructorAvi, ConstructorMp4
from winkirip.constructor_container import ConstructorMkv, Constructor3GPMobile
from winkirip.constructor_mplex import ConstructorMplex
from winkirip.constructor_move import ConstructorMove
from winkirip.constructor_spumux import ConstructorSpumux
from os.path import split
from os import unlink
import re

class WorkerMultiplex(WorkerStream):
    def __init__(self, context, dest_id):
        WorkerStream.__init__(self, context, dest_id, self.get_ui_name())
        self.__vstreams = []
        self.__astreams = []
        self.__sstreams = []
        self.__outname = None
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def init_stream_list(self, dest_cid):
        result = []

        dest_container = self.get_context().get_dest()[dest_cid]
        for dest_title in dest_container:
            result.append(dest_title)
        return result

    def prepare_ui_commands(self):
        return

    def create_main_commands(self, title, title_idx):
        ui_command = UICommand()
        ui_command.set_data([title])
        ui_command.set_can_progress(self.can_progress())
        ui_command.set_can_estimate(self.can_estimate())
        if self.can_progress() or self.can_estimate():
            ui_command.set_parse_callback(self.get_parse_callback())
        return [ui_command]

    def create_post_command(self, title, title_idx):
        # todo: copy subtitles to the output directory
        return None

    def prepare_exec(self, command_data, command):
        assert(len(command_data) == 1)
        title, = command_data

        cid = title.get_container_id()
        if not mkdir('%d' % (cid + 1)): # +1: just like in __create_file_name(). sigh - fixme
            debug('WorkerMultiplex.prepare_exec()', 'cannot create output dir %d' % (cid + 1), error)
            command.set_error(_('Cannot create output folder %d') % (cid + 1))
            return False

        self.__outname = self.__create_file_name(title.get_container_id(), self.get_extension(), title.get_id())
        self.__vstreams = title.get_video()
        self.__astreams = title.get_audio()
        self.__sstreams = title.get_subtitles()

        # fixme: if we don't call use_streams() the base class is very unhappy
        if len(self.__vstreams) > 0:
            self.use_stream(self.__vstreams[0])
        elif len(self.__astreams) > 0:
            self.use_stream(self.__astreams[0])
        elif len(self.__sstreams) > 0:
            self.use_stream(self.__sstreams[0])
        else:
            return False
        return True

    def get_outname(self):
        return self.__outname

    def construct(self, start_ch, end_ch, outname):
        cfg = self.get_context().get_config()

        self.pre_construct(start_ch, end_ch, outname)

        constr = self.get_constructor(start_ch, end_ch, outname)
        if not constr:
            debug('WorkerMultiplex.construct()', 'cannot create constructor', error)
            return None

        constr.set_test_rip(cfg.get_test_rip(), self.get_dst_title().get_length())

        success, shell_cmd = constr.construct()
        if not success:
            debug('WorkerMultiplex.construct()', 'command construction failed', error)
            self.set_error(_('Command construction failed'))
            return None

        return shell_cmd

    def extract(self, shell_cmd, command):
        return command.run_command(shell_cmd)

    def update_failure(self):
        debug('WorkerMultiplex.update_failure()', 'execution failed', error)
        return

    def update_success(self):
        return

    # -- common functions for the derived classes --

    def get_vlist(self):
        return self.__vstreams

    def get_alist(self):
        return self.__astreams

    def get_slist(self):
        return self.__sstreams

    def can_progress(self):
        return False

    def can_estimate(self):
        return False

    def get_parse_callback(self):
        return None

    def get_ui_name(self):
        return _('Multiplex streams')

    def pre_construct(self, start_ch, end_ch, outname):
        return

    def get_constructor(self, start_ch, end_ch, outname):
        # should be implemented by derived classes
        assert(False)
        return None

    def get_extension(self):
        assert(False)
        return ''

    # -- helpers --

    def __create_file_name(self, disc_nr, ext, title_nr = -1):
        name = self.get_context().get_project().get_dest()[disc_nr].get_name()
        if not name or not len(name):
            name = 'disk'

        title = ''
        if title_nr > -1:
            title = '_%d' % (title_nr + 1)

        output_fname = ''
        if len(self.get_context().get_dest()) == 1:
            output_fname = '%s%s.%s' % (fix_fname(name), title, ext)
        else:
            output_fname = '%s_%d%s.%s' % (fix_fname(name), disc_nr + 1, title, ext)

        return build_path(['%d' % (disc_nr + 1), output_fname])


class WorkerDVD(WorkerMultiplex):
    def __init__(self, context, dest_id):
        WorkerMultiplex.__init__(self, context, dest_id)
        self.cur_obj = None
        self.__outname = ''
        self.__tempfiles = []
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def prepare_exec(self, command_data, command):
        assert(len(command_data) == 4)
        video, streams, dest_cid, dest_tid = command_data

        self.use_stream(video)
        self.cur_obj = streams
        self.__outname = self.create_stream_name(dest_cid, dest_tid, 'video', 'mpeg')

        self.exec_hook(video)
        return True

    def get_outname(self):
        return self.__outname

    def extract(self, shell_cmd, command):
        return command.run_command(shell_cmd, from_stderr = True)

    def create_post_command(self, title, title_idx):
        ui_command = UICommand()
        ui_command.set_command(self.__unlink_temp)
        ui_command.set_data([title])
        return ui_command

    # -- methods inherited from worker_multiplex.WorkerMultiplex --

    def pre_construct(self, start_ch, end_ch, outname):
        tname = set_extension(outname, 'temp')

        if not move(outname, tname):
            self.set_error(_('Cannot move %(video_file)s to %(temp_file)s') % {'video_file':outname, 'temp_file':tname})
            debug('WorkerDVD.pre_construct()', 'cannot move %s to %s' % (outname, tname), error)
            return False
        else:
            self.log_append('mv %s %s\n' % (outname, tname))
            self.__tempfiles.append(tname)
        return

    # -- functions for derived classes --

    def exec_hook(self, video):
        return

    def __unlink_temp(self, command_data, command):
        if len(self.__tempfiles) < 1:
            return True

        fnames = ''
        for fname in self.__tempfiles:
            fnames += (' %s' % fname)
        self.log_append('rm %s' % fnames)

        self.log_append('\n')
        self.unlink_temp(self.__tempfiles)
        return True

class WorkerDVDAudioVideo(WorkerDVD):
    def __init__(self, context, dest_id):
        WorkerDVD.__init__(self, context, dest_id)
        self.__mplex_regex = re.compile('Video.*?frame=(\d+)\s+sector=')
        self.__frames = 1.
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def create_main_commands(self, title, title_idx):
        vstreams = title.get_video()
        astreams = title.get_audio()

        if len(vstreams) != 1:
            debug('WorkerDVDAudioVideo.create_main_commands()', 'cannot multiplex multiple video streams', error)
            return []

        # without audio dvdauthor will be very unhappy
        if len(astreams) < 1:
            debug('WorkerDVDAudioVideo.create_main_commands()', 'no audio streams', error)
            return []

        dest_cid = title.get_container_id() + 1 # todo: quiet unreliable?
        dest_tid = title.get_id() + 1 # todo: quiet unreliable?

        result = []
        if len(astreams) > 0:
            ui_command = UICommand()
            ui_command.set_can_progress(True)
            ui_command.set_parse_callback(self.__parse_mplex_output)
            ui_command.set_data([vstreams[0], astreams, dest_cid, dest_tid])
            result.append(ui_command)

        return result

    # -- methods inherited from worker_multiplex.WorkerMultiplex --

    def get_ui_name(self):
        return _('Multiplex video and audio')

    def get_constructor(self, start_ch, end_ch, outname):
        tool_checker = self.get_context().get_project().get_tool_checker()

        constr = ConstructorMplex(tool_checker)
        constr.set_outname(outname)

        tname = set_extension(outname, 'temp')
        constr.set_vfile(tname)

        for audio in self.cur_obj:
            chapt_info = self.expand_chapter_list(start_ch, end_ch, audio.get_filename())
            s, e, audio_fname = chapt_info[0]
            constr.add_afile(audio_fname)
        return constr

    # -- methods inherited from worker_multiplex.WorkerDVD --

    def exec_hook(self, video):
        fps = video.get_fps()
        vlen = to_float(self.get_dst_title().get_length())
        if vlen > 1e-3:
            self.__frames = vlen * fps
        return

    # -- helpers --

    def __parse_mplex_output(self, line):
        progress = 0.

        line_start = line.find('Video')
        if line_start > -1:
            match = self.__mplex_regex.match(line[line_start : ])
            if match:
                item = match.group(1)
                progress = to_int(item)
                if progress < 0:
                    progress = 0.
                else:
                    progress = self.scale_progress(progress / self.__frames)

        return progress, ''

class WorkerDVDSubtitles(WorkerDVD):
    def __init__(self, context, dest_id):
        WorkerDVD.__init__(self, context, dest_id)
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def create_main_commands(self, title, title_idx):
        vstreams = title.get_video()
        if len(vstreams) != 1:
            debug('WorkerDVDSubtitles.create_main_commands()', 'cannot multiplex multiple video streams', error)
            return []

        sstreams = title.get_subtitles()
        dest_cid = title.get_container_id() + 1 # todo: quiet unreliable?
        dest_tid = title.get_id() + 1 # todo: quiet unreliable?

        result = []
        for subtitle in sstreams:
            ui_command = UICommand()
            ui_command.set_data([vstreams[0], subtitle, dest_cid, dest_tid])
            result.append(ui_command)

        return result

    # -- methods inherited from worker_multiplex.WorkerMultiplex --

    def get_ui_name(self):
        return _('Multiplex video and subtitle')

    def get_constructor(self, start_ch, end_ch, outname):
        tool_checker = self.get_context().get_project().get_tool_checker()

        subtitle = self.cur_obj
        subname = self.get_name_for_chapters(start_ch, end_ch, subtitle.get_filename())
        subpos = self.get_dst_title().get_subtitle_pos(subtitle.get_id())

        tname = set_extension(outname, 'temp')
        # the subtitle was removed from the container because it does not contain any data
        # and this would confuse mplex
        if subpos < 0:
            constr = ConstructorMove()
            constr.set_inname(tname)
            constr.set_outname(outname)
            debug('WorkerDVDSubtitles.get_constructor()', 'subtitle was removed. renaming %s to %s' % (tname, outname),  error)
            return constr
        
        constr = ConstructorSpumux(tool_checker)
        constr.set_inname(tname)
        constr.set_outname(outname)
        constr.set_subname(subname)
        constr.set_subpos(subpos)
        success, shell_cmd = constr.construct()
        return constr



class WorkerAVI(WorkerMultiplex):
    def __init__(self, context, dest_id):
        WorkerMultiplex.__init__(self, context, dest_id)
        self.__avimerge_regex = re.compile('\(\d+?-(\d+)\)')
        self.__frames = 1.
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def extract(self, shell_cmd, command):
        return command.run_command(shell_cmd, from_stderr = True)

    # -- methods inherited from worker_multiplex.WorkerMultiplex --

    def get_extension(self):
        return 'avi'

    def get_constructor(self, start_ch, end_ch, outname):
        tool_checker = self.get_context().get_project().get_tool_checker()

        constr = ConstructorAvi(tool_checker)
        constr.set_outname(self.get_outname())

        fps = 1.
        vlist = self.get_vlist()
        if len(vlist) > 0:
            constr.add_video(vlist[0].get_filename())
            fps = vlist[0].get_fps()

        for astream in self.get_alist():
            constr.add_audio(astream.get_filename())

        vlen = to_float(self.get_dst_title().get_length())
        if vlen > 1e-3:
            self.__frames = vlen * fps
        return constr

    def can_progress(self):
        return True

    def get_parse_callback(self):
        return self.__parse_avimerge_output

    # -- helpers --

    def __parse_avimerge_output(self, line):
        progress = 0.

        line_start = line.find('(')
        if line_start > -1:
            match = self.__avimerge_regex.match(line[line_start : ])
            if match:
                item = match.group(1)
                progress = to_int(item)
                if progress < 0:
                    progress = 0.
                else:
                    progress = self.scale_progress(progress / self.__frames)

        return progress, ''



class WorkerMKV(WorkerMultiplex):
    def __init__(self, context, dest_id):
        WorkerMultiplex.__init__(self, context, dest_id)
        self.__mkvmerge_regex = re.compile('progress:\s+(\d+)%')
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def extract(self, shell_cmd, command):
        return command.run_command(shell_cmd, success_codes = [0, 1, 256], from_stderr = True)

    # -- methods inherited from worker_multiplex.WorkerMultiplex --

    def get_extension(self):
        return 'mkv'

    def get_constructor(self, start_ch, end_ch, outname):
        tool_checker = self.get_context().get_project().get_tool_checker()
        lang_conv = self.get_context().get_project().get_iso_codes()

        cidx = self.get_stream().get_container_id() - 1
        tidx = self.get_stream().get_title_id() - 1
        constr = ConstructorMkv(tool_checker, lang_conv, cidx, tidx)
        constr.set_outname(self.get_outname())
        vlist = self.get_vlist()
        if len(vlist) > 0:
            video = vlist[0]
            constr.add_video(video)
            constr.set_fps(video.get_fps())

        constr.set_tag_file(self.get_dst_cont().get_tag_file())
        constr.set_chapter_file(self.get_dst_cont().get_chapter_file())
        constr.set_title(self.get_dst_cont().get_name())

        alist = self.get_alist()
        for astream in alist:
            apos = self.get_dst_title().get_audio_pos(astream.get_id())
            # the audio was removed from the container because it does not contain any data
            if apos > -1:
                constr.add_audio(astream)

        slist = self.get_slist()
        for sstream in slist:
            subpos = self.get_dst_title().get_subtitle_pos(sstream.get_id())
            # the subtitle was removed from the container because it does not contain any data
            if subpos > -1:
                constr.add_subtitle(sstream)
        return constr

    def can_progress(self):
        return True

    def get_parse_callback(self):
        return self.__parse_mkvmerge_output

    # -- helpers --

    def __parse_mkvmerge_output(self, line):
        progress = 0.

        line_start = line.find('progress')
        if line_start > -1:
            match = self.__mkvmerge_regex.match(line[line_start : ])
            if match:
                item = match.group(1)
                progress = to_float(item)
                if progress < 0:
                    progress = 0.
                else:
                    progress = self.scale_progress(progress / 100.)

        return progress, ''


class WorkerMP4(WorkerMultiplex):
    def __init__(self, context, dest_id):
        WorkerMultiplex.__init__(self, context, dest_id)
        self.__mp4_regex = re.compile('(Importing:|Writing:).+?\((\d+)\/\d+\)')
        self.__pass_count = 0
        self.__cur_pass = 0
        self.__last_progress = -1.
        return

    # -- methods inherited from worker_multiplex.WorkerMultiplex --

    def get_extension(self):
        return 'mp4'

    def get_constructor(self, start_ch, end_ch, outname):
        tool_checker = self.get_context().get_project().get_tool_checker()
        lang_conv = self.get_context().get_project().get_iso_codes()

        constr = ConstructorMp4(tool_checker, lang_conv, (self.get_stream().get_container_id() - 1))
        constr.set_outname(self.get_outname())
        vlist = self.get_vlist()
        if len(vlist) > 0:
            constr.add_video(vlist[0])
            self.__pass_count = 1

        alist = self.get_alist()
        for astream in alist:
            apos = self.get_dst_title().get_audio_pos(astream.get_id())
            # the audio was removed from the container because it does not contain any data
            if apos > -1:
                constr.add_audio(astream)

        slist = self.get_slist()
        for sstream in slist:
            if sstream.get_format() == SUBTITLE_SUB or sstream.get_format() == SUBTITLE_SRT or sstream.get_format() == SUBTITLE_VOBSUB:
                spos = self.get_dst_title().get_subtitle_pos(sstream.get_id())
                # the subtitle was removed from the container because it does not contain any data
                if spos > -1:
                    constr.add_subtitle(sstream)

        self.__pass_count += (len(alist) + len(slist))
        if self.__pass_count < 1:
            return None

        self.__pass_count += 1
        return constr

    def can_progress(self):
        return True

    def get_parse_callback(self):
        return self.__parse_mp4_output

    # -- helpers --

    def __parse_mp4_output(self, line):
        progress = 0.

        line_start1 = line.find('Importing')
        line_start2 = line.find('Writing')
        if line_start1 > -1 or line_start2 > -1:
            line_start = max(line_start1, line_start2)
            match = self.__mp4_regex.match(line[line_start : ])
            if match:
                step_name, percent = match.group(1, 2)
                progress = to_float(percent)
                if progress < 0:
                    progress = 0.
                else:
                    progress = progress / 100.

                if progress < self.__last_progress:
                    self.__cur_pass += 1
                self.__last_progress = progress

        stream_portion = float(self.__cur_pass) / float(self.__pass_count)
        scaled = progress / float(self.__pass_count)
        progress = self.scale_progress(stream_portion + scaled)
        return progress, ''

class WorkerMobile(WorkerMultiplex):
    def __init__(self, context, dest_id):
        WorkerMultiplex.__init__(self, context, dest_id)
        self.__stream_len = 1.
        self.__ffmpeg_regex = re.compile('.*?time=(\d+\.\d+)')
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def extract(self, shell_cmd, command):
        return command.run_command(shell_cmd, from_stderr = True)

    # -- methods inherited from worker_multiplex.WorkerMultiplex --

    def get_extension(self):
        return '3gp'

    def get_constructor(self, start_ch, end_ch, outname):
        tool_checker = self.get_context().get_project().get_tool_checker()
        lang_conv = self.get_context().get_project().get_iso_codes()

        constr = Constructor3GPMobile(tool_checker, lang_conv, (self.get_stream().get_container_id() - 1))
        constr.set_outname(self.get_outname())
        vlist = self.get_vlist()
        if len(vlist) > 0:
            constr.add_video(vlist[0])

        for astream in self.get_alist():
            apos = self.get_dst_title().get_audio_pos(astream.get_id())
            # the audio was removed from the container because it does not contain any data
            if apos > -1:
                constr.add_audio(astream)

        self.__stream_len = self.get_dst_title().get_length()
        return constr

    def can_progress(self):
        return True

    def get_parse_callback(self):
        return self.__parse_ffmpeg_output

    # -- helpers --

    def __parse_ffmpeg_output(self, line):
        progress = 0.

        line_start = line.find('frame=')
        if line_start > -1:
            match = self.__ffmpeg_regex.match(line[line_start : ])
            if match:
                item = match.group(1)
                progress = to_float(item)
                if progress < 0:
                    progress = 0.
                else:
                    progress = self.scale_progress(progress / self.__stream_len)

        return progress, ''


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
