# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker
from winkirip.command_ui import UICommand
from winkirip.lib import file_size, split_path, build_path, debug, error
from winkirip.container import CONTAINER_DVD, CONTAINER_FILE
from winkirip.container_chapter import Chapter
from winkirip.container_video import VIDEO_MPEG2

class WorkerCalcSize(Worker):
    def __init__(self, context):
        Worker.__init__(self, context, self.__create_commands, _('Calculate file sizes'))
        self.__exec_log_text = ''
        return

    # -- helpers --

    def __create_commands(self, command_list):
        dest = self.get_context().get_project().get_dest()
        if not dest or len(dest) < 1:
            return False

        if dest[0].get_type() == CONTAINER_DVD:
            return self.__create_dvd(command_list)

        elif dest[0].get_type() == CONTAINER_FILE:
            return self.__create_file(command_list)

        return False

    # -- dvd --

    def __create_dvd(self, command_list):
        disc_size = 4370 * 1024 * 1024
        admin_size = disc_size * 0.05

        cfg = self.get_context().get_project().get_config()
        preferred_bitrate = cfg.get_preferred_bitrate(VIDEO_MPEG2)

        dest = self.get_context().get_dest()
        for disc_nr in range(len(dest)):
            cont = dest[disc_nr]
            disc_size = cont.get_size() * 1024 * 1024
            admin_size = disc_size * 0.05

            ui_command = UICommand()
            ui_command.set_command(self.__exec_dvd)
            ui_command.set_data([disc_nr, disc_size, admin_size, preferred_bitrate])
            ui_command.set_exec_log_callback(self.__exec_log)
            command_list.append(ui_command)

        return True

    def __exec_dvd(self, command_data, command):
        assert(len(command_data) == 4)
        disc_nr, disc_size, admin_size, preferred_bitrate = command_data

        self.__exec_log_text = ''

        list_afiles = []
        list_vfiles = []
        seconds = 0

        cont = self.get_context().get_dest()[disc_nr]
        for title in cont:
            ch_start = title.get_first_chapter_id()
            ch_end = title.get_last_chapter_id()

            audio_list = title.get_audio()
            afiles = []
            for audio in audio_list:
                chapt_info = self.expand_chapter_list(ch_start, ch_end, audio.get_filename())
                for s, e, afile_name in chapt_info:
                    afiles.append(afile_name)
            list_afiles.append(afiles)

            video_list = title.get_video()
            vfiles = []
            for video in video_list:
                chapt_info = self.expand_chapter_list(ch_start, ch_end, video.get_filename())
                for s, e, vfile_name in chapt_info:
                    vfiles.append(vfile_name)
            list_vfiles.append(vfiles)

            seconds += title.get_length()

        asize = 0
        for title_nr in range(len(list_afiles)):
            asize += self.__sum_sizes(list_afiles[title_nr])

        vsize = 0
        for title_nr in range(len(list_vfiles)):
            vsize += self.__sum_sizes(list_vfiles[title_nr])

        free_space = disc_size - admin_size - asize

        cfg = self.get_context().get_project().get_config()
        shrink_method = cfg.get_video_shrink_method_dvd()
        if shrink_method == 'requantize':
            qfactor = self.__calc_quant_factor(free_space, vsize)
            for title in cont:
                video_list = title.get_video()
                for video in video_list:
                    video.set_quant_factor(qfactor)

            self.__exec_log_text += '# disc %d: vfiles: %s; afiles: %s; disc_size = %d; admin_size = %d; aize = %d; vsize = %d; qfactor = %f\n' % (disc_nr, list_vfiles, list_afiles, disc_size, admin_size, asize, vsize, qfactor)

        # always calculate video bitrate since we may fall back from
        # "requantize" to "1 pass" in worker_video_dvd.py
        vbitrate, do_shrink = self.calc_vbitrate(disc_size, admin_size, asize, seconds, vsize, preferred_bitrate)

        for title in cont:
            video_list = title.get_video()
            for video in video_list:
                if video.get_bitrate() < 1:
                    video.set_bitrate(vbitrate)
                else:
                    vbitrate = video.get_bitrate() # for log printing only
                video.set_shrink_needed(do_shrink)

            self.__exec_log_text += '# disc %d: vfiles: %s; afiles: %s; disc_size = %d; admin_size = %d; aize = %d; vsize = %d; vbitrate = %f\n\n' % (disc_nr, list_vfiles, list_afiles, disc_size, admin_size, asize, vsize, vbitrate)

        return True

    def __calc_quant_factor(self, free_space, vsize):
        quant = 0.0
        if abs(free_space) > 1e-30:
            if vsize > free_space:
                quant = vsize / free_space
        else:
            self.__exec_log_text += ('oops: free space = %d\n' % free_space)
            debug('WorkerSizeDVD.__calc_quant_factor()', 'Not enough space on target disc', error)
            return False

        if quant < 1.0:
            quant = 1.0

        return quant

    # -- file --

    def __create_file(self, command_list):
        disc_size = 700 * 1024 * 1024
        admin_size = disc_size * 0.005 # matroska

        cfg = self.get_context().get_project().get_config()
        preferred_bitrate = 800

        dest = self.get_context().get_dest()
        for disc_nr in range(len(dest)):
            cont = self.get_context().get_dest()[disc_nr]
            # no size limits requested
            if cont.get_size() < 0:
                disc_size = -1
                admin_size = -1.
            else:
                disc_size = cont.get_size() * 1024 * 1024
                admin_size = disc_size * 0.05

            first_title = cont[0]
            video_streams = first_title.get_video()
            if len(video_streams) > 0:
                preferred_bitrate = cfg.get_preferred_bitrate(video_streams[0].get_type())

            ui_command = UICommand()
            ui_command.set_command(self.__exec_file)
            ui_command.set_data([disc_nr, disc_size, admin_size, preferred_bitrate])
            ui_command.set_exec_log_callback(self.__exec_log)
            command_list.append(ui_command)

        return True

    def __exec_file(self, command_data, command):
        assert(len(command_data) == 4)
        disc_nr, disc_size, admin_size, preferred_bitrate = command_data

        self.__exec_log_text = ''

        list_vfiles = []
        list_afiles = []
        list_sfiles = []
        seconds = 0
        title_nr = 0

        cont = self.get_context().get_dest()[disc_nr]
        for title in cont:
            ch_start = title.get_first_chapter_id()
            ch_end = title.get_last_chapter_id()

            video_list = title.get_video()
            vfiles = []
            for video in video_list:
                chapt_info = self.expand_chapter_list(ch_start, ch_end, video.get_filename())
                for s, e, vfile_name in chapt_info:
                    vfiles.append(vfile_name)
            list_vfiles.append(vfiles)

            audio_list = title.get_audio()
            afiles = []
            for audio in audio_list:
                chapt_info = self.expand_chapter_list(ch_start, ch_end, audio.get_filename())
                for s, e, afile_name in chapt_info:
                    afiles.append(afile_name)
            list_afiles.append(afiles)

            subtitle_list = title.get_subtitles()
            sfiles = []
            for subtitle in subtitle_list:
                spath, sfname, sext = split_path(subtitle.get_filename())
                chapt_info = self.expand_chapter_list(ch_start, ch_end, '%s.sub' % build_path([spath, sfname]))
                for s, e, sfile_name in chapt_info:
                    sfiles.append(sfile_name)
            list_sfiles.append(sfiles)

            seconds += title.get_length()

            title_nr += 1

        asize = 0
        for title_nr in range(len(list_afiles)):
            asize += self.__sum_sizes(list_afiles[title_nr])

        ssize = 0
        for title_nr in range(len(list_sfiles)):
            ssize += self.__sum_sizes(list_sfiles[title_nr])

        vsize = 0
        for title_nr in range(len(list_vfiles)):
            vsize += self.__sum_sizes(list_vfiles[title_nr])

        occupied = asize + ssize
        vbitrate, do_shrink = self.calc_vbitrate(disc_size, admin_size, occupied, seconds, vsize, preferred_bitrate)

        for title in self.get_context().get_dest()[disc_nr]:
            video_list = title.get_video()
            for video in video_list:
                if video.get_bitrate() < 1:
                    video.set_bitrate(vbitrate)
                else:
                    vbitrate = video.get_bitrate() # for log printing only
                video.set_shrink_needed(do_shrink)

        self.__exec_log_text += '# disc %d: sfiles: %s; afiles: %s; disc_size = %d; admin_size = %d; aize = %d; ssize = %d; vbitrate = %f\n\n' % (disc_nr, list_sfiles, list_afiles, disc_size, admin_size, asize, ssize, vbitrate)

        return True

    # -- common --

    def __sum_sizes(self, file_list):
        result = 0

        for fname in file_list:
            fsize = file_size(fname)
            if fsize < 0:
                debug('WorkerCalcSize.__sum_sizes()', 'skipping %s from size calculation' % fname)
            else:
                result += fsize

        return result

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            fobj.write('no size information yet\n')
            return

        fobj.write('%s' % self.__exec_log_text)
        return

# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
