# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import Worker, EMPTY_FILE
from winkirip.lib import debug, error, sec2str, to_float
import re

class WorkerStream(Worker):
    def __init__(self, context, source_id, ui_text):
        Worker.__init__(self, context, self.__create, ui_text)
        self.__source_container_id = source_id
        self.__current_stream = None
        self.__stream_list = []
        self.__error_text = None
        self.__exec_log_text = ''
        self.__mplayer_regex = re.compile('Pos:\s*(\d+\.\d+)s\s*\d+?f\s*\(\s*(\d+)%\).*?Trem:\s*(\d+)min')
        return

    # -- public interface --

    def set_error(self, text):
        self.__error_text = text
        return

    def get_stream_list(self):
        return self.__stream_list

    def get_stream(self):
        return self.__current_stream

    def init_stream_list(self, src_cid):
        # should be implemented by derived classes
        assert(False)
        return []

    def prepare_ui_commands(self):
        # should be implemented by derived classes
        assert(False)
        return

    def create_main_commands(self, stream, stream_idx):
        # should be implemented by derived classes
        assert(False)
        return None

    def create_post_command(self, stream, stream_idx):
        # should be implemented by derived classes
        assert(False)
        return None

    def prepare_exec(self, command_data, command):
        # should be implemented by derived classes
        assert(False)
        return False

    def get_outname(self):
        # should be implemented by derived classes
        assert(False)
        return ''

    def construct(self, start_ch, end_ch, outname):
        # should be implemented by derived classes
        assert(False)
        return None

    def extract(self, shell_cmd, command):
        # should be implemented by derived classes
        assert(False)
        return False

    def update_failure(self):
        # should be implemented by derived classes
        assert(False)
        return

    def update_success(self):
        # should be implemented by derived classes
        assert(False)
        return

    def exec_remove_empty_file(self, command_data, command):
        assert(len(command_data) == 1)
        stream, = command_data
        return self.remove_stream_if_empty_file(stream, EMPTY_FILE)

    def parse_mplayer_output(self, line):
        progress = 0.
        estimation = ''

        line_start = line.find('Pos:')
        if line_start > -1:
            match = self.__mplayer_regex.match(line[line_start : ])
            if match:
                ready, percentage, eta = match.group(1, 2, 3)
                progress = to_float(percentage)
                ready = to_float(ready)
                if progress < 0:
                    progress = 0.
                elif abs(progress) < 1e-3 and ready > -1. and self.__stream_len > 0:
                    progress = self.scale_progress(ready / self.__stream_len)
                else:
                    progress = self.scale_progress(progress / 100.)

                est = to_float(eta)
                if est < 0:
                    est = 0.
                else:
                    show_ms = False
                    estimation = sec2str(self.scale_estimation(est * 60), show_ms)

        return progress, estimation

    def get_command_progress_factor(self):
        return 1

    # the functions below will use this stream as starting point
    def use_stream(self, stream):
        self.__current_stream = stream
        return

    def get_stream_id(self):
        return self.__current_stream.get_id()

    def get_src_stream(self):
        return self.__current_stream.get_source()

    def get_src_cid(self):
        return self.get_src_stream().get_container_id()

    def get_src_tid(self):
        return self.get_src_stream().get_title_id()

    def get_src_cont(self):
        return self.get_source_container(self.get_src_cid())

    def get_src_cont_type(self):
        return self.get_source_container(self.get_src_cid()).get_type()

    def get_src_title(self):
        return self.get_source_title(self.get_src_cid(), self.get_src_tid())

    def get_dst_cid(self):
        return self.__current_stream.get_container_id()

    def get_dst_tid(self):
        return self.__current_stream.get_title_id()

    def get_dst_cont(self):
        return self.get_dest_container(self.get_dst_cid())

    def get_dst_cont_type(self):
        return self.get_dest_container(self.get_dst_cid()).get_type()

    def get_dst_title(self):
        return self.get_dest_title(self.get_dst_cid(), self.get_dst_tid())

    def get_device(self, cont):
        return cont.get_device()

    def get_chapter_start_end(self):
        ch_start = self.get_dst_title().get_first_chapter_id()
        ch_end = self.get_dst_title().get_last_chapter_id()
        return ch_start, ch_end

    def log_append(self, text):
        self.__exec_log_text += ('%s\n' % text)
        return

    # -- helpers --

    def __create(self, command_list):
        self.__stream_list = self.init_stream_list(self.__source_container_id)
        command_progress_factor = self.get_command_progress_factor()
        self.prepare_ui_commands()

        if not self.init_command_progress(len(self.__stream_list) * command_progress_factor):
            debug('WorkerStream.__create()', 'cannot init progress commands', error)
            return False

        stream_idx = 0
        for stream in self.__stream_list:
            ui_commands = self.create_main_commands(stream, stream_idx)
            stream_idx += 1
            if len(ui_commands) < 1:
                continue

            for ui_command in ui_commands:
                ui_command.set_command(self.__exec)
                ui_command.set_exec_log_callback(self.__exec_log)
                command_list.append(ui_command)

        stream_idx = 0
        for stream in self.__stream_list:
            ui_command = self.create_post_command(stream, stream_idx)
            stream_idx += 1
            if not ui_command:
                continue

            ui_command.set_exec_log_callback(self.__exec_log)
            command_list.append(ui_command)

        return True

    def __exec(self, command_data, command):
        if not self.prepare_exec(command_data, command):
            debug('WorkerStream.__exec()', 'prepare_exec() failed', error)
            command.set_error(self.__error_text)
            self.log_append('# worker: execution preparation failed')
            return False

        self.next_command_progress()

        outname = self.get_outname()
        ch_start, ch_end = self.get_chapter_start_end()

        chapt_info = self.expand_chapter_list(ch_start, ch_end, outname)
        if not self.init_chapter_progress(len(chapt_info)):
            debug('WorkerStream.__exec()', 'cannot init chapter progress', error)
            return False

        result = True
        for start_ch, end_ch, fname_ch in chapt_info:
            self.next_chapter_progress()

            self.__stream_len = self.get_chapter_len(start_ch - 1, end_ch, self.get_dst_title())

            shell_cmd = self.construct(start_ch, end_ch, fname_ch)
            self.log_append(shell_cmd)
            if not shell_cmd:
                debug('WorkerStream.__exec()', 'construct() failed', error)
                self.log_append('# worker: command construction failed')
                result = False
                break

            if not self.extract(shell_cmd, command):
                debug('WorkerStream.__exec()', 'extract() failed', error)
                self.log_append('# worker: command execution failed')
                result = False
                break

        if not result:
            self.update_failure()
            command.set_error(self.__error_text)
        else:
            self.update_success()
            self.log_append('# done\n')

        return result

    def __exec_log(self, fobj):
        if not self.__exec_log_text or not len(self.__exec_log_text):
            return

        fobj.write('%s' % self.__exec_log_text)
        self.__exec_log_text = ''
        return


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
