# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker import EMPTY_FILE, EMPTY_XML_FILE
from winkirip.worker_stream import WorkerStream
from winkirip.command_ui import UICommand
from winkirip.container import CONTAINER_DVD
from winkirip.container_subtitle import SUBTITLE_IMGFILE, SUBTITLE_VOBSUB
from winkirip.container_subtitle import SUBTITLE_SUB, SUBTITLE_SRT
from winkirip.lib import debug, error, set_extension
from winkirip.constructor_subtitle import ConstructorSubtitleDvd
from winkirip.constructor_subtitle import ConstructorSubtitleVobsub
from os import symlink

class WorkerSubtitles(WorkerStream):
    def __init__(self, context, source_id):
        WorkerStream.__init__(self, context, source_id, _('Extract subtitles'))
        self.__subout_index = 0
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def init_stream_list(self, src_cid):
        return self.get_subtitle_list(src_cid)

    def prepare_ui_commands(self):
        return

    def create_main_commands(self, subtitle, subtitle_idx):
        ui_command = UICommand()
        ui_command.set_parse_callback(self.parse_mplayer_output)
        ui_command.set_can_progress(self.can_progress())
        ui_command.set_can_estimate(self.can_estimate())
        ui_command.set_data([subtitle, subtitle_idx])
        return [ui_command]

    def create_post_command(self, subtitle, subtitle_idx):
        ui_command = UICommand()
        ui_command.set_command(self.__exec_remove_file)
        ui_command.set_data([subtitle])
        return ui_command

    def prepare_exec(self, command_data, command):
        assert(len(command_data) == 2)
        subtitle, subout_index = command_data

        self.use_stream(subtitle)
        self.__subout_index = subout_index
        return True

    def get_outname(self):
        cont_id = self.get_dst_cid()
        title_id = self.get_dst_tid()
        spos = self.get_dst_title().get_subtitle_pos(self.get_stream().get_id())

        return self.create_stream_name(cont_id + 1, title_id + 1, 'subtitles', '', spos + 1)

    def construct(self, start_ch, end_ch, outname):
        constr = self.get_constructor()
        if not constr:
            debug('WorkerSubtitles.construct()', 'cannot create constructor', error)
            return None

        cfg = self.get_context().get_project().get_config()
        device = self.get_src_cont().get_device()

        constr.set_source(device, self.get_src_tid())
        constr.set_source_type(self.get_src_cont().get_type())
        constr.set_test_rip(cfg.get_test_rip(), self.get_dst_title().get_length())
        constr.set_chapters(start_ch, end_ch)
        constr.set_outname(outname)
        constr.set_sid(self.get_stream().get_id())

        success, shell_cmd = constr.construct()
        if not success:
            debug('WorkerSubtitles.construct()', 'command construction failed', error)
            self.set_error(_('Command construction failed'))
            return None

        return shell_cmd

    def extract(self, shell_cmd, command):
        return command.run_command(shell_cmd, from_stderr = False) 
    def update_failure(self):
        debug('WorkerSubtitles.update_failure()', 'execution failed', error)
        return

    # -- common functions for the derived classes --

    def get_subout_index(self):
        return self.__subout_index

    def get_constructor(self):
        # should be implemented by derived classes
        assert(False)
        return None

    def can_progress(self):
        return True

    def can_estimate(self):
        return True

    # -- helpers --

    def __exec_remove_file(self, command_data, command):
        assert(len(command_data) == 1)
        subtitle, = command_data

        cont_id = subtitle.get_container_id()

        check_kind = EMPTY_FILE
        if self.get_dest_container_type(cont_id) == CONTAINER_DVD:
            check_kind = EMPTY_XML_FILE

        return self.remove_stream_if_empty_file(subtitle, check_kind)


class WorkerImgFile(WorkerSubtitles):
    def __init__(self, context, source_id):
        WorkerSubtitles.__init__(self, context, source_id)
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def update_success(self):
        self.get_stream().set_filename(set_extension(self.get_outname(), 'xml'))
        self.get_stream().set_format(SUBTITLE_IMGFILE)
        return

    # -- methods inherited from worker_subtitles.WorkerSubtitles --

    def get_constructor(self):
        tool_checker = self.get_context().get_tool_checker()
        palette = self.get_dst_title().get_palette_file()

        constr = ConstructorSubtitleDvd(tool_checker)
        constr.set_palette(palette)
        return constr

    def can_progress(self):
        return False

    def can_estimate(self):
        return False


class WorkerVobsub(WorkerSubtitles):
    def __init__(self, context, source_id):
        WorkerSubtitles.__init__(self, context, source_id)
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def update_success(self):
        self.get_stream().set_filename(set_extension(self.get_outname(), 'idx'))
        self.get_stream().set_format(SUBTITLE_VOBSUB)
        return

    # -- methods inherited from worker_subtitles.WorkerSubtitles --

    def get_constructor(self):
        tool_checker = self.get_context().get_tool_checker()

        constr = ConstructorSubtitleVobsub(tool_checker)
        constr.set_spos(self.get_subout_index())
        return constr

    def get_outname(self):
        cont_id = self.get_dst_cid()
        title_id = self.get_dst_tid()
        #spos = self.get_dst_title().get_subtitle_pos(self.get_stream().get_id())

        return self.create_stream_name(cont_id + 1, title_id + 1, 'subtitles', '')

class WorkerCopy(WorkerSubtitles):
    def __init__(self, context, source_id):
        WorkerSubtitles.__init__(self, context, source_id)
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def update_success(self):
        return

    # -- methods inherited from worker_subtitles.WorkerSubtitles --

    def get_constructor(self):
        # todo
        #if source_subtitle.get_format() == SUBTITLE_SRT or source_subtitle.get_format() == SUBTITLE_SUB:
        #    symlink(source_subtitle.get_filename(), outname)
        #    result = True
        #else:
        #   extract from mkv?
        return None


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
