# coding: utf-8
# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004-2007 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from winkirip.worker_stream import WorkerStream
from winkirip.command_ui import UICommand
from winkirip.container import CONTAINER_DVD, CONTAINER_FILE
from winkirip.container_video import VIDEO_MPEG2, VIDEO_MPEG4, VIDEO_XVID
from winkirip.container_video import VIDEO_H264, VIDEO_UNKNOWN
from winkirip.constructor_ffmpeg4 import ConstructorFFmpeg4
from winkirip.constructor_xvid import ConstructorXvid
from winkirip.constructor_h264 import ConstructorH264
from winkirip.constructor_mpeg2 import ConstructorMpeg2DVD, ConstructorMobile
from winkirip.constructor_tcextract import ConstructorTcextract
from winkirip.constructor_tcrequant import ConstructorTcrequant
from winkirip.constructor_rawvideo import ConstructorRawvideo
from winkirip.lib import debug, error, move, set_extension

class WorkerVideo(WorkerStream):
    def __init__(self, context, source_id, shrink_method):
        WorkerStream.__init__(self, context, source_id, self.get_ui_name())
        self.__pass_count = shrink_method
        self.__cur_pass = -1
        self.__outname = ''
        self.__tname = ''
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def get_command_progress_factor(self):
        dest_type = self.get_context().get_dest()[0].get_type()

        result = 1
        if dest_type == CONTAINER_DVD:
            if self.__pass_count < 2:
                result = 2 # first pass + filtering
            elif self.__pass_count == 2:
                result = 3 # first pass + 2 pass + filtering
        else:
            if self.__pass_count < 2:
                result = 1 # 1 pass / dump
            else:
                result = self.__pass_count # 1 pass + 2 pass

        return result

    def init_stream_list(self, src_cid):
        return self.get_video_list(src_cid)

    def prepare_ui_commands(self):
        return

    def create_main_commands(self, video, video_idx):
        result = []

        ui_command = UICommand()
        ui_command.set_parse_callback(self.parse_mplayer_output)
        ui_command.set_can_progress(self.can_progress())
        ui_command.set_can_estimate(self.can_estimate())
        ui_command.set_data([video, 1, self.__pass_count])
        result.append(ui_command)

        if self.__pass_count == 2:
            ui_command = UICommand()
            ui_command.set_parse_callback(self.parse_mplayer_output)
            ui_command.set_can_progress(True)
            ui_command.set_can_estimate(True)
            ui_command.set_data([video, 2, self.__pass_count])
            result.append(ui_command)

        # mplex workaround: mplex cannot use the files produced by
        # mencoder which dvdauthor swallows without problems, so we
        # reextract the video stream and mplex will be happy again...
        # this should be done only if we are about to create a dvd

        # cur_pass > 2: filter video
        self.use_stream(video)
        dest_type = self.get_dst_cont_type()
        if self.__pass_count > 0 and dest_type == CONTAINER_DVD:
            ui_command = UICommand()
            ui_command.set_parse_callback(self.parse_mplayer_output)
            ui_command.set_can_progress(True)
            ui_command.set_can_estimate(True)
            ui_command.set_data([video, 3, self.__pass_count])
            result.append(ui_command)
        return result

    def create_post_command(self, video, video_idx):
        # build_outname() sets the current stream to video and _should_ be the first call here
        self.__outname = self.build_outname(video)
        tname = set_extension(self.__outname, 'temp')

        ui_command = UICommand()
        ui_command.set_command(self.__update_video)
        ui_command.set_data([video, self.get_format(), self.__outname, tname, self.get_fps()])
        return ui_command

    def prepare_exec(self, command_data, command):
        assert(len(command_data) == 3)
        video, cpass, pcount = command_data

        # build_outname() sets the current stream to video and _should_ be the first call here
        self.__outname = self.build_outname(video)
        self.__pass_count = pcount
        self.__cur_pass = cpass
        if self.__cur_pass > 2:
            self.__tname = set_extension(self.__outname, 'temp')
        else:
            self.__tname = ''
        return True

    def get_outname(self):
        return self.__outname

    def construct(self, start_ch, end_ch, outname):
        constr = self.get_constructor()
        if not constr:
            debug('WorkerVideo.construct()', 'cannot create constructor', error)
            return None

        cfg = self.get_context().get_config()

        source_type = self.get_src_cont_type()
        constr.set_source_type(source_type)
        if source_type == CONTAINER_DVD:
            constr.set_source(self.get_src_cont().get_device(), self.get_src_title().get_id())
            constr.set_chapters(start_ch, end_ch)

        elif source_type == CONTAINER_FILE:
            constr.set_source(self.get_src_cont().get_device(), self.get_src_title().get_id())

        else:
            debug('WorkerVideo.construct()', 'unknown source container type', error)
            return None

        constr.set_test_rip(cfg.get_test_rip(), self.get_src_title().get_length())

        # 1, 2 pass
        if self.__pass_count > 0 and self.__cur_pass > 0 and self.__cur_pass < 3:
            constr.set_pass_count(self.__pass_count)
            constr.set_pass_nr(self.__cur_pass)

            aspect, resol = self.__get_aspect_resolution()
            crop = self.get_dst_title().get_crop()
            aspect, scale_expand = self.compute_scale_expand(self.get_stream(), crop, aspect, resol)

            constr.set_scale_expand(scale_expand)
            constr.set_aspect(aspect)
            constr.set_vbitrate(self.get_stream().get_bitrate())

        # filter video
        if self.__cur_pass > 2:
            assert(len(self.__tname) > 0)
            self.log_append('# filter the video stream')
            self.log_append('mv %s %s\n' % (outname, self.__tname))
            if not move(outname, self.__tname):
                self.set_error(_('Cannot move %(video_file)s to %(temp_file)s') % {'video_file':outname, 'temp_file':self.__tname})
                return None
            constr.set_source(self.__tname)
            constr.set_source_type(CONTAINER_FILE)

        constr.set_outname(outname)

        success, shell_cmd = constr.construct()
        if not success:
            debug('WorkerVideo.construct()', 'command construction failed', error)
            self.set_error(_('Command construction failed'))
            return None

        return shell_cmd

    def extract(self, shell_cmd, command):
        return command.run_command(shell_cmd)

    def update_failure(self):
        debug('WorkerVideo.update_failure()', 'execution failed', error)
        return

    def update_success(self):
        return

    # -- common functions for the derived classes --

    def get_constructor(self):
        # should be implemented by derived classes
        assert(False)
        return None

    def create_outname(self, cid, tid):
        # should be implemented by derived classes
        assert(False)
        return ''

    def get_format(self):
        # should be implemented by derived classes
        assert(False)
        return VIDEO_UNKNOWN

    def get_fps(self):
        # should be implemented by derived classes
        assert(False)
        return -1.

    def can_progress(self):
        return True

    def can_estimate(self):
        return True

    def get_ui_name(self):
        return _('Extract video')

    def get_cur_pass(self):
        return self.__cur_pass

    def build_outname(self, video):
        self.use_stream(video)

        cid = self.get_dst_cid()
        tid = self.get_dst_tid()

        return self.create_outname(cid, tid)

    # -- helpers --

    def __update_video(self, command_data, command):
        assert(len(command_data) == 5)
        video, vtype, fname, tname, fps = command_data

        video.set_filename(fname)
        video.set_type(vtype)
        video.set_id(1) # todo

        if fps > 1e-3:
            video.set_fps(fps)

        if self.__cur_pass > 2:
            assert(len(tname) > 0)
            self.unlink_temp([tname, 'divx2pass.log'])
        return True

    def __get_aspect_resolution(self):
        aspect = '4/3'
        resol = '720x576'
        cfg = self.get_context().get_config()

        if self.get_dst_cont_type() == CONTAINER_DVD:
            aspect = cfg.get_video_aspect_dvd()
            resol = cfg.get_video_resolution_dvd()

        elif self.get_dst_cont_type() == CONTAINER_FILE:
            aspect = cfg.get_video_aspect_file()
            resol = cfg.get_video_resolution_file()

        return aspect, resol


class WorkerTcextract(WorkerVideo):
    def __init__(self, context, source_id):
        WorkerVideo.__init__(self, context, source_id, -1)
        return

    # -- methods inherited from worker_video.WorkerVideo --

    def get_constructor(self):
        tool_checker = self.get_context().get_tool_checker()
        return ConstructorTcextract(tool_checker)

    def create_outname(self, cid, tid):
        return self.create_stream_name(cid + 1, tid + 1, 'video', 'mpeg')

    def get_format(self):
        return VIDEO_MPEG2

    def can_progress(self):
        return False

    def can_estimate(self):
        return False

    def get_fps(self):
        return -1.


class WorkerTcrequant(WorkerVideo):
    def __init__(self, context, source_id):
        WorkerVideo.__init__(self, context, source_id, -1)
        self.__outname = ''
        self.__tempfiles = []
        return

    # -- methods inherited from worker_stream.WorkerStream --

    def get_outname(self):
        return self.__outname

    def create_post_command(self, video, video_idx):
        ui_command = UICommand()
        ui_command.set_command(self.__unlink_temp)
        ui_command.set_data([video])
        return ui_command

    def prepare_exec(self, command_data, command):
        assert(len(command_data) == 3)
        video, cpass, pcount = command_data

        self.use_stream(video)

        self.__outname = video.get_filename()
        return True

    def construct(self, start_ch, end_ch, outname):
        tool_checker = self.get_context().get_tool_checker()
        cfg = self.get_context().get_config()

        factor = self.get_stream().get_quant_factor()
        tname = set_extension(outname, 'temp')

        self.log_append('mv %s %s\n' % (outname, tname))
        if not move(outname, tname):
            err_text = _('Cannot move %(video_file)s to %(temp_file)s') % {'video_file':outname, 'temp_file':tname}
            debug('WorkerTcrequant.construct()', 'cannot move %s to %s' % (outname, tname), error)
            self.set_error(err_text)
            return None
        else:
            self.__tempfiles.append(tname)

        constr = ConstructorTcrequant(tool_checker)
        constr.set_quant_factor(factor)
        constr.set_outname(outname)
        constr.set_inname(tname)

        constr.set_test_rip(cfg.get_test_rip(), self.get_src_title().get_length())

        success, shell_cmd = constr.construct()
        if not success:
            debug('WorkerTcrequant.construct()', 'command construction failed', error)
            self.set_error(_('Command construction failed'))
            return None

        return shell_cmd

    def extract(self, shell_cmd, command):
        return command.run_command(shell_cmd)


    # -- methods inherited from worker_video.WorkerVideo --

    def get_ui_name(self):
        return _('Shrink video')

    def create_outname(self, cid, tid):
        return self.create_stream_name(cid + 1, tid + 1, 'video', 'mpeg')

    def get_format(self):
        return VIDEO_MPEG2

    def can_progress(self):
        return False

    def can_estimate(self):
        return False

    def get_fps(self):
        return -1.

    # -- helpers --

    def __unlink_temp(self, command_data, command):
        if len(self.__tempfiles) < 1:
            return True

        fnames = ''
        for fname in self.__tempfiles:
            fnames += (' %s' % fname)
        self.log_append('rm %s' % fnames)

        self.log_append('\n')
        self.unlink_temp(self.__tempfiles)
        return True


class WorkerFFmpeg4(WorkerVideo):
    def __init__(self, context, source_id, shrink_method):
        WorkerVideo.__init__(self, context, source_id, shrink_method)
        return

    # -- methods inherited from worker_video.WorkerVideo --

    def get_constructor(self):
        tool_checker = self.get_context().get_tool_checker()
        return ConstructorFFmpeg4(tool_checker)

    def create_outname(self, cid, tid):
        return self.create_stream_name(cid + 1, tid + 1, 'video', 'avi')

    def get_format(self):
        return VIDEO_MPEG4

    def get_fps(self):
        return -1.


class WorkerXvid(WorkerVideo):
    def __init__(self, context, source_id, shrink_method):
        WorkerVideo.__init__(self, context, source_id, shrink_method)
        return

    # -- methods inherited from worker_video.WorkerVideo --

    def get_constructor(self):
        tool_checker = self.get_context().get_tool_checker()
        constr = ConstructorXvid(tool_checker)
        #constr.set_audio_encoding('copy')
        return constr

    def create_outname(self, cid, tid):
        return self.create_stream_name(cid + 1, tid + 1, 'video', 'avi')

    def get_format(self):
        return VIDEO_XVID

    def get_fps(self):
        return -1.


class WorkerH264(WorkerVideo):
    def __init__(self, context, source_id, shrink_method):
        WorkerVideo.__init__(self, context, source_id, shrink_method)
        return

    # -- methods inherited from worker_video.WorkerVideo --

    def get_constructor(self):
        tool_checker = self.get_context().get_tool_checker()
        return ConstructorH264(tool_checker)

    def create_outname(self, cid, tid):
        return self.create_stream_name(cid + 1, tid + 1, 'video', 'h264')

    def get_format(self):
        return VIDEO_H264

    def get_fps(self):
        return -1.


class WorkerMpeg2DVD(WorkerVideo):
    def __init__(self, context, source_id, shrink_method):
        WorkerVideo.__init__(self, context, source_id, shrink_method)
        return

    # -- methods inherited from worker_video.WorkerVideo --

    def get_constructor(self):
        tool_checker = self.get_context().get_tool_checker()

        # filter video
        if self.get_cur_pass() > 2:
            constr = ConstructorRawvideo(tool_checker)
        else:
            constr = ConstructorMpeg2DVD(tool_checker)
        return constr

    def create_outname(self, cid, tid):
        return self.create_stream_name(cid + 1, tid + 1, 'video', 'mpeg')

    def get_format(self):
        return VIDEO_MPEG2

    def get_fps(self):
        return 25.


class WorkerMobile(WorkerVideo):
    def __init__(self, context, source_id, shrink_method):
        WorkerVideo.__init__(self, context, source_id, shrink_method)
        return

    # -- methods inherited from worker_video.WorkerVideo --

    def get_constructor(self):
        tool_checker = self.get_context().get_tool_checker()
        return ConstructorMobile(tool_checker)

    def create_outname(self, cid, tid):
        return self.create_stream_name(cid + 1, tid + 1, 'video', 'avi')

    def get_format(self):
        return VIDEO_MPEG4

    def get_fps(self):
        return 12.


# vim:number:autoindent:shiftwidth=4:tabstop=4:softtabstop=4:expandtab
